//
//	Libra TAPI Service Provider Device Specific Extensions
//
//	Copyright (C) 1998 Dialogic Corporation
//	All rights reserved.
//
//	Module Name:
//
//		LibraSpec.h
//
//	Notes:
//
//		Device Specific Extensions
//		--------------------------
//
//		Device specific extensions are defined for dlglibra.tsp.
//		These commands are accessed via the TAPI function:
//
//			lineDevSpecific( hLine, dwAddressID, hCall, lpParams, dwSize )
//
//			where:
//					hLine - handle to line for which command applies;
//
//					dwAddressID - not used;
//
//					hCall - not used;
//
//					lpParams - LIBRAREQUESTS block with command struct appended;
//
//					dwSize - size of lpParams block.
//
// Nov 20, 1998	M.Fortenberry		added LI_TransferResult_enum	enTransferResult field
//												to LibraCallInfo Structure for BlindTransfer status
//												See line.cpp Nov 20, 1998 entry for explanation.
// Feb 2, 1999    O.Omitowoju       added LIBRAREJECTREASON, LI_TransferResult_enum, 
//                                  LI_LineRejectReason_enum
/////////////////////////////////////////////////////////////////////////////
//

#ifndef __LIBRASPEC_H__
#define __LIBRASPEC_H__

//#include <devspec.h>

#pragma pack(1)


#define LIBRA_EXTENSIONID0 0xDC478E63L
#define LIBRA_EXTENSIONID1 0x101A7A27L
#define LIBRA_EXTENSIONID2 0x2B6A8E88L
#define LIBRA_EXTENSIONID3 0x0E007866L


// ----------------------------------------------------------------------------
// Device Specific Extensions
// ----------------------------------------------------------------------------

//
// Device specific command codes
//
#define DEVSPEC_LIBRA	0x00030000
#define PHONE_DISPLAY	(DEVSPEC_LIBRA + 1)		// Send display message to caller's phone.
#define SEND_ALARM		(DEVSPEC_LIBRA + 2)		// Send alarm notification to PBX.
#define SEND_MWI		(DEVSPEC_LIBRA + 3)		// Enable message waiting indicator for user on PBX.
#define SET_PIN			(DEVSPEC_LIBRA + 4)		// Set pin and calling no. for transfer command.


//
// Display command (PHONE_DISPLAY) structure.
// Used to send a display message to the phone of the caller at the
// other end of the line.
// This struct is appended to the LIBRADEVSPEC header at pbData.
//
typedef struct _M3DisplayReqStruct
{
	char szDisplayString1[25];
	char szDisplayString2[25];
} M3DISPLAYREQ, *PM3DISPLAYREQ;


//
// Alarm command (SEND_ALARM) structure.
// Used to send an alarm notification to the PBX.
// This struct is appended to the LIBRADEVSPEC header at pbData.
//
typedef struct _M3AlarmReqStruct
{
	char szAlarmString[129];
} M3ALARMREQ, *PM3ALARMREQ;


//
// Send message waiting command (SC_LISTEN) structure.
// Used to enable message waiting indicators for a user on the PBX.
// This struct is appended to the LIBRADEVSPEC header at pbData.
//
typedef struct _M3MWIReqStruct
{
	BYTE		cMessageCount;
	BOOL		bOldMessage;
	DWORD		bUrgentMessage;
	char		szCallingName[25];
	char		szMessageWaitCBNumber[33]; 
	char		szCalledNumber[33];
} M3MWIREQ, *PM3MWIREQ;


//
// Set PIN command (SET_PIN) structure.
// Used to set the PIN and calling number for a transfer operation.
// This struct is appended to the LIBRADEVSPEC header at pbData.
//
typedef struct _librapin
{
	char szCallingNumber[33];
	char szPIN[33];
} LIBRAPIN, *PLIBRAPIN;


// Taken from section 6.3.14 of "Libra TSP SP Specification"
//
// LIBRAREJECTREASON struct
//
typedef struct _librarejectreason
{
	WORD	wEscapeChar;		// Set to 0
	INT		nRejectReason;
} LIBRAREJECTREASON;

//
// ----------------------------------------------------
// Device Specific data appended to LINECALLINFO struct
// ----------------------------------------------------
//

#ifndef LIBRAAPITSPMISC_H

//
// Libra Call Bearer Types
//
typedef enum _LI_CallBearerType_enum
{
   LI_UNKNOWNCALL = 0,
   LI_VOICECALL,
   LI_FAXCALL
} LI_CallBearerType_enum;

//
// Libra Forward Reasons
//
typedef enum _LI_ForwardReason_enum
{
   LI_FWD_UNKNOWN = 0,     // Unknown value passed to host    
   LI_FWD_DIRCALL,         // Direct call into PhoneMail    
   LI_FWD_BUSY,            // Busy forwarded into PM        
   LI_FWD_RNA,             // Ring-no-answer forwarded onto PM
   LI_FWD_DND,             // Do-Not-Disturb forwarded into PM 
   LI_FWD_UNCONDITIONAL    // Station Forwarded
} LI_ForwardReason_enum;

//
// Libra Station Display Capabilities
//
typedef enum _LI_DisplayCapability_enum
{
   LI_UNKNOWNDISPLAY = 0,    // Unknown value passed to host   
   LI_NODISPLAY,             // No display supported   
   LI_2x24DISPLAY            // 2 line by 24 character display supported  
} LI_DisplayCapability_enum;


//
// Libra Access Types
//
typedef enum _LI_AccessType_enum
{
   LI_UNKNOWN_ACCESS = 0, // Illegal value passed to host 
   LI_NUMBER_ACCESS,      // Access directly to PM 
   LI_INTERCEPT_ACCESS,   // Switch intercepted call to PM
   LI_CALLBACK_ACCESS     // User hit Callback function to get PM
} LI_AccessType_enum;


//
// Libra Transfer Results
//
typedef enum _LI_TransferResult_enum
{
   LI_XFER_UNKNOWN = 0,
   LI_XFER_BUSY,
   LI_XFER_REJECTED,
   LI_XFER_CONNECTED,
   LI_XFER_RNA_TMOUT,
} LI_TransferResult_enum;

//
// LI_LineRejectReason_enum
//
typedef enum _LI_LineRejectReason_enum
{
   LI_LINE_REJECT_NOT = 0 ,      
   LI_LINE_REJECT_PM_DOWN,  
   LI_LINE_REJECT_NO_CHANNEL,  
   LI_LINE_REJECT_INVALID,      
   LI_LINE_REJECT_DATA_BAD        
} LI_LineRejectReason_enum;

#endif LIBRAAPITSPMISC_H

//
// Call information sent from PBX
//
typedef struct _Libra_Call_Infotag
{
	char					pinID[33];          // PIN ID sent by PBX
	char 					szCallingNumber[33];// calling number sent by PBX
	BOOL					bInternalCall;      // Internal call or not.
	BOOL					bConsultCall;       // Consultation call or not
	LI_ForwardReason_enum	enForwardReason;	// Forward reason per Libra spec
	BOOL					bAllowTransferOut;  // Allow transfer out
	BOOL					bAllowCampon;       // Caller allowed to do Camp-On
	LI_DisplayCapability_enum enDisplayCap;		// See NewPartyInfo
	LI_CallBearerType_enum	enBearerType;		// See NewPartyInfo
	BOOL 					bDtmfDevice;        // digits mode
	LI_AccessType_enum		enAccessType;       // directly from NewPartyInfo
	LI_TransferResult_enum	enTransferResult;	// Transfer result of lineBlindTransfer
} LIBRA_CALL_INFO, *PLIBRA_CALL_INFO;

#pragma pack()

#endif //!__LIBRASPEC_H__
