//
//  Copyright (c) 1999-2007 by AudioCodes USA, Inc.
//  All rights reserved.
//
//  NTiData.h
//      SmartWORKS API data structure declaration file
//
//  AUDIOCODES USA, INC. PROPRIETARY.
//

#ifndef _INC_NTIDATA_H
#define _INC_NTIDATA_H

#if _WIN32
#include <pshpack1.h>
#endif


/*
 * General data types
 */

typedef SHORT           BOARD;              /* Board number             */
typedef PSHORT          PBOARD;             /* Pointer to board number  */
typedef SHORT           CHANNEL;            /* Logical channel number   */
typedef PSHORT          PCHANNEL;           /* Pointer to channel number*/
typedef HANDLE          HEVTQ;              /* Event queue              */
typedef ULONG           HPROMPT;            /* Prompt handle            */
typedef USHORT          PHYSICALLINE;       /* Physical line number     */
typedef ULONG           HGROUP;             /* Group identifier         */

//
// I/O data transfer structure
//
typedef ULONG MT_DIGIT_ENABLE;

static const MT_DIGIT_ENABLE
    MT_DIGIT_C                              = 0x8000,
    MT_DIGIT_B                              = 0x4000,
    MT_DIGIT_A                              = 0x2000,
    MT_DIGIT_POUND                          = 0x1000,
    MT_DIGIT_CROSSHATCH                     = 0x1000,
    MT_DIGIT_STAR                           = 0x0800,
    MT_DIGIT_ASTERISK                       = 0x0800,
    MT_DIGIT_0                              = 0x0400,
    MT_DIGIT_9                              = 0x0200,
    MT_DIGIT_8                              = 0x0100,
    MT_DIGIT_7                              = 0x0080,
    MT_DIGIT_6                              = 0x0040,
    MT_DIGIT_5                              = 0x0020,
    MT_DIGIT_4                              = 0x0010,
    MT_DIGIT_3                              = 0x0008,
    MT_DIGIT_2                              = 0x0004,
    MT_DIGIT_1                              = 0x0002,
    MT_DIGIT_D                              = 0x0001;


typedef struct _MT_IO_CONTROL {
    //
    // general data
    //
    ULONG           PreACTDuration;         // ms of data to buffer before START_ACTD triggers
    ULONG           CtrlData;               // # of activity events to trigger recording
                                            // for START_ACTD control
    //
    // general termination
    //
    ULONG           MaxTime;                // Action timeout period in milliseconds
    ULONG           MaxBytes;               // Action terminate on size of bytes
    //
    // signal termination
    //
    ULONG           MaxSilence;             // Max. silence time for record termination
    ULONG           MaxActivity;            // Max. activity time for record termination

    ULONG           CpmMode;                // Term/Moni mode: MONI_USE_TASK_ONLY, or
                                            //      TERM_USE_CHANNEL_AND_TASK
    ULONG           TermEnable;             // CPM termination enabled by
                                            // MONI_USE_TASK_ONLY or MONI_USE_CHANNEL_AND_TASK

    ULONG           MoniEnable;             // CPM event monitor enabled by
                                            // MONI_USE_TASK_ONLY or MONI_USE_CHANNEL_AND_TASK
    //
    // digit termination
    //
    ULONG           MaxDigits;              // # of digits for termination
    MT_DIGIT_ENABLE DigitEnable;            // Enable bits for termination digits
    ULONG           EndDigit;               // Term digit (@ = any digit)
    ULONG           DigitTime;              // Inter-digit timeout when MaxDigit is not zero
    //
    // miscellaneous control data
    //
    ULONG           StartControl;           // Start Control Flag Field
    ULONG           ToneDuration;           // Prompt-tone duration in milliseconds
    ULONG           TermControl;            // Flush queued playback(s) on termination
                                            // other than EVT_EOF, EVT_MAXBYTE, EVT_MAXTIME
    //
    // playfile() & playbuffer() specific data
    //
    ULONG           FileIndex;              // File starting offset for play/record
    ULONG           FileHandle;             // File handle; Device handle for user specified media access
    ULONG           FileType;               // Type codec to use for recording, eg: MT_PCM_uLaw
    //
    // Data portion
    //
    ULONG           Length;                 // Length of the following data buffer
    PUCHAR          Buffer;                 // Data buffer

} MT_IO_CONTROL, *PMT_IO_CONTROL;

#define MtIoBuffer(IoControl)  (PUCHAR)((PUCHAR)IoControl + sizeof(MT_IO_CONTROL))

//
// Option for MT_IO_CONTROL.StartControl field for background functions
//
#define PROMPT_TONE             0x0002      // Play a prompt tone at the begining of recording

#define INTER_DIGIT_DELAY       0x0004      // Set to delay inter-digit time checking till
                                            // reception of the 1st digit, otherwise, timing
                                            // starts at the beginning of the function

#define REPEAT_TONE             0x0008      // Repeat the playing of tone(s)
#define START_ACTD              0x0010      // Start recording on detection of activity
#define START_LOOPON            0x0020      // Start recording on detection of loop current
#define NOT_TO_TOUCH            0x0040      // Obsoleted NOAGC, reserved this for a while
#define ALERTTONE_ON            0x0080      // Plays an alert tone during call logging

#define SILENCE_TRUNCATION      0x0100      // Truncate MaxSilence on termination
#define SILENCE_SUPPRESSION     0x0200      // Suppress Silence during recording, TDB
#define START_LVOLTAGE_MIDDLE   0x0400      // Start recording on detection of loop voltage MIDDLE state
#define START_LVOLTAGE_OFFHOOK  START_LVOLTAGE_MIDDLE
                                            // Start recording on detection of loop voltage
                                            // offhook and stop recording on loop voltage changed
                                            // to notoffhook; available on SmartTAP PT16 channel only

#define GSM_FRAME34             0x1000      // Be NVDSP GSM 34-byte frame format compatible,
                                            // Valid only when VF_GSM610 format specified
#define WAVE_RIFFFMTDATA_CHUNK  0x2000      // Prefix data with descriptors of RIFF/Fmt/data chunks,
                                            // not available on streaming and DeviceIO functions
#define WAVE_DATACHUNK_ONLY     0x4000      // Prefix data with wave data sub-chunk descriptor,
                                            // not available on streaming and DeviceIO functions
#define FRAME_HEADER            0x8000      // TBD, Add a 2-byte frame size header to each frame
//
// Definitions changed for NV co-existing
//
#ifndef START_VOICE                         // For NV compatibility
#define START_VOICE             START_ACTD
#endif
#ifndef START_TONE                          // For NV compatibility
#define START_TONE              PROMPT_TONE
#endif
#ifndef INTERDIGIT_DELAY                    // For NV compatibility
#define INTERDIGIT_DELAY        INTER_DIGIT_DELAY
#endif
#ifndef START_LOOP_ON                       // For NV compatibility
#define START_LOOP_ON           START_LOOPON
#endif
#ifndef ALERT_TONE                          // For NV compatibility
#define ALERT_TONE              ALERTTONE_ON
#endif

//
// Option for MT_IO_CONTROL.TermControl field for background functions
//
#define FLUSH_PLAY              0x0001      // Flush all pending play function at termination other
                                            // than EVT_EOF, EVT_MAXBYTE and EVT_MAXTIME


typedef struct _MTFAX_IO_CONTROL {
    ULONG       fhandle;
    ULONG       datatype;       /* data type - TIFF or RAW                      */
    ULONG       length;         /* number of page or length of data */
    ULONG       width;          /* image width for RAW format only      */
    ULONG       resln;          /* vertical resolution for RAW format only*/
    ULONG       encode;         /* encoding for RAW format only         */
    ULONG       reserved[4];
    PUCHAR      ptr;
} MTFAX_IO_CONTROL, *PMTFAX_IO_CONTROL;


#if 0
//
// cpm control
//
typedef struct _DEF_CPM_MODE {
    ULONG   Status;
    ULONG   Mode;
} DEF_CPM_MODE, *PDEF_CPM_MODE;
#endif


//
// System info structure
//
typedef struct _MTSYS_INFO {
    ULONG   Status;                 // status value
    UCHAR   NumSummations;          // # of summation functions in the system
    UCHAR   NumAudios;              // # of audio jacks in the system
    USHORT  NumBoards;              // # of boards in the system
    USHORT  NumChans;               // Number of local channels
    USHORT  StartChan;              // Starting internal mux channel
    UCHAR   StartBoard;             // Starting internal mux board, i.e. the
                                    // offer between local and global board numbers
    UCHAR   MuxBoards;              // Total number of global mux boards
    USHORT  MuxSystem;              // Type of mux system
    USHORT  MuxMVIPSlot;            // Starting MVIP slot assigned
    USHORT  MuxMVIPTotalSlot;       // Total MVIP slot assigned
    USHORT  MuxChans;               // Total number of global mux channels
    UCHAR   MuxGroups;              // Number of mux groups
    UCHAR   GCIStartingIndex;       // GCI starting index: 0 or 1
    CHAR    H100StreamSpeed;        // H.100 Data Rate for the 1st 16 streams:
                                    //    CT_2MHz, CT_4MHz, or CT_8MHz
    CHAR    MVIPStreamSpeed;        // MVIP Data Rate for the 1st 16 streams: CT_2MHz
    CHAR    CTBusSegmented;         // 0 for segmentation not allowed
    CHAR    res;                    // reserved
    CHAR    NumMVIPMasterBrd;       // # of the MVIP master board
    CHAR    NumH100MasterABrd;      // # of the H100 master A board
    CHAR    NumH100MasterBBrd;      // # of the H100 masster B board
    CHAR    NumSec8kNetrefBrd;      // # of the Sec8k/Netref clock sourcing board
    SHORT   NumDllLoaded;           // # of NTiDrv DLL currently loaded
    UCHAR   res2[4];
} MTSYS_INFO, *PMTSYS_INFO;

//
// System info structure (Old and to be obsoleted soon)
//
typedef struct _SYS_INFO {
    ULONG   Status;                 // status value
    ULONG   NumSummations;          // # of summation functions in the system
    ULONG   NumAudios;              // # of audio jacks in the system
    ULONG   NumBoards;              // # of boards in the system
    ULONG   NumChans;               // Number of local channels
    ULONG   Mode;                   // Always 1, Enhanced mode
    ULONG   StartChan;              // Starting internal mux channel
    ULONG   StartBoard;             // Starting internal mux board, i.e. the
                                    // offer between local and global board numbers
    ULONG   MuxBoards;              // Total number of global mux boards
    ULONG   MuxSystem;              // Type of mux system
    ULONG   MuxMVIPSlot;            // Starting MVIP slot assigned
    ULONG   MuxMVIPTotalSlot;       // Total MVIP slot assigned
    ULONG   MuxGroups;              // Number of mux groups
    ULONG   MuxChans;               // Total number of global mux channels
    ULONG   MuxH100Clock;           // H100 Clock: CT_2MHz, CT_4MHz, or CT_8MHz
    ULONG   MuxH100PrivateSlot;     // H100 slot for DLL private usage, the total
                                    // H100 time slot needed is 256
} SYS_INFO, *PSYS_INFO;


//
// System status codes (MTSYS_INFO.Status)
//
#define SYSTEM_INITIALIZED          1
#define SYSTEM_STARTED              2
#define SYSTEM_SHUTDOWN             3

//
// MUX type codes (MTSYS_INFO.MuxSystem)
//
#define MUX_NONE                    1       // Board with NO MUX found
#define MUX_H100                    2       // Board with H100 found
#define MUX_MVIP                    4       // Board with MVIP found

//
// H100 clock codes (MTSYS_INFO.H100StreamSpeed)
//
#define CT_2MHz                     0       // CT stream speed of 2 MHz
#define CT_4MHz                     1       // CT stream speed of 4 MHz
#define CT_8MHz                     2       // CT stream speed of 8 MHz

//
// Version definition
//
typedef struct _VERSION
{
    UCHAR   Major;
    UCHAR   Minor;
    USHORT  Internal;
    USHORT  Build;
    UCHAR   Release;
    UCHAR   Feature;
} MT_VERSION, * PMT_VERSION;


//
// Board info structure
//
typedef struct _MTADAPTER_INFO {
    ULONG   Status;                 // Status value of BOARD_OK, etc.
    ULONG   BoardNumber;            // Board index in SDK, i.e. SystemIndex
    MT_VERSION   FWVersion;         // On board firmware version
    ULONG   BaseAddress;            // I/O base address
    ULONG   Interrupt;              // Interrupt number
    ULONG   StartChannel;           // Logical # of the 1st on board channel
    ULONG   Channels;               // Number of channels on the board
    ULONG   Option1;                // Basic Board option
                                    // option 1
                                    //  bit 0           WAV
                                    //  bit 1           MVIP
                                    //  bit 2           Volume/Pitch
                                    //  bit 3           MF
                                    //  bit 4           Silence Compression
    ULONG   Option2;                // Extended Board option
                                    // option 2
                                    //  bit 0           Caller ID
                                    //  bit 1           Conference
                                    //  bit 2           Call Logging
                                    //  bit 4           FAX
                                    //  bit 6           Pulse detection
    ULONG   Digirate;               // Digitizing rate
    ULONG   Bandwidth;              // Channel bandwidth
    ULONG   PCISlot;
    ULONG   PCIBusNumber;

    ULONG   BoardType;              // Board Type of DUAL_T1_PASSIVE_HI_CARD, etc.

    UCHAR   BusType;                // MUX_H100, MUX_MVIP, or MUX_NONE
    UCHAR   MasterMode;             // Board Master Mode of the following:
                                    //  0               slave board
                                    //  1               MVIP master, or H100 Master A
                                    //  2               H100 Master B
    UCHAR   MasterClock;            // Master clock source reference value
                                    //                  of CLOCK_SOURCE_OSC, and etc.
    UCHAR   Sec8kNetrefClock;       // MVIP Sec8K or H100 Netref clock output
                                    //  0               Disabled, output tri-stated
                                    //  1, 3 - 4        Type of clock reference of
                                    //                  CLOCK_SOURCE_OSC, etc.
    UCHAR   NumSummations;          // Number of summation resource on board, if any
    UCHAR   StartSummation;         // Index of the first on board summation resource
    UCHAR   NumAudios;              // Number of audio jack resource on board, if any
    UCHAR   StartAudio;             // Index of the first on board audio jack resource
    UCHAR   TDMSignaling;           // CT TDM bus signaling: 0 for u-law, 1 for A-law
    UCHAR   ConfigurationId;        // Adapter Configuration Index (i.e. SwitchIndex)
    UCHAR   res[2];
    MT_VERSION   FWVersionExpected; // Expected on board firmware version
} MTADAPTER_INFO, *PMTADAPTER_INFO;

//
// Board info structure (Old and to be obsoleted)
//
typedef struct _MT_BOARD_INFO {
    ULONG   Status;                 // Status value of BOARD_OK, etc.
    ULONG   BoardNumber;            // Board index in SDK, i.e. SystemIndex
    MT_VERSION   FWVersion;         // On board firmware version
    ULONG   BaseAddress;            // I/O base address
    ULONG   Interrupt;              // Interrupt number
    ULONG   StartChannel;           // Logical # of the 1st on board channel
    ULONG   Channels;               // Number of channels on the board
    ULONG   Option1;                // Basic Board option
                                    // option 1
                                    //  bit 0           WAV
                                    //  bit 1           MVIP
                                    //  bit 2           Volume/Pitch
                                    //  bit 3           MF
                                    //  bit 4           Silence Compression
    ULONG   Option2;                // Extended Board option
                                    // option 2
                                    //  bit 0           Caller ID
                                    //  bit 1           Conference
                                    //  bit 2           Call Logging
                                    //  bit 4           FAX
                                    //  bit 6           Pulse detection
    ULONG   Digrate;                // Digitizing rate
    ULONG   Bandwidth;              // Channel bandwidth
    USHORT  MasterMode;             // Board Master Mode of CLOCK_SOURCE_....
                                    //  0               slave board
                                    //  1 - 4           Master with clock reference
                                    //                  of CLOCK_SOURCE_OSC, etc.
    USHORT  Sec8kOutput;            // MVIP Sec8K output
                                    //  0               Disabled
                                    //  1, 3 - 4        Type of clock reference of
                                    //                  CLOCK_SOURCE_OSC, etc.
    ULONG   PCISlot;
    ULONG   PCIBusNumber;
    USHORT  TDMSignaling;           // TDM bus signaling: 0 for u-law, 1 for A-law
    USHORT  NumFramers;             // Number of local framers
    ULONG   TDMBoardType;           // TDM Board Type of DUAL_T1_PASSIVE_HI_CARD, etc.
} MTBOARD_INFO, *PMTBOARD_INFO;

//
// Board status codes (use with MTBOARD_INFO.Status)
//

#define BOARD_IDLE                  0   // Board is idle
#define BOARD_BLOCKED               1   // Board does not accept commands
#define BOARD_NO_RESP               2   // Board does not respond
#define BOARD_ACK                   3   // Board responds but nothing else
#define BOARD_READY_FOR_DOWNLOAD    4   // Board is ready for download
#define BOARD_OK                    5   // Board works fine
#define BOARD_CHAN_ERROR            6   // Board generates bad channel nrs
#define BOARD_HWERROR               7   // HW failure reported during reset
#define BOARD_RESOURCE_ERROR        8   // Not enough system resource error
#define BOARD_UNSUPPORTED           9   // Unsupported board detected
#define BOARD_PANIC                 10  // Board FW error detected
#define BOARD_INCOMPATIBLE_FW       11  // Driver incompatible FW detected
#define BOARD_DLLINCOMPATIBLE_FW    12  // DLL incompatible FW detected
#define BOARD_DLLINCOMPATIBLE_DRV   13  // DLL incompatible Driver detected

#define BOARD_DOWNLOAD_GRANTED      20  // Board download right granted
#define BOARD_DOWNLOAD_STARTED      21  // Board download process started
#define BOARD_DOWNLOAD_COMPLETED    22  // Board download process completed


//
// Board clock reference codes (MTBOARD_INFO.MasterClockSource & MTBOARD_INFO.Sec8kOutput)
//
#define CLOCK_SOURCE_DISABLED       0x00
#define CLOCK_SOURCE_OSC            0x01
#define CLOCK_SOURCE_SEC8K          0x02    // MVIP only
#define CLOCK_SOURCE_CT_NETREF      CLOCK_SOURCE_SEC8K  // H100 only
#define CLOCK_SOURCE_CT_H100        0x03    // H100 only
#define CLOCK_SOURCE_NET1           0x04
#define CLOCK_SOURCE_NET2           0x05
#define CLOCK_SOURCE_NET3           0x06

//
// Board Master mode (MTBOARD_INFO.MasterMode)
//
#define MODE_SLAVE                  0x00
#define MODE_MASTER                 0x01    // MVIP only
#define MODE_MASTER_A               MODE_MASTER // H100 only
#define MODE_MASTER_B               0x02    // H100 only

//
// CT H100 slave clock selection
//
#define MODE_CLOCK_A                0
#define MODE_CLOCK_B                1

//
//
// Board type options
//
#define DSP_5409                        0x80000000L
#define PCIe                            0x40000000L

#define VOICE_RESOURCE_CARD             0x0010L
    #define VR_64                                   0x0000L
    #define VR_32                                   0x0001L

#define SINGLE_T1E1_TERMINATE_CARD      0x0020L
#define SINGLE_T1E1_TERMINATE_CARD_E    (SINGLE_T1E1_TERMINATE_CARD|DSP_5409)
#define SINGLE_TE_TERMINATE_CARD        0x0028L
#define SINGLE_TE_TERMINATE_CARD_E      (SINGLE_TE_TERMINATE_CARD|DSP_5409)
#define DUAL_T1E1_TERMINATE_CARD        0x0040L
#define DUAL_T1E1_TERMINATE_CARD_E      (DUAL_T1E1_TERMINATE_CARD|DSP_5409)
#define DUAL_TE_TERMINATE_CARD          0x0048L
#define DUAL_TE_TERMINATE_CARD_E        (DUAL_TE_TERMINATE_CARD|DSP_5409)
#define QUAD_T1E1_TERMINATE_CARD        0x0080L
    #define T1_OPTION                               0x0000L
    #define E1_OPTION                               0x0001L
    #define PCM32_OPTION                            0x0002L

#define TERMINATE_CARDS_GROUP           0x00e0L

#define SINGLE_T1E1_PASSIVE_HI_CARD     0x0100L     // High Impedence card
#define SINGLE_T1E1_PASSIVE_HI_CARD_E   (SINGLE_T1E1_PASSIVE_HI_CARD|DSP_5409)
#define DUAL_T1E1_PASSIVE_HI_CARD       0x0200L
#define DUAL_T1E1_PASSIVE_HI_CARD_E     (DUAL_T1E1_PASSIVE_HI_CARD|DSP_5409)
#define SINGLE_T1E1_PASSIVE_LI_CARD     0x0400L     // Low Impedence card
#define DUAL_T1E1_PASSIVE_LI_CARD       0x0800L

#define PASSIVE_CARDS_GROUP             0x0f00L
#define CALL_LOGGING_CARDS_GROUP        0x0f00L

#define IS_DIGITAL_TERMINATE_CARD(brdType) ( (brdType & TERMINATE_CARDS_GROUP) != 0 )

#define IS_DIGITAL_PASSIVE_TAP_CARD(brdType) ( (brdType & PASSIVE_CARDS_GROUP) != 0 )

#define ANALOG_PASSIVE_CARD             0x1000L
    #define PT16                                    0x0000L
    #define PT8                                     0x0001L
    #define PT4                                     0x0002L
#define ANALOG_TERMINATE_CARD           0x2000L
    #define AT4                                     0x0000L
    #define AT8                                     0x0001L
    #define AT16                                    0x0002L
    #define LD8                                     0x0004L
    #define LD4                                     0x0008L
    #define LD24                                    0x0010L
    #define LD16                                    0x0020L
    #define T1S1                                    0x0040L
    #define T2S1                                    0x0080L
    #define LD4H                                    0x0100L
//    #define ST16                                    0x0400L
//    #define AT4_STATION12                           0x0800L
    #define LD8X                                    0x0400L
    #define ST16                                    0x1000L
    #define AT4_STATION12                           0x2000L
#define ANALOG_CARD                     0x3000L

#define NGX_CARD                        0x4000L
#define IS_NGX_CARD(brdType)            ( ((brdType) & NGX_CARD) == NGX_CARD )

#define NET_TAP_CARD                    0x8000L
	#define IPX                                 0x0001L					
	#define IPX_C                               0x0002L
    #define IPR6400                             0x0003L
    #define IPR12800                            0x0004L
    #define IPR25600                            0x0005L
    #define HPX                                 0x0200L
//
// Board type codes (use with MTBOARD_INFO.TDMBoardType)
//
#define UNKNOWN_CARD                    0x0000L

// SmartDSP - VR card
#define VR64                            VR6400
#define VR32                            VR3200
#define VR6400                          (VOICE_RESOURCE_CARD|VR_64)
#define VR3200                          (VOICE_RESOURCE_CARD|VR_32)
#define VR6409                          (VOICE_RESOURCE_CARD|VR_64|DSP_5409)
#define VR3209                          (VOICE_RESOURCE_CARD|VR_32|DSP_5409)

// SmartTAP - Digital Network cards (T1E1 Passive Tap)
#define DUAL_T1_TERMINATE_CARD          DT6400_T1_CARD
#define DUAL_E1_TERMINATE_CARD          DT6400_E1_CARD
#if 1   // Cards not available yet
#define QUAD_T1_TERMINATE_CARD          (QUAD_T1E1_TERMINATE_CARD|T1_OPTION)
#define QUAD_E1_TERMINATE_CARD          (QUAD_T1E1_TERMINATE_CARD|E1_OPTION)
#endif

#define DT6400_T1_CARD                  (DUAL_T1E1_TERMINATE_CARD|T1_OPTION)
#define DT6400_E1_CARD                  (DUAL_T1E1_TERMINATE_CARD|E1_OPTION)
#define DT6409_T1_CARD                  (DUAL_T1E1_TERMINATE_CARD|T1_OPTION|DSP_5409)
#define DT6409_E1_CARD                  (DUAL_T1E1_TERMINATE_CARD|E1_OPTION|DSP_5409)
#define DT6409TE_T1_CARD                (DUAL_TE_TERMINATE_CARD|T1_OPTION|DSP_5409)
#define DT6409TE_E1_CARD                (DUAL_TE_TERMINATE_CARD|E1_OPTION|DSP_5409)
#define DT3200_T1_CARD                  (SINGLE_T1E1_TERMINATE_CARD|T1_OPTION)
#define DT3200_E1_CARD                  (SINGLE_T1E1_TERMINATE_CARD|E1_OPTION)
#define DT3209_T1_CARD                  (SINGLE_T1E1_TERMINATE_CARD|T1_OPTION|DSP_5409)
#define DT3209_E1_CARD                  (SINGLE_T1E1_TERMINATE_CARD|E1_OPTION|DSP_5409)
#define DT3209TE_T1_CARD                (SINGLE_TE_TERMINATE_CARD|T1_OPTION|DSP_5409)
#define DT3209TE_E1_CARD                (SINGLE_TE_TERMINATE_CARD|E1_OPTION|DSP_5409)
#define DT6409EH_T1_CARD                (DUAL_TE_TERMINATE_CARD|T1_OPTION|DSP_5409|PCIe)
#define DT6409EH_E1_CARD                (DUAL_TE_TERMINATE_CARD|E1_OPTION|DSP_5409|PCIe)
#define DT3209EH_T1_CARD                (SINGLE_TE_TERMINATE_CARD|T1_OPTION|DSP_5409|PCIe)
#define DT3209EH_E1_CARD                (SINGLE_TE_TERMINATE_CARD|E1_OPTION|DSP_5409|PCIe)

#define SINGLE_T1_PASSIVE_HI_CARD       DP3200_T1_CARD
#define SINGLE_E1_PASSIVE_HI_CARD       DP3200_E1_CARD
#define DUAL_T1_PASSIVE_HI_CARD         DP6400_T1_CARD
#define DUAL_E1_PASSIVE_HI_CARD         DP6400_E1_CARD

#define DP6400_T1_CARD                  (DUAL_T1E1_PASSIVE_HI_CARD|T1_OPTION)
#define DP6400_E1_CARD                  (DUAL_T1E1_PASSIVE_HI_CARD|E1_OPTION)
#define DP3200_T1_CARD                  (SINGLE_T1E1_PASSIVE_HI_CARD|T1_OPTION)
#define DP3200_E1_CARD                  (SINGLE_T1E1_PASSIVE_HI_CARD|E1_OPTION)
#define DP6409_T1_CARD                  (DUAL_T1E1_PASSIVE_HI_CARD|T1_OPTION|DSP_5409)
#define DP6409_E1_CARD                  (DUAL_T1E1_PASSIVE_HI_CARD|E1_OPTION|DSP_5409)
#define DP3209_T1_CARD                  (SINGLE_T1E1_PASSIVE_HI_CARD|T1_OPTION|DSP_5409)
#define DP3209_E1_CARD                  (SINGLE_T1E1_PASSIVE_HI_CARD|E1_OPTION|DSP_5409)
#define DP6409EH_T1_CARD                (DUAL_T1E1_PASSIVE_HI_CARD|T1_OPTION|DSP_5409|PCIe)
#define DP6409EH_E1_CARD                (DUAL_T1E1_PASSIVE_HI_CARD|E1_OPTION|DSP_5409|PCIe)
#define DP3209EH_T1_CARD                (SINGLE_T1E1_PASSIVE_HI_CARD|T1_OPTION|DSP_5409|PCIe)
#define DP3209EH_E1_CARD                (SINGLE_T1E1_PASSIVE_HI_CARD|E1_OPTION|DSP_5409|PCIe)

#if 1   // Cards not available yet
#define SINGLE_T1_PASSIVE_LI_CARD       (SINGLE_T1E1_PASSIVE_LI_CARD|T1_OPTION)
#define SINGLE_E1_PASSIVE_LI_CARD       (SINGLE_T1E1_PASSIVE_LI_CARD|E1_OPTION)
#define DUAL_T1_PASSIVE_LI_CARD         (DUAL_T1E1_PASSIVE_LI_CARD|T1_OPTION)
#define DUAL_E1_PASSIVE_LI_CARD         (DUAL_T1E1_PASSIVE_LI_CARD|E1_OPTION)
#endif

#define PCM6409_32T_CARD                (DUAL_T1E1_TERMINATE_CARD_E|PCM32_OPTION)
#define PCM3209_32T_CARD                (SINGLE_T1E1_TERMINATE_CARD_E|PCM32_OPTION)
#define PCM6409EH_32T_CARD              (DUAL_T1E1_TERMINATE_CARD_E|PCM32_OPTION|PCIe)
#define PCM3209EH_32T_CARD              (SINGLE_T1E1_TERMINATE_CARD_E|PCM32_OPTION|PCIe)



// SmartWORKS Analog cards
#define ANALOG_PT16                     ANALOG_PT1600
#define ANALOG_PT8                      ANALOG_PT800
#define ANALOG_T16                      ANALOG_T1600
#define ANALOG_S16                      ANALOG_S1600

#define ANALOG_PT1600                   (ANALOG_PASSIVE_CARD|PT16)
#define ANALOG_PT800                    (ANALOG_PASSIVE_CARD|PT8)
//#define ANALOG_PT400                    (ANALOG_PASSIVE_CARD|PT4)
#define ANALOG_PT1609                   (ANALOG_PASSIVE_CARD|PT16|DSP_5409)
#define ANALOG_PT809                    (ANALOG_PASSIVE_CARD|PT8|DSP_5409)
#define ANALOG_PT409                    (ANALOG_PASSIVE_CARD|PT4|DSP_5409)
#define ANALOG_T1600                    (ANALOG_TERMINATE_CARD|AT16)
//#define ANALOG_T800                     (ANALOG_TERMINATE_CARD|AT8)
//#define ANALOG_T400                     (ANALOG_TERMINATE_CARD|AT4)
#define ANALOG_T1609                    (ANALOG_TERMINATE_CARD|AT16|DSP_5409)
#define ANALOG_T809                     (ANALOG_TERMINATE_CARD|AT8|DSP_5409)
#define ANALOG_T409                     (ANALOG_TERMINATE_CARD|AT4|DSP_5409)
#define ANALOG_LD809                    (ANALOG_TERMINATE_CARD|LD8|DSP_5409)
#define ANALOG_LD809X                   (ANALOG_TERMINATE_CARD|LD8X|DSP_5409)
#define ANALOG_LD409                    (ANALOG_TERMINATE_CARD|LD4|DSP_5409)
#define ANALOG_LD2409                   (ANALOG_TERMINATE_CARD|LD24|DSP_5409)
#define ANALOG_LD1609                   (ANALOG_TERMINATE_CARD|LD16|DSP_5409)
#define ANALOG_LD409H                   (ANALOG_TERMINATE_CARD|LD4H|DSP_5409)
#define ANALOG_LD101                    (ANALOG_TERMINATE_CARD|T1S1|DSP_5409)
#define ANALOG_LD201                    (ANALOG_TERMINATE_CARD|T2S1|DSP_5409)
#define ANALOG_S1600                    (ANALOG_TERMINATE_CARD|ST16)
#define ANALOG_T4S12                    (ANALOG_TERMINATE_CARD|AT4_STATION12)
#define ANALOG_LD809EH                  (ANALOG_TERMINATE_CARD|LD8|DSP_5409|PCIe)
#define ANALOG_LD2409EH                 (ANALOG_TERMINATE_CARD|LD24|DSP_5409|PCIe)
#define ANALOG_LD1609EH                 (ANALOG_TERMINATE_CARD|LD16|DSP_5409|PCIe)

// SmartWORKS NGX cards
#define NGX_EH                          (NGX_CARD | PCIe)
#define NGX_AH                          (NGX_CARD | 0x0001L)
#define NGX_AH_1600                     (NGX_CARD | 0x010)
#define NGX_EH_1600                     (NGX_AH_1600 | PCIe)
#define NGX_AH_800                      (NGX_CARD | 0x020)
#define NGX_EH_800                      (NGX_AH_800 | PCIe)
#define NGX_AH_2400                     (NGX_CARD | 0x030)
#define NGX_EH_2400                     (NGX_AH_2400 | PCIe)

// SmartWORKS NetTAP cards
#define RATTLER                         NET_TAP_CARD
#define NETTAP_IPX                        (NET_TAP_CARD|IPX)
#define NETTAP_IPX_C                      (NET_TAP_CARD|IPX_C)
#define NETTAP_IPR                        (NET_TAP_CARD|IPR)
#define NETTAP_HPX                        (NET_TAP_CARD|HPX)

// ASCII string for SmartWORKS board type
#define strVR6400                       "SmartDSP VR6400"
#define strVR6409                       "SmartDSP VR6409"
#define strVR3200                       "SmartDSP VR3200"
#define strVR3209                       "SmartDSP VR3209"
#define strVR64                         strVR6400
#define strVR32                         strVR3200


#define strDUAL_T1E1_TERMINATE_CARD     "SmartTERM Digital Terminate Dual T1E1"
#define strDUAL_T1E1_TERMINATE_CARD_E   "SmartTERM Digital Terminate Dual T1E1 Enhanced"
#define strDT6400_T1_CARD               "SmartTERM DT6400 Dual T1"
#define strDT6409_T1_CARD               "SmartTERM DT6409 Dual T1"
#define strDT6409TE_T1_CARD             "SmartTERM DT6409TE Dual T1"
#define strDT6400_E1_CARD               "SmartTERM DT6400 Dual E1"
#define strDT6409_E1_CARD               "SmartTERM DT6409 Dual E1"
#define strDT6409TE_E1_CARD             "SmartTERM DT6409TE Dual E1"
#define strDUAL_T1_TERMINATE_CARD       "SmartTERM Digital Terminate Dual T1"
#define strDUAL_E1_TERMINATE_CARD       "SmartTERM Digital Terminate Dual E1"
#define strDT6409EH_T1_CARD             "SmartTERM DT6409 PCIe Dual T1"
#define strDT6409EH_E1_CARD             "SmartTERM DT6409 PCIe Dual E1"


#if 0   // Not supported yet
#define strQUAD_T1_TERMINATE_CARD       "QUAD_T1_TERMINATE_CARD"
#define strQUAD_E1_TERMINATE_CARD       "QUAD_E1_TERMINATE_CARD"
#endif

#define strSINGLE_T1E1_TERMINATE_CARD   "SmartTERM Digital Terminate Single T1E1"
#define strSINGLE_T1E1_TERMINATE_CARD_E "SmartTERM Digital Terminate Single T1E1 Enhanced"
#define strDT3200_T1_CARD               "SmartTERM DT3200 Single T1"
#define strDT3209_T1_CARD               "SmartTERM DT3209 Single T1"
#define strDT3209TE_T1_CARD             "SmartTERM DT3209TE Single T1"
#define strDT3200_E1_CARD               "SmartTERM DT3200 Single E1"
#define strDT3209_E1_CARD               "SmartTERM DT3209 Single E1"
#define strDT3209TE_E1_CARD             "SmartTERM DT3209TE Single E1"
#define strDT3209EH_T1_CARD             "SmartTERM DT3209 PCIe Single T1"
#define strDT3209EH_E1_CARD             "SmartTERM DT3209 PCIe Single E1"

#define strDUAL_T1E1_PASSIVE_HI_CARD    "SmartTAP Digital Passive Dual T1E1"
#define strDUAL_T1E1_PASSIVE_HI_CARD_E  "SmartTAP Digital Passive Dual T1E1 Enhanced"
#define strDP6400_T1_CARD               "SmartTAP DP6400 Dual T1"
#define strDP6409_T1_CARD               "SmartTAP DP6409 Dual T1"
#define strDP6400_E1_CARD               "SmartTAP DP6400 Dual E1"
#define strDP6409_E1_CARD               "SmartTAP DP6409 Dual E1"
#define strDUAL_T1_PASSIVE_HI_CARD      "SmartTAP Digital Passive Dual T1"
#define strDUAL_E1_PASSIVE_HI_CARD      "SmartTAP Digital Passive Dual E1"
#define strDP6409EH_T1_CARD             "SmartTAP DP6409 PCIe Dual T1"
#define strDP6409EH_E1_CARD             "SmartTAP DP6409 PCIe Dual E1"

#define strSINGLE_T1E1_PASSIVE_HI_CARD  "SmartTAP Digital Passive Single T1E1"
#define strSINGLE_T1E1_PASSIVE_HI_CARD_E "SmartTAP Digital Passive Single T1E1 Enhanced"
#define strDP3200_T1_CARD               "SmartTAP DP3200 Single T1"
#define strDP3209_T1_CARD               "SmartTAP DP3209 Single T1"
#define strDP3200_E1_CARD               "SmartTAP DP3200 Single E1"
#define strDP3209_E1_CARD               "SmartTAP DP3209 Single E1"
#define strDP3209EH_T1_CARD             "SmartTAP DP3209 PCIe Single T1"
#define strDP3209EH_E1_CARD             "SmartTAP DP3209 PCIe Single E1"

#define strSINGLE_T1_PASSIVE_LI_CARD    "SINGLE_T1_PASSIVE_LI_CARD"
#define strSINGLE_E1_PASSIVE_LI_CARD    "SINGLE_E1_PASSIVE_LI_CARD"
#define strDUAL_T1_PASSIVE_LI_CARD      "DUAL_T1_PASSIVE_LI_CARD"
#define strDUAL_E1_PASSIVE_LI_CARD      "DUAL_E1_PASSIVE_LI_CARD"


#define strPCM6409_32T_CARD             "SmartTERM PCM6409-32T"
#define strPCM3209_32T_CARD             "SmartTERM PCM3209-32T"
#define strPCM6409EH_32T_CARD           "SmartTERM PCM6409EH-32T"
#define strPCM3209EH_32T_CARD           "SmartTERM PCM3209EH-32T"


#define strANALOG_PT16                  "SmartTAP PT1600"
#define strANALOG_PT8                   "SmartTAP PT800"
#define strANALOG_PT1609                "SmartTAP PT1609"
#define strANALOG_PT809                 "SmartTAP PT809"
#define strANALOG_PT409                 "SmartTAP PT409"
#define strANALOG_T16                   "SmartTERM AT1600"
#define strANALOG_T1609                 "SmartTERM AT1609"
#define strANALOG_T809                  "SmartTERM AT809"
#define strANALOG_T409                  "SmartTERM AT409"
#define strANALOG_S16                   "ANALOG_STATION1600"
#define strANALOG_LD809                 "SmartTAP LD809"
#define strANALOG_LD809X                "SmartTAP LD809X"
#define strANALOG_LD409                 "SmartTAP LD409"
#define strANALOG_LD409H                "SmartTAP LD409H"
#define strANALOG_LD2409                "SmartTAP LD2409"
#define strANALOG_LD1609                "SmartTAP LD1609"
#define strANALOG_LD101                 "SmartTAP LD101 (T1S1)"
#define strANALOG_LD201                 "SmartTAP LD201 (T2S1)"
#define strANALOG_T4S12                 "ANALOG_AT4_STATION12"
#define strANALOG_LD809EH               "SmartTAP LD809 PCIe"
#define strANALOG_LD2409EH              "SmartTAP LD2409 PCIe"
#define strANALOG_LD1609EH              "SmartTAP LD1609 PCIe"

#define strNGX                          "SmartTAP NGX"
#define strNGX_AH                       "SmartTAP NGX A-h"
#define strNGX_EH                       "SmartTAP NGX PCIe"
#define strNGX_AH_16                    "SmartTAP NGX1600A-h"
#define strNGX_EH_16                    "SmartTAP NGX1600 PCIe"
#define strNGX_AH_8                     "SmartTAP NGX800A-h"
#define strNGX_EH_8                     "SmartTAP NGX800 PCIe"
#define strNGX_AH_24                    "SmartTAP NGX2400A-h"
#define strNGX_EH_24                    "SmartTAP NGX2400 PCIe"

#define strRATTLER                      "Rattler"
#define strNETTAP_IPX                   "NetTAP IPX"
#define strNETTAP_IPX_C                 "NetTAP IPX C"
#define strNETTAP_IPR                   "NetTAP IPR"
#define strNETTAP_HPX                   "HPX"

#define MAX_T1_CHANNEL                  24
#define MAX_E1_CHANNEL                  31
#define MAX_DT_E1_CHANNEL               30

//
// Board Serial Info
//
typedef struct {
    CHAR    AssemblyNumber[16];
    CHAR    HWRevision[4];
    CHAR    SerialNumber[8];
    CHAR    AssemblyCode[4];
    CHAR    DateCode[8];
} SERIAL_INFO, *PSERIAL_INFO;

//
// Board OEM Info
//
#define OEM_INFO_LEN            64
typedef struct _MT_OEM_INFO {
    CHAR    OEMInfo[OEM_INFO_LEN];      // OEM info
    CHAR    Copyright[OEM_INFO_LEN];    // Copyright info
} MTOEM_INFO, *PMTOEM_INFO;


//
// Xtra Board info structure (additional info not in MTBOARD_INFO)
//
typedef struct _MTADAPTER_XINFO {
    ULONG   BoardNumber;            // Board index in SDK, i.e. SystemIndex:
                                    //  0 for the 1st board
    USHORT  NumFramers;             // Number of local framers: 0 - 4
    UCHAR   FramerSignaling[4];     // Signaling for the max. of 4 framers
    UCHAR   FramerProtocol[4];      // Protocol variant for the max. of 4 framers

    USHORT  NumTrunks;              // Number of trunks on board: 0 - 2
    UCHAR   TrunkChannelCount[2];   // Number of channels on each trunk
    UCHAR   TrunkSignaling[2];      // Signaling for the max. of 2 trunks
    UCHAR   TrunkProtocol[2];       // Protocol variant for the max. of 2 trunks
    UCHAR   TrunkInterfaceSide[2];  // Terminal/Network side support for 2 trunks
    UCHAR   TrunkNFASId[2];         // Signaling NFAS Group Index
} MTADAPTER_XINFO, *PMTADAPTER_XINFO, MTBOARD_XINFO, *PMTBOARD_XINFO;


// Board framer signaling values for field MTBOARD_XINFO.FramerSignaling[]
#define SIGNALING_NONE          0       // No signaling, default
#define SIGNALING_ISDN          1       // ISDN for both T1 and E1
#define SIGNALING_RBS           2       // RBS for T1
#define SIGNALING_CAS           2       // CAS for E1
#define SIGNALING_DASS2         4       // DASS2
#define SIGNALING_DPNSS         5       // DPNSS
#define SIGNALING_MFR2          6       // MFR2 (E1, DP)
#define SIGNALING_MULTIFRAME    7       // E1 CRC4 Multiframe (CAS signaling not implemented)
#define SIGNALING_PCM_31        8       // E1 timeslot 16 used as 16th voice channel, 31 voice channels

#define SIGNALING_NFAS          0x80    // NFAS for both T1 (and E1 in the future)
#define SIGNALING_NFAS_DCH          0x40    // Extention for NFAS definition
                                            // - bit 0x40 is for D-cannel enable
#define SIGNALING_NFAS_BK_DCH       0x20    // Extention for NFAS definition
                                            // - bit 0x20 is for backup D-cannel enable
#define SIGNALING_NFAS_TRUNK_BITS   0x0F    // - the lower nibble is for unique
                                            //   trunk number within the NFAS indexed
                                            //   TrunkNFASId


// Board framer protocol values for field MTBOARD_XINFO.TrunkProtocol[]
//  RBS
#define PROTOCOL_RBS_IMMEDIATE      0
#define PROTOCOL_RBS_WINK           1       // Default
#define PROTOCOL_RBS_LOOP_FXS       2
#define PROTOCOL_RBS_LOOP_FXO       3
//  ISDN Terminate
#define PROTOCOL_ISDN_ETS300        0       // Default
#define PROTOCOL_ISDN_NATIONAL2     1
#define PROTOCOL_ISDN_ATT5ESS       2
#define PROTOCOL_ISDN_DMS100        3
#define PROTOCOL_ISDN_NTT           4
#define PROTOCOL_ISDN_AUSTELL1      5
#define PROTOCOL_ISDN_QSIG          6
//  MFR2
#define PROTOCOL_MFR2_CHINA         0       // Default
#define PROTOCOL_MFR2_CHINA_EXP     1
#define PROTOCOL_MFR2_BRAZIL        2

// Board framer support values for field MTBOARD_XINFO.TrunkSupport[]
// ISDN Terminate
#define SUPPORT_TE                  0       // Terminal side support
#define SUPPORT_NT                  1       // Network side support


typedef struct _TS_ASSOC
{
    SHORT   Framer;                 // -1 for no association
    SHORT   Timeslot;               // -1 for no association
} MTCHAN_TS;

//
// Channel information
//
typedef struct _MT_CHAN_INFO {
    ULONG       Type;               // Channel type of VOICE_RESOURCE and etc.
    ULONG       Variant;            // Channel variant such as protocol, NGX phone type, etc.
    ULONG       LoadStatus;         // Channel loading status from driver
    ULONG       nAssociatedTS;      // Number of timeslots associated with this channel
    MTCHAN_TS   AssociatedTS[2];    // Associated timeslot information
    CHAR        BusType;            // CT bus of MUX_MVIP, MUX_H100, or MUX_NONE
    CHAR        StreamSpeed;        // CT bus data rate for the 1st 16 streams
    CHAR        Res0;               // CT bus signaling: 0 for u-law, 1 for A-law
    CHAR        Res1;               // Reserved
} MTCHAN_INFO, *PMTCHAN_INFO;

// Channel type for MTCHAN_INFO.Type field
#define VOICE_RESOURCE              0x00000001  // For channels on VR6400, VR3200
#define VOICE_RESOURCE_E            (VOICE_RESOURCE|DSP_5409)
                                                // For channels on VR6409, VR3209

#define ANALOG_VDS                  0x00000002  // For channels on PT1600, PT800
#define ANALOG_VDS_E                (ANALOG_VDS|DSP_5409)
                                                // For channels on PT1609, PT809

#define ANALOG_FXO_LS               0x00000006  // For channels on AT1600, AT800
#define ANALOG_FXO_LS_E             (ANALOG_FXO_LS|DSP_5409)
                                                // For channels on AT1609, AT809

#define ANALOG_FXS_LS_E             (0x00000007|DSP_5409)
                                                // For station channels on AT4X12

#define ANALOG_VFXO_LS_E            (0x00000008|DSP_5409)
                                                // For channels on LD809, LD409, LD2409, LD1609

#define ANALOG_VFXS_LS_E            (0x00000009|DSP_5409)
                                                // For station channels on LD101, LD201

#define DIGITAL_PASSIVE_T1_NONE     0x00000010  // For channels on DP6400-T1, DP3200-T1
#define DIGITAL_PASSIVE_E_T1_NONE   (DIGITAL_PASSIVE_T1_NONE|DSP_5409)
                                                // For channels on DP6409-T1, DP3209-T1

#define DIGITAL_PASSIVE_T1_ISDN     0x00000011  // For channels on DP6400-T1, DP3200-T1
#define DIGITAL_PASSIVE_E_T1_ISDN   (DIGITAL_PASSIVE_T1_ISDN|DSP_5409)
                                                // For channels on DP6409-T1, DP3209-T1

#define DIGITAL_PASSIVE_T1_RBS      0x00000012  // For channels on DP6400-T1, DP3200-T1
#define DIGITAL_PASSIVE_E_T1_RBS    (DIGITAL_PASSIVE_T1_RBS|DSP_5409)
                                                // For channels on DP6409-T1, DP3209-T1

#define DIGITAL_PASSIVE_T1_NFAS     0x00000013  // For channels on DP6400-T1, DP3200-T1
#define DIGITAL_PASSIVE_E_T1_NFAS   (DIGITAL_PASSIVE_T1_NFAS|DSP_5409)
                                                // For channels on DP6409-T1, DP3209-T1

#define DIGITAL_PASSIVE_E1_NONE     0x00000020  // For channels on DP6400-E1, DP3200-E1
#define DIGITAL_PASSIVE_E_E1_NONE   (DIGITAL_PASSIVE_E1_NONE|DSP_5409)
                                                // For channels on DP6409-E1, DP3209-E1

#define DIGITAL_PASSIVE_E1_ISDN     0x00000021  // For channels on DP6400-E1, DP3200-E1
#define DIGITAL_PASSIVE_E_E1_ISDN   (DIGITAL_PASSIVE_E1_ISDN|DSP_5409)
                                                // For channels on DP6409-E1, DP3209-E1

#define DIGITAL_PASSIVE_E1_CAS      0x00000022  // For channels on DP6400-E1, DP3200-E1
#define DIGITAL_PASSIVE_E_E1_CAS    (DIGITAL_PASSIVE_E1_CAS|DSP_5409)
                                                // For channels on DP6409-E1, DP3209-E1

#define DIGITAL_PASSIVE_E1_NFAS     0x00000023  // For channels on DP6400-E1, DP3200-E1
#define DIGITAL_PASSIVE_E_E1_NFAS   (DIGITAL_PASSIVE_E1_NFAS|DSP_5409)
                                                // For channels on DP6409-E1, DP3209-E1

#define DIGITAL_PASSIVE_E1_DASS2    0x00000024  // For channels on DP6400-E1, DP3200-E1
#define DIGITAL_PASSIVE_E_E1_DASS2  (DIGITAL_PASSIVE_E1_DASS2|DSP_5409)
                                                // For channels on DP6409-E1, DP3209-E1

#define DIGITAL_PASSIVE_E1_DPNSS    0x00000025  // For channels on DP6400-E1, DP3200-E1
#define DIGITAL_PASSIVE_E_E1_DPNSS  (DIGITAL_PASSIVE_E1_DPNSS|DSP_5409)
                                                // For channels on DP6409-E1, DP3209-E1

#define DIGITAL_TERMINATE_T1_NONE   0x00000030  // For channels on DT6400-T1, DT3200-T1
#define DIGITAL_TERMINATE_E_T1_NONE (DIGITAL_TERMINATE_T1_NONE|DSP_5409)
                                                // For channels on DT6409-T1, DT3209-T1

#define DIGITAL_TERMINATE_T1_ISDN   0x00000031  // For channels on DT6400-T1, DT3200-T1
#define DIGITAL_TERMINATE_E_T1_ISDN (DIGITAL_TERMINATE_T1_ISDN|DSP_5409)
                                                // For channels on DT6409-T1, DT3209-T1

#define DIGITAL_TERMINATE_T1_RBS    0x00000032  // For channels on DT6400-T1, DT3200-T1
#define DIGITAL_TERMINATE_E_T1_RBS  (DIGITAL_TERMINATE_T1_RBS|DSP_5409)
                                                // For channels on DT6409-T1, DT3209-T1

#define DIGITAL_TERMINATE_T1_NFAS   0x00000033  // For channels on DT6400-T1, DT3200-T1
#define DIGITAL_TERMINATE_E_T1_NFAS (DIGITAL_TERMINATE_T1_NFAS|DSP_5409)
                                                // For channels on DT6409-T1, DT3209-T1

#define DIGITAL_TERMINATE_E1_NONE   0x00000040  // For channels on DT6400-E1, DT3200-E1
#define DIGITAL_TERMINATE_E_E1_NONE (DIGITAL_TERMINATE_E1_NONE|DSP_5409)
                                                // For channels on DT6409-E1, DT3209-E1

#define DIGITAL_TERMINATE_E1_ISDN   0x00000041  // For channels on DT6400-E1, DT3200-E1
#define DIGITAL_TERMINATE_E_E1_ISDN (DIGITAL_TERMINATE_E1_ISDN|DSP_5409)
                                                // For channels on DT6409-E1, DT3209-E1

#define DIGITAL_TERMINATE_E1_CAS    0x00000042  // For channels on DT6400-E1, DT3200-E1
#define DIGITAL_TERMINATE_E_E1_CAS  (DIGITAL_TERMINATE_E1_CAS|DSP_5409)
                                                // For channels on DT6409-E1, DT3209-E1

#define DIGITAL_TERMINATE_E1_NFAS   0x00000043  // For channels on DT6400-T1, DT3200-T1
#define DIGITAL_TERMINATE_E_E1_NFAS (DIGITAL_TERMINATE_E1_NFAS|DSP_5409)
                                                // For channels on DT6409-T1, DT3209-T1

#define DIGITAL_TERMINATE_PCM32     0x00000050
#define DIGITAL_TERMINATE_E_PCM32   (DIGITAL_TERMINATE_PCM32|DSP_5409)
                                                // For channels on DT6409-32T

#define DIGITAL_PASSIVE_NGX          0x00010000  // For channels on Passive NGX
#define DIGITAL_PASSIVE_NGX_ACD         0x00010000
#define DIGITAL_PASSIVE_NGX_DEFINITY    0x00010001
#define DIGITAL_PASSIVE_NGX_MERLIN      0x00010002
#define DIGITAL_PASSIVE_NGX_ELITE       0x00010003
#define DIGITAL_PASSIVE_NGX_NEAX        0x00010004
#define DIGITAL_PASSIVE_NGX_NORSTAR     0x00010005
#define DIGITAL_PASSIVE_NGX_MEDIDIAN    0x00010006
#define DIGITAL_PASSIVE_NGX_AXXESS      0x00010007

#if 0
#define DIGITAL_TERMINATE_NGX_TBD   0x00020000  // For channels on Terminate NGX
#endif

// Channel type for MTCHAN_INFO.Variant field
#define VARIANT_NONE                0x00000000  // For channels on VOICE_RESOURCE, ANALOG_VDS, etc.
#define RBS_IMMEDIATE               0x00000020  // For channels of T1 RBS
#define RBS_WINK                    0x00000021  // For channels of T1 RBS
#define RBS_LOOP_FXS                0x00000022  // For channels of T1 RBS
#define RBS_LOOP_FXO                0x00000023  // For channels of T1 RBS

//
// Channel load status codes (use with MTCHAN_INFO.LoadStatus)
//
#define CHAN_OK                     0x0000      // Channel initialized fine
#define CHAN_ERROR                  0x0001      // Channel initialization failed
#define CHAN_NOT_RESPONDING         0x0002      // Channel stopped responding
#define CHAN_PANIC                  0x0004      // Channel panic due to FW halted
#define CHAN_INCOMPATIBLE           0x0008      // Channel incompatible with FW or driver

//
// Channel status information
//
typedef struct _MT_CHAN_STATUS {
    ULONG   EventEnable;            // Transition event enable bits
    ULONG   Status;                 // Current channel status
    ULONG   CurrentDecFunction;     // Current channel decode (play) function
    ULONG   CurrentEncFunction;     // Current channel encode (record) function
    ULONG   Bytes;                  // Bytes being processed (encode+decode)
    ULONG   PreviousEvent;          // Previous termination event
    ULONG   RecordedBytes;          // Bytes recorded
    ULONG   PlayedBytes;            // Bytes played
    USHORT  DTMFCount;              // # of DTMF digits queued
    USHORT  MFCount;                // # of MF(R2 or R1) tones queued
    ULONG   Error;                  // Last error code
    ULONG   LineStatus;             // Status of phone line
    ULONG   CpmResult;              // Last CPM termination type
} MTCHAN_STATUS, *PMTCHAN_STATUS;


//
// Channel function status (MTCHAN_STATUS.Status field)
//
#define TASK_IDLE                  0x00000000
#define TASK_RECORD                0x00000001
#define TASK_PLAY                  0x00000002
#define TASK_RECPLAY               0x00000003   // TASK_RECORD+TASK_PLAY

#define TASK_GETDIGITS             0x00000004
#define TASK_OFFHOOK               0x00000008
#define TASK_DIALING               0x00000010
#define TASK_ONHOOK                0x00000020
#define TASK_CPM                   0x00000040
#define TASK_PARAM                 0x00000080
#define TASK_WINK                  0x00000100
#define TASK_PLAYTONE              0x00000200
#define TASK_PREREC                0x00000400
#define TASK_STREAMING             0x00000800

#define NOT_OPENED                 0xffffffff   // Channel not opened

//
// Phone line status enable bits (use with MTCHAN_STATUS.LineStatus)
//
#ifndef NV_CO_RESIDE
#define LINE_CPM_COMP               0x00000001    // CPM on
#define LINE_POLARITY               0x00000002    // LoopStart Tip&Ring polarity:
                                                  //   0 for normal; 1 for reversed
#define LINE_RINGBACK               0x00000004    // CPM detected ringback
#define LINE_ON_HOOK                0x00000008    // Channel is on-hook
#define LINE_NO_RINGS               0x00000010    // No incoming rings detected
#define LINE_NO_LOOP                0x00000020    // No loop current present
#define LINE_DTMF                   0x00000040    // DTMF signal currently present
#define LINE_SILENT                 0x00000080    // Silence on the line detected
#endif
#define LINE_NO_LVOLTAGE_MIDDLE     0x00000100    // Loop voltage state not MIDDLE
#define LINE_NO_LVOLTAGE_OFFHOOK    LINE_NO_LVOLTAGE_MIDDLE
                                                  // No loop voltage offhook present
#define LINE_NO_DIAL                0x00000200    // No dial tone present


//
// Channel run-time statistic information
//
typedef struct _MT_RUNTIME_STAT {
    // This reports the trouble firmware is in
    ULONG   PanicCode;              // Panic code for technical support

    //  The following happens when driver and firmware are incompatible
    ULONG   UnknownEvent;           // Unknown event
    ULONG   UnknownRsp;             // Unknown response
    ULONG   CmdError;               // Command error

    // The following happens when application fails to retrieve info from queues
    ULONG   EventQueueOverflow;     // Channel event queue overflow
    ULONG   DTMFQueueOverflow;      // Channel DTMF queue overflow
    ULONG   MFQueueOverflow;        // Channel MF queue overflow
    ULONG   CIDQueueOverflow;       // Channel Caller ID queue overflow

    // The following happens when host ran into performance trouble
    ULONG   EncodeOverflow;         // Channel record oveflow
    ULONG   DecodeUnderflow;        // Channel playback underflow

    // The following may happen when firmware is running into trouble
    ULONG   EncodeTimeout;          // Channel record timeout
    ULONG   DecodeTimeout;          // Channel playback timeout

    // The following is for HDLC packets
    ULONG   HDLCStreamError;        // Firmware detects bad HDLC stream

    ULONG   Reserved[48];           // Reserved for SDK internal use
} MTRUNTIME_STATISTICS, *PMTRUNTIME_STATISTICS;


//
// CALL_RESULT is currently defined in NTI_NV.h
//

//
// Summation resource information
//
typedef struct _MTSUMMATION_INFO {
    int     tResource;              // Summation resource sub-type, if any
    int     iResource;              // Summation resource system index
    int     iChGCI;                 // GCI index of the channel which controls
                                    // the summation resource
    int     iBoard;                 // System index of the board the summation resides
    int     iOnBoard;               // Summation index within the board it resides
    int     nInputs;                // Number of inputs for the summation
} MTSUMMATION_INFO, * PMTSUMMATION_INFO;


//
// Subtypes in CALL_RESULT.SubTerm:
//

#if 0
//
// CPM_BUSY types:
//
#define CPM_BUSY_NORMAL          129    // Tone CPM_BUSY1 (Normal busy) termination
#define CPM_BUSY_TRUNK           130    // Tone CPM_BUSY2 Trunk (fast) busy termination

//
// CPM_NO_RINGBACK types:
//
#define CPM_NOR_DIAL             131    // Continuous dial tone, no ringback
#define CPM_NOR_SILENCE          132    // Continuous silence, no ringback
#define CPM_NOR_ACTIVITY         133    // Continuous activity, no ringback
#endif

//
// CPM_CONNECT types (in MT_EVENT.XtraInfo field) for MTCallString():
//
#define CPM_CONNECT_MODEM        148    // Connect to modem
#define CPM_CONNECT_MACHINE      149    // Connect on answering machine
#define CPM_CONNECT_HUMAN        152    // Connect on human voice
#define CPM_CONNECT_VOICE        CPM_CONNECT_HUMAN
#if 0
#define CPM_CONNECT_FAX          147    // Connect to fax machine
#define CPM_CONNECT_LOOP         151    // Connect on loop current drop
#define CPM_CONNECT_CADBRK       153    // Connect on cadence break
#endif

//
// Indexed-play table structure
//

typedef struct _MT_INDEX_TABLE {
    ULONG     PhraseOffset;               // Phrase offset in the index file
    ULONG     PhraseLength;               // Length of the phrase, in bytes
} MT_INDEX_TABLE, *PMT_INDEX_TABLE;

typedef struct _MT_INDEX_TABLE const * PCMT_INDEX_TABLE;

#define MAX_FILEINDEXES     4096


//
// Indexed-play table structure (includes file name)
//

typedef struct _MT_FINDEX_TABLEA {
  PSTR      FileName;                   // File name
  ULONG     PhraseOffset;               // Phrase offset in the index file
  ULONG     PhraseLength;               // Length of the phrase, in bytes
} MT_FINDEX_TABLEA, *PMT_FINDEX_TABLEA;

typedef struct _MT_FINDEX_TABLEA const *PCMT_FINDEX_TABLEA;

typedef struct _MT_FINDEX_TABLEW {
  PWSTR     FileName;                   // File name (wide)
  ULONG     PhraseOffset;               // Phrase offset in the index file
  ULONG     PhraseLength;               // Length of the phrase, in bytes
} MT_FINDEX_TABLEW, *PMT_FINDEX_TABLEW;

typedef struct _MT_FINDEX_TABLEW const *PCMT_FINDEX_TABLEW;


//
// performance counters
//
typedef struct _MT_PERF_INFO {
    ULONG   Status;
    PVOID   VirtualAddress;
} MT_PERF_INFO, *PMT_PERF_INFO;

typedef struct _MT_PERF_COUNTERS {
    ULONG   Interrupts;
    ULONG   ReadCount;
    ULONG   ReadBytes;
    ULONG   WriteCount;
    ULONG   WriteBytes;
    ULONG   IoCount;
    ULONG   IoBytes;
    ULONG   OffHookCount;
    ULONG   OnHookCount;
    ULONG   RingCount;
    ULONG   FileOpenClose;
    ULONG   PromptsPlayed;
    ULONG   ChannelsInCpmMode;
    ULONG   BkgrndIoCompletions;
    ULONG   PlayBufferCount;
    ULONG   FilePlayCount;
    ULONG   CallCount;
    ULONG   ErrorCount;
} MT_PERF_COUNTERS, *PMT_PERF_COUNTERS;


//
// Tone Structures

// CPM Tone Pulse structure
typedef struct {
    USHORT      onmin;              // Signal minimum on time (ms)
    USHORT      onmax;              // Signal maximum on time (ms)
    USHORT      offmin;             // Signal minimum off time (ms)
    USHORT      offmax;             // Signal maximum off time (ms)
} CPM_PULSE;

#define MAX_CPM_PULSE       3       // Maximum pulse per CPM tone signal
#define MAX_CPM_FREQ        3       // Maximum frequency per CPM tone signal
#define MAX_CPM_CYCLE       256     // Maximum cycles per CPM tone signal

typedef struct {
    USHORT      fEnable;
    USHORT      fDetectSIT;
    USHORT      fDetectLate;
    USHORT      fDetectCycle;
    USHORT      fDetectLost;
    USHORT      nCycleDetect;
    USHORT      Freq[MAX_CPM_FREQ];
    CPM_PULSE   Pulse[MAX_CPM_PULSE];
} CPM_SIGNAL;


// MT CPM Tone Structure
typedef struct {
    USHORT      index;                  // Tone table index (0-19)
                                        //  0-14: CPM tone, 15-19: USR tone
    USHORT      signalname;             // Tone ID: 0-199 for USR tone; or CPM_DIAL1, etc.
    USHORT      signaltype;             // Tone type (CPM_TONE, USR_TONE, SIT_TONE)

    CPM_SIGNAL  signal;                 // CPM tone specification
} MT_CPMTONE, *PMT_CPMTONE; //, MT_USRTONE, *PMT_USRTONE;


typedef struct {
    MT_CPMTONE  Tone;                   // Tone specification
    UCHAR       asCPMTone;              // Treat this tone as the specified CPM tone
    UCHAR       useDigit;               // The digit if asCPMTone is set to USR_DIGIT
} MT_USRTONE, *PMT_USRTONE;


// Call Progress Tone Structures
typedef struct {
    USHORT      nLowFreqCutoff;         // Low frequency cutoff
    USHORT      nHighFreqCutoff;        // Upper frequency cutoff
} CPM_FILTER, *PCPM_FILTER;


typedef struct {
    USHORT      fEnable;
    USHORT      fDetectSIT;
    USHORT      fDetectLate;
    USHORT      fDetectCycle;
    USHORT      fDetectLost;
    USHORT      nCycleDetect;
    CPM_FILTER  Filter[MAX_CPM_FREQ];
    CPM_PULSE   Pulse[MAX_CPM_PULSE];
} CPT_SIGNAL, *PCPT_SIGNAL;


// MT CPT Tone Structure
typedef struct {
    USHORT      index;                  // Tone table index (0-19)
                                        //  0-14: CPM tone, 15-19: USR tone
    USHORT      signalname;             // Tone ID: 1-199 for USR tone; or CPM_DIAL1, etc.
    USHORT      signaltype;             // Tone type (CPM_TONE, USR_TONE, SIT_TONE)

    CPT_SIGNAL  signal;                 // CPT tone specification
} MT_CPT_TONE, *PMT_CPT_TONE;

typedef struct {
    MT_CPT_TONE  Tone;                  // Tone specification
    UCHAR       asCPMTone;              // Treat this tone as the specified CPM tone
    UCHAR       useDigit;               // The digit if asCPMTone is set to USR_DIGIT
} MT_USR_TONE, *PMT_USR_TONE;


// MT ALERT Tone Structure
typedef struct {
    float       amplitude;              // Amplitude in dBm, 3.0 dBm to -60.0 dBm
    USHORT      frequency;              // Frequency in Hz
    USHORT      cycle_duration;         // Repeat duration in unit of 20ms; eg. 750->15sec
    USHORT      tone_duration;          // Alert tone duration in unit of 20ms: eg. 10->200ms
    USHORT      first_cycle_duration;   // The very first alert tone cycle duration in 20ms
} MT_ALERTTONE, *PMT_ALERTTONE;


//
// Parameter Structures
//

/**** ACT structure  ****/
typedef struct {
        float   threshold_low;          // Silence threshold 0 to -60.0 dBm, default -51.0 dBm
        float   threshold_high;         // Activity threshold 0 to -60.0 dBm, default -48.0 dBm
        USHORT  min_silence;            // Minimum silence duration in msec
        USHORT  max_silence;            // Maximum silence duration in msec
        USHORT  min_activity;           // Minimum activity duration in msec
        USHORT  max_activity;           // Maximum activity duration in msec
} MT_ACTPARAMS, *PMT_ACTPARAMS;

/***  DTMF/MF Detection ***:       */
typedef struct {
        USHORT  subtype;                // 0 for MF-R1; others for MF-R2
        USHORT  trigger;                // leading and/or tailing detection
} MT_DETECTREGS, *PMT_DETECTREGS;


// DTMF Detection specific parameters
typedef struct {
        USHORT  min_detect_time;        // Min. time to detect a DTMF tone, 40(default) - 100 ms
        USHORT  max_report_time;        // Max. time to report leading edge detection of a DTMF
                                        // tone, ranged min_detect_time to 100(default) ms
} MT_DTMF_DETECTPARAMS, *PMT_DTMF_DETECTPARAMS;

#define TRAILING_EDGE               1
#define LEADING_EDGE                2   // Default for MT_DETECTREGS.trigger

#define TYPE_DTMF                   0
#define TYPE_MF                     1

#define SUBTYPE_MF_R1               0
#define SUBTYPE_MF_R2               1

#define Q_DISABLED                  0   // Tone (DTMF or MF) detect disabled
#define Q_FLUSH                     1   // Tone reported without adding to the queue
#define Q_KEEP                      2   // Tone reported and added to the queue


/***  DTMF/MF Generation ***:      */
typedef  struct {
        USHORT  width;                 /* tone duration time   */
        USHORT  gap;                   /* inter-digit duration */
        USHORT  low_ampl;              /* amplitude for low freq. */
        USHORT  high_ampl;             /* amplitude for high freq. */
} MT_GENREGS, *PMT_GENREGS;

/**** PSTN structure   ****/
typedef struct {
        USHORT   flash_char;            /* flash char (default &)         */
        USHORT   flash_time;            /* flash time (ms)                */
        USHORT   wink_delay;            /* wink delay (for reading only)  */
        USHORT   wink_time;             /* wink time  (for reading only)  */
        USHORT   pause_char;            /* pause char (default ',')       */
        USHORT   pause_time;            // pause time (ms) per pause char, max 8000 ms
        USHORT   pulse_on;              /* Pulse dialing onhook length    */
        USHORT   pulse_off;             /* Pulse dialing offhook length   */
        USHORT   pulse_delay_time;      /* Pulse dial inter-digit delay   */
        USHORT   loop_deglitch;         // minimum (ms) glitch duration, default 50
                                        // ranged 10 to 2550 in increments of 10
        USHORT   loop_delay;            /* loop event delay (in ms)       */
        USHORT   ring_on_time;          /* ring on duration (ms)          */
        USHORT   ring_off_time;         /* ring off duration(ms)          */
        USHORT   ring_reset;            // duration (ms) to reset ring count to 0, default 8000
        USHORT   ring_count;            // ring detected to report event EVT_RING, default 1
        USHORT   ring_trigger;          // ring detection: 0-leading, 1-tailing, default 0
        USHORT   ring_autoanswer;       // auto-answer: 0-disable, 1-enable, default 0
        USHORT   cpm_start_delay;       /* CPM analysis delay after dialing */
        USHORT   offhook_delay;         /* offhook event delay             */
        USHORT   min_rings_no_answer;   /* Minimum cycles to report no answer */
        USHORT   max_sig_time;          /* Maximum continuous signal time  */
        USHORT   max_sil_time;          /* Maximum continuous silence time */
        USHORT   ring_deglitch;         // minimum (ms) glitch duration, default 50 for VDS
                                        // 300 for TRUNK channels, ranged the same as loop_deglitch
        USHORT   reserved[3];
} MT_PSTN, *PMT_PSTN;


/**** EC (Echo cancellation) Parameter structure  ****/
typedef struct{
        SHORT    max_comfort_noise;     // Comfort noise range: -40 to -66 dBm inclusive
        SHORT    fEnableNLP;            // NLP switch: 1 for enable, 0 for disable
} MT_ECPARAMS, *PMT_ECPARAMS;


/*** Play Tone structure ***: */
typedef struct  {
        float   ampl1;                  // Amplitude for tone 1 (dBm), 3.0 dBm to -60.0 dBm
        USHORT  freq1;                  // Frequency for tone 1 (Hz)
        float   ampl2;                  // Amplitude for tone 2 (dBm), 3.0 dBm to -60.0 dBm
        USHORT  freq2;                  // Frequency for tone 2 (Hz)
        USHORT  durtone;                // Duration of generated tone (125us)
        USHORT  dursilence;             // Duration of silence (125us)
} MT_PLAYTONE, *PMT_PLAYTONE;


//================ FAX Channel Information
typedef struct {
        USHORT  number_of_pages;
        USHORT  channel_number;
        USHORT  phone_validity;
        WCHAR   phone_number[20];
        USHORT  transmitter_receiver;
        USHORT  inbound_outbound;
        USHORT  automatic_manual;
        USHORT  modem_protocol;
        USHORT  vertical_resolution;
        USHORT  coding_2d_capability;
        USHORT  recording_width;
        USHORT  recording_length;
        USHORT  minimum_scan_line_time;
        USHORT  handshake2400;
        USHORT  uncompressed_mode;
        USHORT  operator_alert_request;
} MTFAX_INFO;

/**** Voice/Answering Machine Detection Parameter structure  ****/
typedef struct {
        float   threshold_power;        //
        USHORT  relative_threshold;     //
        USHORT  threshold_filter_count; //
        USHORT  no_voice_min;           //
        USHORT  reset;                  //
        USHORT  voice_min;              //
        USHORT  machine_min;            //
} MT_VOICEDETECTPARAMS, *PMT_VOICEDETECTPARAMS;

//
// Options for 'Source' for MTSetOutputSource()
//
#define CHANNEL_DEVICE      0
#define AUDIO_STREAM        1
#define PRIMARY_INPUT       2
#define SECONDARY_INPUT     3


//
// CPM Tone signal name:
// CPM tone Definition MT_CPMTONE.signalname field, i.e. toneID in CPM profile structure
//
#define CPM_NONE                0
#define CPM_DIAL1               250         // Local dial tone
#define CPM_DIAL2               251         // International dial tone
#define CPM_DIAL3               252         //
#define CPM_BUSY1               253         // Normal busy tone
#define CPM_RINGBACK1           254         // Normal ringback tone
#define CPM_BUSY2               255         // Trunk busy tone
#define CPM_RINGBACK2           256         // Double ringback tone
#define CPM_FAX1                258         // Called station FAX tone
#define CPM_FAX2                259         // Calling station FAX tone
#define CPM_SIT1                284         // No circuit found (NC)
#define CPM_SIT2                285         // Operator intercept (IC)
#define CPM_SIT3                286         // Vacant circuit (VC)
#define CPM_SIT4                287         // Re-order (RO)
#define CPM_SIT5                288         // Ineffective other (IO)
#define CPM_CALLWAITING         282         // Call waiting tone
#define CPM_RECEIVEOFF          283         // Receive Off-Hook tone

//
// Tone type in MT_CPMTONE.signaltype field
//
#define CPM_TONE                0
#define SIT_TONE                1
#define USR_TONE                2

//
// User Tone signal name:
// User tone range Definition MT_CPMTONE.signalname field,
//  i.e. toneID in user tone profile structure
//
#define USR_ZERO                0           // The

//
// Tone type in MT_USRTONE.asCPMTone field
//
#define USR_NONE                0           // Report EVT_UTONE_ON on detection

#define USR_DIAL1               1           // Treat this tone as CPM_DIAL1
#define USR_RINGBACK1           2           // Treat this tone as CPM_RINGBACK1
#define USR_BUSY1               3           // Treat this tone as CPM_BUSY1
#define USR_FAX1                4           // Treat this tone as CPM_FAX1
#define USR_SIT1                5           // Treat this tone as CPM_SIT1
#if 1
#define USR_DIAL                USR_DIAL1       // Backward compatibility macro
#define USR_RINGBACK            USR_RINGBACK1   // Backward compatibility macro
#define USR_BUSY                USR_BUSY1       // Backward compatibility macro
#define USR_FAX                 USR_FAX1        // Backward compatibility macro
#define USR_SIT                 USR_SIT1        // Backward compatibility macro
#endif

#define USR_LOOPOFF             6           // Treat this tone as CPM_RECEIVEOFF
#define USR_DIGIT               7           // Report EVT_DIGIT and the specified
                                            // digit through DTMF queue

#define USR_DIAL2               8           // Treat this tone as CPM_DIAL2
#define USR_DIAL3               9           // Treat this tone as CPM_DIAL3
#define USR_RINGBACK2           10          // Treat this tone as CPM_RINGBACK2
#define USR_BUSY2               11          // Treat this tone as CPM_BUSY2
#define USR_FAX2                12          // Treat this tone as CPM_FAX2
#define USR_SIT2                13          // Treat this tone as CPM_SIT2
#define USR_SIT3                14          // Treat this tone as CPM_SIT3
#define USR_SIT4                15          // Treat this tone as CPM_SIT4
#define USR_SIT5                16          // Treat this tone as CPM_SIT5
#define USR_CALLWAITING         17          // Treat this tone as CPM_CALLWAITING


//
// Termination Event enable bits
//
#define TERM_LOOP_DROP        0x00000001    // Terminate on loop current drop
#if 0   // Obsoleted 11/22/01
#define TERM_RINGS            0x00000002    // Terminate on incoming Rings
#endif
#define TERM_MON_SIL          0x00000004    // Terminate on Silence detection
#define TERM_MON_ACT          0x00000008    // Terminate on Activity detection
#define TERM_MON_DIAL         0x00000010    // Terminate on ANY Dial Tone detection
#define TERM_MON_BUSY         0x00000020    // Terminate on ANY Busy Tone detection
#if 0   // Not available in NTI, use TERM_MON_BUSY
#define TERM_BUSY             0x00000100    // Terminate on Busy
#endif
#define TERM_CONNECT          0x00000200    // Terminate on Ringback Tone disappearing
#define TERM_NO_ANSWER        0x00000400    // Terminate on No Answer
#define TERM_INTERCEPT        0x00000800    // Terminate on ANY SIT tone detection
#if 0   // Not available in NTI
#define TERM_NOR_DIAL         0x00001000    // Terminate on contin. Dial Tone
#endif
#define TERM_NOR_SIL          0x00002000    // Terminate on contin. Silence
#define TERM_NOR_ACT          0x00004000    // Terminate on contin. Activity

#define TERM_LVOLTAGE_ABOVEORBELOW 0x00010000// Terminate on loop voltage changed
                                            // to state ABOVE or BELOW
#define TERM_LVOLTAGE_NOTOFFHOOK TERM_LVOLTAGE_ABOVEORBELOW

#define TERM_MON_UTONE        0x00020000    // Terminate on detection of tone defined as USR_TONE with
                                            // asCPMTone set to CPM_NONE
#define TERM_MON_CALLWAITING  0x00040000    // Terminate on detection of tone defined as CPM_CALLWAITING
#define TERM_MON_RECEIVEOFF   0x00080000    // Terminate on detection of tone defined as CPM_RECEIVEOFF

#define TERM_MON_DIAL1        0x00100000    // Terminate on detection of tone defined as CPM_DIAL1
#define TERM_MON_DIAL2        0x00200000    // Terminate on detection of tone defined as CPM_DIAL2
#define TERM_MON_DIAL3        0x00400000    // Terminate on detection of tone defined as CPM_DIAL3
#define TERM_MON_BUSY1        0x00800000    // Terminate on detection of tone defined as CPM_BUSY1
#define TERM_MON_BUSY2        0x01000000    // Terminate on detection of tone defined as CPM_BUSY2
#define TERM_MON_FAX1         0x02000000    // Terminate on detection of tone defined as CPM_FAX1
#define TERM_MON_FAX2         0x04000000    // Terminate on detection of tone defined as CPM_FAX2
#define TERM_MON_SIT1         0x08000000    // Terminate on detection of tone defined as CPM_SIT1
#define TERM_MON_SIT2         0x10000000    // Terminate on detection of tone defined as CPM_SIT2
#define TERM_MON_SIT3         0x20000000    // Terminate on detection of tone defined as CPM_SIT3
#define TERM_MON_SIT4         0x40000000    // Terminate on detection of tone defined as CPM_SIT4
#define TERM_MON_SIT5         0x80000000    // Terminate on detection of tone defined as CPM_SIT5

#define TERM_DEFAULT          0x0000FF01    // Default termination enabled bits

//
// Monitor Events enable bits for MTSetMoni()
//
#define MONI_BUSY1            0x00000001    // Monitor tone defined as CPM_BUSY1
#define MONI_BUSY             MONI_BUSY1    // Backward compatibility macro
#define MONI_BUSY2            0x00000002    // Monitor tone defined as CPM_BUSY2
#define MONI_TBUSY            MONI_BUSY2    // Backward compatibility macro
#define MONI_DIAL             0x00000004    // Monitor ALL tone defined as CPM_DIAL
#define MONI_CALLWAITING      0x00000008    // Monitor tone defined as CPM_CALLWAITING
#define MONI_RECEIVEOFF       0x00000010    // Monitor tone defined as CPM_RECEIVEOFF
#define MONI_RINGBACK1        0x00000020    // Monitor tone defined as CPM_RINGBACK1
#define MONI_RINGBACK         MONI_RINGBACK1   // Backward compatibility macro
#define MONI_RINGBACK2        0x00000040    // Monitor tone defined as CPM_RINGBAKC2
#define MONI_DRINGBACK        MONI_RINGBACK2   // Backward compatibility macro
#define MONI_SIT              0x00000080    // Monitor ALL tones defined as CPM_SIT
#define MONI_FAX              0x00000100    // Monitor ALL tones defined as CPM_FAX

#define MONI_SIGNAL_CYCLE     0x00000200    // Monitor cycle of CPM/User signal(tone)

#define MONI_REVERSAL         0x00000400    // Monitor polarity changes
#define MONI_VOICE            0x00000800    // Monitor voice/human/machine/novoice changes

#define MONI_SIT1             0x00001000    // Monitor tone defined as CPM_SIT1
#define MONI_SIT2             0x00002000    // Monitor tone defined as CPM_SIT2
#define MONI_SIT3             0x00004000    // Monitor tone defined as CPM_SIT3
#define MONI_SIT4             0x00008000    // Monitor tone defined as CPM_SIT4
#define MONI_SIT5             0x00010000    // Monitor tone defined as CPM_SIT5
#define MONI_DIAL1            0x00020000    // Monitor tone defined as CPM_DIAL1
#define MONI_DIAL2            0x00040000    // Monitor tone defined as CPM_DIAL2
#define MONI_DIAL3            0x00080000    // Monitor tone defined as CPM_DIAL3

#define MONI_FAX1             0x00100000    // Monitor tone defined as CPM_FAX1
#define MONI_FAX2             0x00200000    // Monitor tone defined as CPM_FAX2
#define MONI_UTONE            0x00400000    // Monitor tone defined as USR_TONE with asCPMTone set to CPM_NONE

#define MONI_DEFAULT          (MONI_BUSY | MONI_DIAL | MONI_RINGBACK)

//
// T1 Alarm enable bits
//
#define ALARM_LOS             0x00000001    // Loss of Signal alarm enable
#define ALARM_LOF             0x00000002    // Loss of Frame alarm enable
#define ALARM_AIS             0x00000004    // Alarm Indication Signal alarm enable
#define ALARM_YELLOW          0x00000008    // Yellow alarm enable

//
// E1 Alarm enable bits
//
#define ALARM_LOSMF           0x00000100    // Loss of Signaling Multiframe Alignment alarm enable
#define ALARM_LOCRCMF         0x00000200    // Loss of CRC Multiframe Alignment alarm enable
#define ALARM_TS16RAI         0x00000400    // Timeslot 16 Remote Alarm Indication Signal alarm enable


//#define ALARM_T1_DEFAULT      (ALARM_T1_LOS | ALARM_T1_LOF | ALARM_T1_AIS | ALARM_T1_YELLOW)
#if 0       // Obsoleted 6/11/02
typedef struct _MT_FRAMER_STATISTICS {
        ULONG   FrameErr;           // T1 Framing bit error or E1 FAS error count
        ULONG   OutOfFrameErr;      // T1 out of frame event count
        ULONG   FarEndBlockErr;     // E1 far end block error count
        ULONG   BitErr;             // T1 ESF CRC-6 error, for SF framing bit error count
        ULONG   CRCErr;             // E1 CRC-4 error count
        ULONG   LineCode_ZeroRunErr;// Line code vialation count
} MT_FRAMER_STAT, *PMT_FRAMER_STAT;
#endif

//
// PCM32 Alarm enable bits
//
#define ALARM_LOSYNC          0x00000001    // Loss of SYNC alarm enable


typedef struct _MT_T1_STATISTICS {
        ULONG   FrameErr;           // T1 Framing bit error
        ULONG   OutOfFrameErr;      // T1 out of frame event count
        ULONG   BitErr;             // T1 ESF CRC-6 error, for SF framing bit error count
        ULONG   LineCode_ZeroRunErr;// Line code vialation count
} MT_T1_FRAMER_STAT, *PMT_T1_FRAMER_STAT;

typedef struct _MT_E1_STATISTICS {
        ULONG   FASErr;             // E1 FAS error count
        ULONG   FarEndBlockErr;     // E1 far end block error count
        ULONG   CRCErr;             // E1 CRC-4 error count
        ULONG   LineCode_ZeroRunErr;// Line code vialation count
} MT_E1_FRAMER_STAT, *PMT_E1_FRAMER_STAT;


typedef struct _MT_NTTE_STATISTICS {
        ULONG NTErr;                // Pbx errors
        ULONG TEErr;                // Phone errors
        ULONG SyncErr;              // How many times signal loss
        float NTAmpl;               // Pbx amplitude in volts
        float TEAmpl;               // Phone amplitude in volts
        float Noise;                // Noise level in volts
        UCHAR SignalClipStatus;         // Clipping status
} MT_NTTE_FRAMER_STAT, *PMT_NTTE_FRAMER_STAT;

typedef struct {
        UCHAR   nFramer;            // Framer index
        UCHAR   LOS_Alarm;          // LOS alarm present
        UCHAR   LOF_Alarm;          // LOF alarm present
        UCHAR   AIS_Alarm;          // AIS alarm present
        UCHAR   YELLOW_Alarm;       // T1 YELLOW alarm present, E1 RAI present
        UCHAR   LOSMF_Alarm;        // E1 loss of signaling MF alignment indicator
        UCHAR   LOCRC4MF_Alarm;     // E1 loss of CRC-4 MF alignment indicator
        UCHAR   TS16RAI_Alarm;      // E1 timeslot 16 RAI (Remote Alarm Indicator) indicator
} MT_FRAMER_STATUS, *PMT_FRAMER_STATUS;

typedef struct _MT_TS_SIGNALLING {
        UCHAR           Framer;
        UCHAR           TimeSlot;
        UCHAR           Signal;         // ABCD bits:
                                        //  0x01 - D bit
                                        //  0x02 - C bit
                                        //  0x04 - B bit
                                        //  0x08 - A bit
//      UCHAR           PrevSignal;     // Previous signaling
//      ULONG           ElapsedTime;    // Duration (in 10ms unit) between Signal and PrevSignal
} MT_TS_SIGNALING, *PMT_TS_SIGNALING;

typedef struct {
        UCHAR   nBoard;             // Board index
        UCHAR   nFramer;            // Framer index
        UCHAR   State;              // Framer state: reset, passive or terminate
        UCHAR   reserved;
        UCHAR   FramingFormat;      // T1: SF or ESF; E1: G.704 or CRC-4
        UCHAR   LineCoding;         // T1: AMI or B8ZS; E1: AMI or HDB3
        UCHAR   LBO;                // See LBO_ definitions
        UCHAR   ZCS;                // See ZCS_ definitions
} MT_FRAMER_STATE, *PMT_FRAMER_STATE;

// Possible values for MT_FRAMER_STATE.State field which is ignored in
//  updated in MTGetFraming()
#define     FRAMER_RESET            0x00
#define     FRAMER_T1_PASSIVE       0x10
#define     FRAMER_T1_TERMINATE     0x11
#define     FRAMER_E1_PASSIVE       0x20
#define     FRAMER_E1_TERMINATE     0x21

// Possible values for MTADAPTER_CONFIG.Framing[] field
#define     FRM_E1_G704              0x00   // default
#define     FRM_E1_CRC4              0x01
#define     FRM_E1_UNFRAMED          0x02
#define     FRM_T1_SF                0x00   // default
#define     FRM_T1_ESF               0x01
#define     FRM_T1_UNFRAMED          0x02

// Possible values for MT_FRAMER_STATE.FramingFormat field
#define     FF_E1_G704              0x00    // default
#define     FF_E1_CRC4              0x01
#define     FF_E1_UNFRAMED          0x02
#define     FF_T1_SF                0x03
#define     FF_T1_ESF               0x07
#define     FF_T1_UNFRAMED          0x0d

// Possible values for MTADAPTER_CONFIG.LineCoding[] field
#define     LC_AMI                  0x00    // default
#define     LC_E1HDB3_T1B8ZS        0x01

// Possible values for MT_FRAMER_STATE.LineCoding field
#define     LC_AMI                  0x00    // default
#define     LC_E1_HDB3              0x01
#define     LC_T1_B8ZS              0x02

// Possible values for MT_FRAMER_STATE.ZCS field
#define     ZCS_NONE                0x00    // default
#define     ZCS_GTE                 0x01
#define     ZCS_DDS                 0x02
#define     ZCS_BELL                0x03

// Possible values for MT_FRAMER_STATE.LBO
#define     LBO_T1_0DB              0x00
#define     LBO_T1_7_5DB            0x01
#define     LBO_T1_15DB             0x02    // T1 Default
#define     LBO_T1_22_5DB           0x03
#define     LBO_T1_TR62411_0DB      0x04
#define     LBO_T1_110FT            0x05
#define     LBO_T1_220FT            0x06
#define     LBO_T1_330FT            0x07
#define     LBO_T1_440FT            0x08
#define     LBO_T1_550FT            0x09
#define     LBO_T1_660FT            0x0a
#define     LBO_T1_TR62411_110FT    0x0b
#define     LBO_T1_TR62411_220FT    0x0c
#define     LBO_T1_TR62411_330FT    0x0d
#define     LBO_T1_TR62411_440FT    0x0e
#define     LBO_T1_TR62411_550FT    0x0f
#define     LBO_T1_TR62411_660FT    0x10
#define     LBO_E1_75OHM            0x11
#define     LBO_E1_120OHM           0x12    // E1 Default


//
// DTMF_PARAMS
//
#define DTMF_BREAKOFF              1    // Disable DTMF break
#define DTMF_BREAKON               2    // Enable DTMF break

//
// Term/Moni Modes (Old CPM Modes)
//
#define MONI_MODE_MASK             0x80000001  // defined by bit 0 and bit 31
#define TERM_MODE_MASK             0x40000002  // defined by bit 1 and bit 30

#define MONI_USE_CHANNEL_ONLY      0x00000000  // Use channel setting set through MTSetMoni()
#define MONI_USE_NONE              0x80000000  // Ignore monitoring settings in MoniEnable and
                                               // channel setting set through MTSetMoni()
#define MONI_USE_TASK_ONLY         0x80000001  // Use only setting in MoniEnable field
#define MONI_USE_CHANNEL_AND_TASK  0x00000001  // Use combined setting of task MoniEnable and
                                               // channel setting set through MTSetMoni()

#define TERM_USE_NONE              0x00000000  // Ignore TERM settings in TermEnable and
                                               // settings set through MTSetTerm()
#define TERM_USE_CHANNEL_AND_TASK  0x00000002  // Use TERM settings in TermEnable and
                                               // settings set through MTSetTerm()
#define TERM_USE_CHANNEL_ONLY      0x40000000  // Use TERM settings set through MTSetTerm()
#define TERM_USE_TASK_ONLY         0x40000002  // Use TERM settings in TermEnable

// Backward compatible definitions
#define CPMMODE_NONE               (MONI_USE_CHANNEL_ONLY | TERM_USE_NONE)
#define CPMMODE_MONI               (MONI_USE_CHANNEL_AND_TASK | TERM_USE_NONE)    // CPM monitor, no termination
#define CPMMODE_COMP               (MONI_USE_CHANNEL_ONLY | TERM_USE_CHANNEL_AND_TASK) // CPM compatible to current channel setting

//
// Digit Queue Control bits
#define DQ_FLUSH              0x00000001    // Flush digit queue
#define DQ_EVENTS             0x00000002    // Generate digit events

//
// SET_EVENTS bits for MTSetEventFilters()
//
#define SE_LCURRENT_CHANGE      0x00000001      // Loop current change
#ifndef SE_LOFF
#define SE_LOFF                 SE_LCURRENT_CHANGE      // Loop current OFF->Change
#endif
#ifndef SE_RING
#define SE_RING                 0x00000002      // Rings received
#define SE_ACT                  0x00000004      // Activity detected
#define SE_SIL                  0x00000008      // Silence detected
#define SE_OFFH                 0x00000010      // Offhook complete
#define SE_ONH                  0x00000020      // Onhook complete
#endif
#ifndef SE_LON
#define SE_LON                  SE_LCURRENT_CHANGE      // Loop current ON->Change
#endif
#ifndef SE_WKRECV
#define SE_WKRECV               0x00000080      // Recv incoming wink, Obsoleted
#define SE_LREV                 0x00000100      // Obsoleted, this is inclusive of
                                               // SE_LCURRENT_CHANGE and SE_LVOLTAGE_CHANGE
#endif
#define SE_LVOLTAGE_CHANGE      0x00000200      // Loop voltage change

#if 0   // Obsoleted, 11/22/01
//
// Equates for cst_enable (old cst_mask)
//
#define  C_LC        0x01               // Loop signal drop
#define  C_RING      0x02               // Rings received
#define  C_SILOFF    0x04               // Silence off
#define  C_SILON     0x08               // Silence on
#define  C_OFFH      0x10               // Offhook complete
#define  C_ONH       0x20               // Onhook complete
#define  C_LCON      0x40               // Loop signal on
#define  C_WKRECV    0x80               // Recv incoming wink
#define  C_LREV      0x100              // Loop Reversal

#endif

//
// Gain step values
//

#define STEP_DOWN                 -1    // Step channel gain down
#define STEP_RESET                 0    // Reset channel gain
#define STEP_UP                    1    // Step channel gain up

//
// Direction values when entering a group
//
#define DIR_NONE                             0x00    // Direction not specified
#define DIR_TALK                             0x01    // Direction Talk to group
#define DIR_LISTEN                           0x02    // Direction Listen to group
#define DIR_BOTH                             0x03    // Both Talk and Listion to group
#define DIR_SUPER                            0x04    // Supervisor Mode
#define DIR_STALK          (DIR_SUPER | DIR_TALK)    // Supervisor Talk to group
#define DIR_SLISTEN      (DIR_SUPER | DIR_LISTEN)    // Supervisor Listen to group
#define DIR_SBOTH          (DIR_SUPER | DIR_BOTH)    // Supervisor both Talk and Listen
#define DIR_LEAVE                            0x08    // Leave a group

//
// Group status values
//
#define GROUP_NONE                 0    // Group not in use
#define GROUP_ACTIVE               1    // Group is active
#define GROUP_HOLD                 2    // Group is on hold


/****  MTDSP signal route control: ***/
// network type:
#define  MTDSP_TDM_SLOT_PORT                    2
#define  MTDSP_CHANNEL_PORT                     0
#define  MTDSP_LINE_PORT                        3

// mode
#define  MTDSP_ROUTE_SET                        0
#define  MTDSP_ROUTE_RESET                      1



/****  PXD Control  : ****/
#define  ULAWPXD                                1
#define  ALAWPXD                                2


/* =============== Global Event Queue  ====================== */
#define MTDSP_GEVT_ENABLE               MT_GEVT_ENABLE
#define MTDSP_GEVT_DISABLE              MT_GEVT_DISABLE
#define MTDSP_GEVT_FLUSHQUEUE           MT_GEVT_FLUSHQUEUE

#define MT_GEVT_ENABLE                  1
#define MT_GEVT_DISABLE                 2
#define MT_GEVT_FLUSHQUEUE              3


/************ Voice format for record & play ***************/

#define MT_PCM_uLaw             0x100   // u-Law 8 KHz 8-bit PCM 64000 bps
#define MT_PCM_ALaw             0x110   // A-Law 8 KHz 8-bit PCM 64000 bps
#define MT_PCM_Linear_8bit      0x120   // Signed linear 8 KHz 8-bit PCM 64000 bps
#define MT_PCM_Linear_16bit     0x130   // Signed linear 8 KHz 16-bit PCM 128000 bps
#define MT_PCM_Linear_u8bit     0x140   // Unsigned linear 8 KHz 8-bit PCM 64000 bps
#define MT_PCM_Linear_u16bit    0x150   // Unsigend linear 8 KHz 16-bit PCM 128000 bps
#define MT_PCM_Linear6K_16bit   0x160   // Signed linear 6 KHz 16-bit PCM 96000 bps
#define MT_PCM_uLaw_Stereo      0x170   // u-Law 8 KHz 8-bit stereo PCM 128000 bps (record only)
#define MT_PCM_ALaw_Stereo      0x180   // A-Law 8 KHz 8-bit stereo PCM 128000 bps (record only)
#define MT_PCM_uLAW_POWER       0x190   // u-Law 8 KHz 8-bit with power tagged (record only)
#define MT_PCM_ALAW_POWER       0x1A0   // A-Law 8 KHz 8-bit with power tagged (record only)

// Old definition
#define MT_PCM                  MT_PCM_uLaw
#define MT_LINEAR               MT_PCM_Linear_16bit
#define MT_PCM_Raw_8bit         MT_PCM_Linear_8bit
#define MT_PCM_Raw_16bit        MT_PCM_Linear_16bit
#define MT_PCM_Raw_u8bit        MT_PCM_Linear_u8bit
#define MT_PCM_Raw_u16bit       MT_PCM_Linear_u16bit
#define MT_PCM_Raw6K_16bit      MT_PCM_Linear6K_16bit

#define MT_OKI_ADPCM_SR8        0x200   // Dialogic ADPCM 32000 bps, 8 KHz sample rate
#define MT_OKI_ADPCM_SR6        0x210   // Dialogic ADPCM 24000 bps, 6 KHz sample rate

// Format available only on boards enhanced with 5409 DSPs
#define MT_G729_8K              0x300   // G.729 8000 bps, Not supported 10/08/2002
#define MT_G729A_8K             0x310   // G.729A 8000 bps
#define MT_G723DOT1_5300_FIX    0x380   // G.723.1 5.3K bps fixed rate
#define MT_G723DOT1_6300_FIX    0x390   // G.723.1 6.3K bps fixed rate, not supported on NGX
#define MT_G723DOT1_5300_VAR    0x3A0   // G.723.1 5.3K bps variable rate, Not supported 06/20/2003
#define MT_G723DOT1_6300_VAR    0x3B0   // G.723.1 6.3K bps variable rate, Not supported 06/20/2003
#define MT_G723DOT1_5300_FIX_POWER  0x3C0  // G.723.1 5.3K bps fixed rate with power tagged (record only)

#define MT_GSM610               0x400   // GSM 6.10 13000 bps recorded into 33 bytes per 20ms
#define MT_MSGSM                0x410   // Microsoft GSM

#define MT_G726_16K             0x500   // G.726 16000 bps
#define MT_G726_16K_uLaw        0x510   // G.726 16000 bps for u-Law
#define MT_G726_16K_ALaw        0x520   // G.726 16000 bps for A-Law
#define MT_G726_24K             0x540   // G.726 24000 bps
#define MT_G726_24K_uLaw        0x550   // G.726 24000 bps for u-Law
#define MT_G726_24K_ALaw        0x560   // G.726 24000 bps for A-Law
#define MT_G726_32K             0x580   // G.726 32000 bps
#define MT_G726_32K_uLaw        0x590   // G.726 32000 bps for u-Law
#define MT_G726_32K_ALaw        0x5A0   // G.726 32000 bps for A-Law
#define MT_G726_40K             0x5C0   // G.726 40000 bps
#define MT_G726_40K_uLaw        0x5D0   // G.726 40000 bps for u-Law
#define MT_G726_40K_ALaw        0x5E0   // G.726 40000 bps for A-Law

#define MT_DATA8BIT_LSB_1ST     0x600   // Data 8 bit LSB first
#define MT_DATA8BIT_MSB_1ST     0x610   // Data 8 bit MSB first
#define MT_DATA16BIT_LSB_1ST    0x620   // Data 16 bit LSB first
#define MT_DATA16BIT_MSB_1ST    0x630   // Data 16 bit MSB first

#define MT_G726_16K_MSb_1ST             0x800   // G.726 16000 bps, MSb first
#define MT_G726_16K_uLaw_MSb_1ST        0x810   // G.726 16000 bps for u-Law, MSb first
#define MT_G726_16K_ALaw_MSb_1ST        0x820   // G.726 16000 bps for A-Law, MSb first
#define MT_G726_24K_MSb_1ST             0x840   // G.726 24000 bps, MSb first
#define MT_G726_24K_uLaw_MSb_1ST        0x850   // G.726 24000 bps for u-Law, MSb first
#define MT_G726_24K_ALaw_MSb_1ST        0x860   // G.726 24000 bps for A-Law, MSb first
#define MT_G726_32K_MSb_1ST             0x880   // G.726 32000 bps, MSb first
#define MT_G726_32K_uLaw_MSb_1ST        0x890   // G.726 32000 bps for u-Law, MSb first
#define MT_G726_32K_ALaw_MSb_1ST        0x8A0   // G.726 32000 bps for A-Law, MSb first
#define MT_G726_40K_MSb_1ST             0x8C0   // G.726 40000 bps, MSb first
#define MT_G726_40K_uLaw_MSb_1ST        0x8D0   // G.726 40000 bps for u-Law, MSb first
#define MT_G726_40K_ALaw_MSb_1ST        0x8E0   // G.726 40000 bps for A-Law, MSb first


// The following formats are for variable sized frame, where a 2-byte header of
// frame size will be inserted in front of each frame

#define MT_HDLC                 0x1000  // HDLC frame (variable sized)



// WAV format supported
//  wave formst is supported through setting MT_IO_CONTROL.StartControl field the following flags:
//      WAVE_RIFFFMTDATA_CHUNK  to add wave header of RIFF, Fmt, and data chunk descriptors
//      WAVE_DATACHUNK_ONLY     to add data chunk descriptor only
//  For example, Wave linear is configured by selecting format MT_PCM_Raw_16bit with
//  WAVE_RIFFFMTDATA_CHUNK flag in MT_IO_CONTROL.StartControl field set
#if 0   // Obsoleted
#define MT_WAV_LINEAR           0x700   // Recording WAV format- 8000 sampled rate and 16 bits, format=MT_PCM_Raw_16bit
#define MT_WAV_ULAWPCM          0x710   // Recording WAV format- 8000 sampled rate and 8 bits, format=MT_PCM_uLaw
#define MT_WAV_ALAWPCM          0x720   // Recording WAV format- 8000 sampled rate and 8 bits, format=MT_PCM_ALaw
#define MT_WAV                  MT_WAV_LINEAR   // Playback for all WAV format
#define MT_WAV_PCM              MT_WAV_ULAWPCM
#endif

// Byte size of Frame for supported format
#define VS_PCM                  160     // Format MT_PCM_uLaw, MT_PCM_ALaw, 20 ms frame
#define VS_PCM_POWER            162     // Format MT_PCM_uLAW_POWER, MT_PCM_ALAW_POWER, 20ms frame
#define VS_LINEAR               320     // Format MT_LINEAR, 20 ms frame
#define VS_LINEAR_6K            240     // Format MT_PCM_Raw6K_16bit, 20 ms frame
#define VS_OKI_ADPCM_SR8        80      // Format MT_OKI_ADPCM_SR8, 20 ms frame
#define VS_OKI_ADPCM_SR6        60      // Format MT_OKI_ADPCM_SR6, 20 ms frame

#define VS_G729_8K              20      // Format MT_G729A_8K, MT_G729_8K, 20 ms frame
#define VS_G723DOT1_5300        20      // Format MT_G723DOT1_5300_FIX, 30 ms frame
#define VS_G723DOT1_6300        24      // Format MT_G723DOT1_6300_FIX, 30 ms frame
#define VS_G723DOT1_5300_POWER  22      // Format MT_G723DOT1_5300_FIX_POWER, 30ms frame

#define VS_GSM610               33      // Format MT_GSM610, 20 ms frame
#define VS_MSGSM                65      // Format MT_MSGSM, 40 ms frame

#define VS_G726_16K             40      // Format MT_G726_16K, 20 ms frame
#define VS_G726_24K             60      // Format MT_G726_24K, 20 ms frame
#define VS_G726_32K             80      // Format MT_G726_32K, 20 ms frame
#define VS_G726_40K             100     // Format MT_G726_40K, 20 ms frame

#define VS_DATA8BIT             160     // Format MT_DATA8BIT_L(M)SB_1ST, 20 ms frame
#define VS_DATA16BIT            320     // Format MT_DATA16BIT_L(M)SB_1ST, 20 ms frame

#define VS_HDLC_MAX             278     // Max frame for variable sized frame format HDLC


// Define a WAVE HEADER structure appropriate for GSM610 style data.
// The information used to generate this header has come from MS
// Developers Connection documentation and examination of header
// structure in assorted GSM WAV files.
#if _WIN32
#include "mmreg.h"      // Include multi-media header
#elif LINUX
#include "MS_Wave.h"
#endif

typedef struct {
    CHAR    RIFF[4];                // 'R','I','F','F'
    DWORD   Size;                   // size of wave file from here on
    CHAR    WAVEfmt[8];             // 'W','A','V','E','f','m','t',' '
    DWORD   FormatSize;             // size of GSM610WAVEFORMAT info (20 for MSGSM)
    GSM610WAVEFORMAT gsm;           // gsm type data
    CHAR    Fact[4];                // fact section
    DWORD   FactSize;               // size of the fact data field
    DWORD   nSamples;               // more fact data
    CHAR    Data[4];                // 'd','a','t','a'
    DWORD   DataSize;               // size of the 'data' chunk
} GSM_WAVE_HEADER, *PGSM_WAVE_HEADER;


typedef struct {
    CHAR    RIFF[4];                // 'R','I','F','F'
    DWORD   Size;                   // size of wave file from here on
    CHAR    WAVEfmt[8];             // 'W','A','V','E','f','m','t',' '
    DWORD   FormatSize;             // size of PCMWAVEFORMAT info (16 for PCM)
    PCMWAVEFORMAT pcm;              // PCM type data
    CHAR    Data[4];                // 'd','a','t','a'
    DWORD   DataSize;               // size of the 'data' chunk
} PCM_WAVE_HEADER, *PPCM_WAVE_HEADER;



/*** MTFAX format ***/
#define         MTFAX_RAW               0x0
#define         MTFAX_TIFF              0x1

/*** MTFAX length : ***/
#define         MTFAX_297mm             0x1
#define         MTFAX_UNLIMITED         0x2
#define         MTFAX_NONE              0x0

/*** MTFAX width : ***/
#define         MTFAX_1728              0x1
#define         MTFAX_2048              0x2
#define         MTFAX_2432              0x3

/*** MTFAX resln :  ***/
#define         MTFAX_RESLN_LO          0x1
#define         MTFAX_RESLN_HI          0x2


/*** MTFAX parameter names for MTSetParam()  ***/
#define         MTFAX_LOCALID   0x10
#define         MTFAX_REMOTEID  0x11

/*** MTFAX Attribute Request Defines ***/
#define         MTFAX_BAUDRT            0x3     /*  Fax transfer speed          */
#define         MTFAX_WIDTH             0x4     /*      Negotiated width if phase B done */
#define         MTFAX_RESLN             0x5     /*  Negotiated resolution if phase B done */
#define         MTFAX_PGXFER            0x6     /*      number of pages transferred */
#define         MTFAX_STATUS            0x8
#define         MTFAX_TIFBADTAG         0xB
#define         MTFAX_TIFMSTAG          0xC

/*** MTFAX status values  for Attribute MTFAX_STATUS ***/
#define         MTFAX_IDLE              0
#define         MTFAX_SENDING           1
#define         MTFAX_RCVING            2

/*** MTFAX Resolution status values  for Attribute MTFAX_RESLN ***/
#define         MTFAX_HIGH_RESL         0
#define         MTFAX_LOW_RESL          8


/***  MTFAX Phase B Status Values ***/

#define         MTFAX_BSTS_NONE       0x0     /* no status, Phase B not done */
#define         MTFAX_BSTS_REMOTEID   0x1     /* Remote Id available on Phase B */


/*** MTFAX Phase D Commands ***/
#define  MTFAX_DSTS_EOP         0x0    /* end of procedure */
#define  MTFAX_DSTS_MPS         0x1    /* end of page */
#define  MTFAX_DSTS_EOM         0x2    /* end of message */
#define  MTFAX_DSTS_POLL        0x3    /* poll request */
#define  MTFAX_DSTS_PRI_EOP     0x10   /* request for operator intervention (PRI_EOP) sent */
#define  MTFAX_DSTS_PRI_MPS     0x11   /* request for operator intervention (PRI_MPS) sent */
#define  MTFAX_DSTS_PRI_EOM     0x12   /* request for operator intervention (PRI_EOM) sent */

/*** FAX Phase D Replies ***/
#define  MTFAX_DRPLY_MCF         0x1    /* message confirmation - do next page */
#define  MTFAX_DRPLY_RTN         0x2    /* retrain negative */
#define  MTFAX_DRPLY_RTP         0x3    /* retrain positive */
#define  MTFAX_DRPLY_PIP         0x4    /* procedure interrupt positive */
#define  MTFAX_DRPLY_PIN         0x5    /* procedure interrupt negative */

//
// Audio payload type definitions for IP Products.  
//	 Note: These payload type definitions are in accordance with IETF RFC 3551 (July 2003).
//
#define MT_RTP_PCMU             0       // ITU-T G.711 PCMU
#define MT_RTP_1016             1       // USA Federal Standard FS-1016
#define MT_RTP_G721             2       // ITU-T G.721
#define MT_RTP_GSM              3       // GSM 06.10
#define MT_RTP_G723             4       // ITU-T G.723
#define MT_RTP_DVI4_8000        5       // DVI4 8000 samples/s
#define MT_RTP_DVI4_16000       6       // DVI4 16000 samples/s
#define MT_RTP_LPC              7       // LPC
#define MT_RTP_PCMA             8       // ITU-T G.711 PCMA
#define MT_RTP_G722             9       // ITU-T G.722
#define MT_RTP_L16_STEREO       10      // 16-bit uncompressed audio, stereo
#define MT_RTP_L16_MONO         11      // 16-bit uncompressed audio, monaural
#define MT_RTP_MPA              14      // MPEG-I/II Audio
#define MT_RTP_G728             15      // MPEG-I/II Audio
#define MT_RTP_G729             18      // ITU-T G.729
#define MT_RTP_CELB             25      // Sun CELL-B
#define MT_RTP_JPEG             26      // JPEG
#define MT_RTP_NV               28      // 'nv' program
#define MT_RTP_H261             31      // ITU-T H.261
#define MT_RTP_MPV              32      // MPEG-I/II Video
#define MT_RTP_MP2T             33      // MPEG-II transport streams
#define MT_RTP_H263             34      // ITU-T H.263

// Operation definitions for StopCurrentFunction
#define  MT_ENCODE              1
#define  MT_DECODE              2
#define  MT_COLLECT_DTMF        3
#define  MT_RECORD              MT_ENCODE
#define  MT_PLAY                MT_DECODE

// Action definitions for Streaming functions
#define  MT_STREAM_OUTPUT       1
#define  MT_STREAM_INPUT        2

// StopMode denifition for Streaming functions
#define MT_STREAM_STOP_NOW      0       // Stop streaming and flush the buffer
#define MT_STREAM_STOP_DELAY    1       // Stop streaming yet keep the buffer till retrieval/palyback done

// Mode definitions for WAV Stream functions
#define ASYNC_MODE              0       // For WAV driver such as TAPI who handles WAV with header stripped
#define SYNC_MODE               1       // For driver other than TAPI


#if 0

// Action definitions for WAV Stream functions
#define WAV_PLAY                1
#define WAV_RECORD              2
#define WAV_PLAY_STOP           3

// StopMode denifition for Streaming functions
#define WAV_STOP_NOW            0       // Kill the current and cancel all pending streaming functions
#define WAV_STOP_DELAY          1       // Stop after the pending streaming functions are completed


//
// Station Master structures
//
typedef struct {
        USHORT  cadence1_on;
        USHORT  cadence1_off;
        USHORT  cadence2_on;
        USHORT  cadence2_off;
        USHORT  freq1;
        USHORT  freq2;
        USHORT  volume;
        USHORT  cycles;
} MTSM_TONE;

// MTSM tone type:
#define MTSM_POWER_RING1                0
#define MTSM_POWER_RING2                1
#define MTSM_BUSY1                      2
#define MTSM_BUSY2                      3
#define MTSM_RINGBACK1                  4
#define MTSM_RINGBACK2                  5
#define MTSM_DIAL                       6
#define MTSM_USER1                      7
#define MTSM_USER2                      8

// MTSM Parameters:
#define MTSM_FLASH                      1
#define MTSM_DTMF                       2
#define MTSM_LOOP                       3
#define MTSM_STATION                    4

#endif

//
// Configurable System Parameter structure
//
typedef struct _MTSYS_CONFIGURATION {

    // General system parameters
    ULONG   SDKCompiledWith;        // SDK version the application is compiled with,
                                    // default 0x02070000, i.e. ver 2.7.0.0
    ULONG   APILogCount;            // Max. number of event-log entry allowed, default 100,
                                    // 0 for no limitation
    ULONG   SDKLogLevel;            // [not supported yet]
    ULONG   APITraceLevel;          // [not supported yet]

    ULONG   SDKChannel;             // Max. number of channel allowed, default MAX_CH_DEVICES

    UCHAR   GCIStartingIndex;       // 0 or 1, default 0
    UCHAR   HaltOnStartupError;     // [0 or 1, not supported yet]
    UCHAR   FlagDataTruncated;      // 0 or 1 for APIs to return MT_RET_DATA_TRUNCATED,
                                    // default 0
    UCHAR   SteppedRun;             // [Not implemented yet]
    UCHAR   SysRes[4];              // Reserved for future use

    // CT bus parameters
    UCHAR   H100StreamSpeed;        // H100 stream speed of CT_2MHz, CT_4MHz, or CT_8MHz
    UCHAR   BusSegmentation;        // 0 for no bus segmentation supported
    USHORT  MVIPStartSlot;          // Starting MVIP timeslot: 0 to 511
    USHORT  MVIPTotalSlot;          // Max. number of MVIP slot allowed: 0 to 512

    UCHAR   SysRes2[2];             // Reserved for future use

} MTSYS_CONFIGURATION, *PMTSYS_CONFIGURATION;


#define MAX_DC                      3
#define MAX_TRUNKS                  4
#define MAX_IP_NET_PORTS            3

//
// Flags for APITraceLevel
//
#define API_TRACE_ERROR_RET         0x00000001  // Trace return code != (
                                                // MT_RET_OK or MT_RET_IO_PENDING)
#define API_TRACE_CALLBACK          0x00000002  // Trace callback only
#define API_TRACE_ALLAPI            0x00000004  // Trace all API calls
#define API_TRACE_ALL               0xFFFFFFFF  // All trace on

// LT board offhook impedance
#define FCC_600                     0
#define ETSI_270                    1
#define AUSTRALIA_220               2
#define CHINA_200                   3


// Possible PBX types
#define PBXTYPE_AASTRA_INTELLIGATE			0x1B01	// Aastra (Ascotel) IntelliGate 150/300/2025/2045/2065
#define PBXTYPE_ALCATEL_4095                0x0703  // Alcatel 4095 (analog phone)
#define PBXTYPE_ALCATEL_4200_4400           0x0701  // Alcatel 4200/4400
#define PBXTYPE_ASPECT                      0x0601  // Aspect
#define PBXTYPE_AVAYA_DEFINITY_2W           0x0201  // Avaya Definity 2W
#define PBXTYPE_AVAYA_DEFINITY_4W           0x0202  // Avaya Definity 4W
#define PBXTYPE_AVAYA_INDEX                 0x0204  // Avaya Index
#define PBXTYPE_AVAYA_MERLIN_MAGIX          0x0203  // Avaya Merlin Magix
#define PBXTYPE_AVAYA_MLX                   0x0205  // Avaya MLX
#define PBXTYPE_AVAYA_DEFINITY_CONSOLE302   0x0206  // Avaya Console 302
#define PBXTYPE_BOSCH_2W                    0x0801  // Bosch Integral 2W
#define PBXTYPE_BOSCH_4W                    0x0802  // Bosch Integral 4W
#define PBXTYPE_BOSCH_INTEGRAL_55           0x0803  // Bosch Integral 55
#define PBXTYPE_BOSCH_INTEGRAL_3            0x0804  // Bosch Integral3 4W  
#define PBXTYPE_BRI_ISDN                    0x1101  // BRI ISDN
#define PBXTYPE_BRI_ISDN_NI1                0x1102  // BRI ISDN National 1
#define PBXTYPE_EON_EQ                      0x1801  // EON EQ
#define PBXTYPE_ERICSSON_ELU25              0x0401  // Ericsson ELU25
#define PBXTYPE_ERICSSON_ELU5               0x0402  // Ericsson ELU5
#define PBXTYPE_ERICSSON_ELUD3              0x0403  // Ericsson MFC and ELU-D3
#define PBXTYPE_ESI_100                     0x1D01  // ESI-100 Estech Systems
#define PBXTYPE_FUJITSU_F9600               0x1401  // Fujitsu F9600
#define PBXTYPE_INTECOM_E2W                 0x0A01  // Intecom EADS 2W
#define PBXTYPE_INTECOM_E4W                 0x0A02  // Intecom EADS 4W
#define PBXTYPE_INTER_TEL                   0x0b01  // Inter-Tel
#define PBXTYPE_LG_STAREX_CS                0x1701  // LG Starex CS
#define PBXTYPE_LG_LDK_50                   0x1702	  // LG LDK-50
#define PBXTYPE_MITEL_SX2000                0x1001  // Mitel SX2000
#define PBXTYPE_NEC                         0x0304  // NEC
#define PBXTYPE_NORTEL_MATRA                0x0103  // Nortel Matra
#define PBXTYPE_NORTEL_MERIDIAN1            0x0101  // Nortel Meridian1 (default)
#define PBXTYPE_NORTEL_MERIDIAN1_CONSOLE    0x0104  // Nortel Merdian1 Console
#define PBXTYPE_NORTEL_NORSTAR              0x0102  // Nortel Norstar
#define PBXTYPE_PANASONIC_7600_SPECIAL      0x0F02  // Panasonic 7600 Special
#define PBXTYPE_PANASONIC_KX                0x0F01  // Panasonic KX
#define PBXTYPE_PHILIPS_SOPHO_IS3000_2W     0x1301  // Philips SOPHO iS3000 2W
#define PBXTYPE_PHILIPS_SOPHO_IS3000_4W     0x1302  // Philips SOPHO iS3000 4W
#define PBXTYPE_PHILIPS_SOPHO_IS3000_4W_TMP 0x1303  // Philips SOPHO iS3000 4W TMP Phones
#define PBXTYPE_ROCKWELL_SPECTRUM           0x1501  // Rockwell Spectrum
#define PBXTYPE_SAMSUNG_DCS828              0x1201  // Samsung DCS828
#define PBXTYPE_SAMSUNG_INFOREX             0x1202  // Samsung Inforex
#define PBXTYPE_SIEMENS_HICOM               0x0501  // Siemens Hicom
#define PBXTYPE_SIEMENS_REALITIS_DTI_4W     0x0503  // Siemens Realitis DTI 4W
#define PBXTYPE_SIEMENS_REALITIS_ISDT_2W    0x0504  // Siemens Realitis iSDT 2W
#define PBXTYPE_SIEMENS_ROLMLINK            0x0502  // Siemens RolmLINK
#define PBXTYPE_TADIRAN_CORAL               0x1901  // Tadiran Coral and Coral IPx Office
#define PBXTYPE_TELTRONICS_HARRIS2020       0x1A01  // Teltronics Harris 2020
#define PBXTYPE_TOSHIBA_STRATA_DK           0x1601  // Toshiba Strata DK
#define PBXTYPE_AASTRA_INTELLIGATE			0x1B01  // AASTRA_INTELLIGATE(Ascotel)
#define PBXTYPE_ESI_100						0x1D01  // ESI_100
#define PBXTYPE_POLYCOM_550D				0x1E01  // POLYCOM_550D


// Possible Framer Loopback Mode DT and DP cards, 12/03
#define FRAMER_LOOPBACK_NONE                0x0000  // No loopback, default
#define FRAMER_SYSTEM_DIGITAL_LOOPBACK      0x0001  // System diagnostic digital loopback
#define FRAMER_NETWORK_LINE_LOOPBACK        0x0002  // Network line loopback
#define FRAMER_NETWORK_PAYLOAD_LOOPBACK     0x0003  // Network payload loopback

//
// Configurable PCM Trunk Signal structure
//
typedef struct _MTPCM_SIGNAL_POLARITY {
    BOOL    TxSignal;                 // false for Normal, true for invert
    BOOL    RxSignal;                 // false for Normal, true for invert
    BOOL    FrameSyncSignal;          // false for Normal, true for invert
} MTBOARD_PCM_SIGNAL_POLARITY, * PMTBOARD_PCM_SIGNAL_POLARITY ;


///////////////////////////////////////////////////////////////////////////////
// License Information
///////////////////////////////////////////////////////////////////////////////
#define MT_MAX_SESSIONS                  1
#define MT_MAX_FORWARDING_MEDIA_SESSIONS MT_MAX_SESSIONS
#define MT_MAX_MONITORING_STATIONS       2
#define MT_MAX_MONITORING_CALLS          3

#define LIC_STRING_LEN 20
#define LIC_LONG_STRING_LEN 256
#define LICENSE_KEY_SIZE 2200

///////////////////////////////////////////////////////////////////////////////
// IP Network 
///////////////////////////////////////////////////////////////////////////////

//
// IP Protocol Definitions
//
#define MT_DHCP                  0  // TCP/IP Configuration via DHCP
#define MT_STATIC                1  // TCP/IP Configuration is Static

#define MT_PASSIVE_VLAN_DISABLE  0  // Passive VLAN disabled
#define MT_PASSIVE_VLAN_ENABLE   1  // Passive VLAN enabled

#define MT_NAT_TOPOLOGY_DISABLE  0  // NAT is not supported on Network
#define MT_NAT_TOPOLOGY_ENABLE   1  // NAT is supported on Network

#define MT_TCP                   6  // TCP Protocol
#define MT_UDP                   17 // UDP Protocol

//
// Protocol ID's
//
#define MT_IP_UNASSIGNED         0  // Reserved 
#define MT_IP_CISCO_SKINNY       1  // Cisco SCCP Protocol
#define MT_IP_AVAYA_H323         2  // Avaya H323 Protocol
#define MT_IP_H323               3  // H323 Protocol
#define MT_IP_NORTEL_UNISTIM     4  // Nortel UNIStim Protocol
#define MT_IP_ERICSSON_H323      5  // Ericsson H323 Protocol
#define MT_IP_SIP                6  // Session Initiation Protocol
#define MT_IP_SIEMENS            7  // Siemens Protocol
#define MT_IP_ALCATEL            8  // Alcatel Protocol
#define MT_IP_NEC_PROTIMS        9  // NEC PROTIMS Protocol
#define MT_IP_INTERTEL           10 // Intertel Protocol
#define MT_IP_SHORETEL_MGCP      11 // ShoreTel Mgcp Protocol
#define MT_IP_AASTRA_MATRA       12 // Aastra Matra Protocol
#define MT_IP_MAX_PROTOCOL       12 // Maximum protocol ID

#define MT_IP_MAX_MEDIA_ROUTING_RULES 10
#define MT_IP_MAX_NON_STATION_LIST 25
#define MT_IP_MAX_ADDTIONAL_PROTO_PORT_LIST 20
#define MT_IP_MAX_H323_PROTO_PORT_LIST MT_IP_MAX_ADDTIONAL_PROTO_PORT_LIST
#define MT_IP_MAX_SIP_PROTO_PORT_LIST MT_IP_MAX_ADDTIONAL_PROTO_PORT_LIST

typedef enum
{
    ALL,             // All media sessions
    SOURCE_NET,      // Media sessions where source IP address equals mask
    CODEC            // Media sessions where codec type equals mask
} IP_FILTER_OPTS, *PIP_FILTER_OPTS;

typedef enum
{
    NETWORK,         // Media session destination is a network device
    LOCAL_CHANNEL,   // Media session destination is an on-board channel
    HOST             // Media session destination is the host
} IP_TARGET_OPTS, *PIP_TARGET_OPTS;

//
// Ethernet configuration options
//
typedef struct _MT_IPCONFIG 
{
    ULONG IPAddress;           // IP Address for this port
    ULONG SubNetMask;          // SubNet Mask for this port
    ULONG DefaultGatewayAddr;  // IP address of default gateway
} MT_IPCONFIG, *PMT_IPCONFIG;


//
// Packet Filter Parameters
//
typedef struct _MT_IPFILTER 
{
    ULONG   flags;             // Bit wise option flags
    ULONG   BeginSrcIPAddr;    // First source IP address (if using range)
    ULONG   EndSrcIPAddr;      // Last source IP address or NULL
    ULONG   BeginDestIPAddr;   // First destination IP address (if using range)
    ULONG   EndDestIPAddr;     // Last destination IP address or NULL
    ULONG   IPProtocol;        // Numeric protocol value or one of: TCP, UDP, ICMP
    USHORT  BeginSrcTCPPort;   // First source TCP port (if using range)
    USHORT  EndSrcTCPPort;     // Last source TCP port or NULL
    USHORT  BeginDestTCPPort;  // First destination TCP port (if using range)
    USHORT  EndDestTCPPort;    // Last destination TCP port or NULL
    USHORT  BeginSrcUDPPort;   // First source UDP port (if using range)
    USHORT  EndSrcUDPPort;     // Last source UDP port or NULL
    USHORT  BeginDestUDPPort;  // First destination UDP port (if using range)
    USHORT  EndDestUDPPort;    // Last destination UDP port or NULL
    ULONG   BeginSrcMACAddr;   // First source MAC address (if using range)
    ULONG   EndSrcMACAddr;     // Last source MAC address or NULL
} MT_IPFILTER, *PMT_IPFILTER;

// IP Signaling Protocol Params
//   Note: These are protocol dependant data structures used to configure 
//         a signaling protocol.
//
typedef struct
{
    UCHAR           IPPROTO;    // Protocol type (i.e., MT_TCP or MT_UDP)
    USHORT          Port;       // Port Number
}MT_IP_PROTO_PORT;

typedef struct
{
    MT_IP_PROTO_PORT SCCP;      // CISCO "Skinny Client Control Protocol" parameters
}MT_SCCP_PARAMS;

typedef struct
{
    MT_IP_PROTO_PORT H225CS;    // H225 Call Signaling IP Protocol Type and Port
    MT_IP_PROTO_PORT H225RAS;   // H225 Registration Admission and Status IP Protocol Type and Port
    USHORT NonStationListCount;
    ULONG NonStationList [MT_IP_MAX_NON_STATION_LIST];
    USHORT H225CSAdditionalCount;
    MT_IP_PROTO_PORT H225CS_Additional[MT_IP_MAX_H323_PROTO_PORT_LIST];  // Optional H225 Call Signaling ports.
    USHORT H225RASAdditionalCount;
    MT_IP_PROTO_PORT H225RAS_Additional[MT_IP_MAX_H323_PROTO_PORT_LIST]; // Optional H223 Registration ports.
}MT_H323_PARAMS;

typedef struct
{
    MT_IP_PROTO_PORT LTPS;    // Nortel UNIStim "Line Terminal Proxy Server" parameters
    MT_IP_PROTO_PORT IPPhone; // Nortel IP Phone parameters
} MT_NORTEL_UNISTIM_PARAMS;

typedef struct
{
    MT_IP_PROTO_PORT H225CS;   // H225 Call Signaling IP Protocol Type and Port
    MT_IP_PROTO_PORT H225RAS;  // H225 Registration Admission and Status IP Protocol Type and Port
    USHORT NonStationListCount;
    ULONG NonStationList [MT_IP_MAX_NON_STATION_LIST];
    USHORT H225CSAdditionalCount;
    MT_IP_PROTO_PORT H225CS_Additional[MT_IP_MAX_H323_PROTO_PORT_LIST];  // Optional H225 Call Signaling ports.
    USHORT H225RASAdditionalCount;
    MT_IP_PROTO_PORT H225RAS_Additional[MT_IP_MAX_H323_PROTO_PORT_LIST]; // Optional H223 Registration ports.
    // Ericsson specific parameters, TBD
} MT_ERICSSON_H323_PARAMS;

typedef struct
{
    MT_IP_PROTO_PORT Transport;  // SIP IP Protocol Type and Port
    ULONG ProxyIPAddress;        // SIP Proxy/ALG/PBX IP Address
    USHORT NonStationListCount;
    ULONG NonStationList [MT_IP_MAX_NON_STATION_LIST];
    USHORT TransportAdditionalCount;
    MT_IP_PROTO_PORT Transport_Additional[MT_IP_MAX_SIP_PROTO_PORT_LIST];  // Optional SIP IP protocol type and ports.
} MT_SIP_PARAMS;

typedef struct
{
    MT_IP_PROTO_PORT Proprietary; // Siemens Proprietary Signaling IP Protocol Type and Port
    MT_IP_PROTO_PORT H225CS;      // H225 Call Signaling IP Protocol Type and Port
} MT_SIEMENS_PARAMS;

typedef struct
{
    MT_IP_PROTO_PORT Transport; // Alcatel IP Protocol Type and Port
} MT_ALCATEL_PARAMS;

typedef struct
{
    MT_IP_PROTO_PORT Transport;         // NEC PROTIMS D-Channel IP Protocol Type and Port
    MT_IP_PROTO_PORT MediaSignaling;    // NEC PROTIMS Media Signaling IP Protocol Type and Port
} MT_NEC_PROTIMS_PARAMS;

typedef struct
{
    MT_IP_PROTO_PORT Transport;    // Intertel IP Protocol Type and Port
    MT_IP_PROTO_PORT Registration; // Intertel Phone's Registration IP Protocol Type and Port
} MT_INTERTEL_PARAMS;

typedef struct
{
    MT_IP_PROTO_PORT CallAgent;    // ShoreTel MGCP Call agent IP Protocol Type and Port
    MT_IP_PROTO_PORT Gateway;      // ShoreTel MGCP Gateway IP Protocol Type and Port
} MT_SHORETEL_MGCP_PARAMS;

typedef struct
{
    MT_IP_PROTO_PORT Transport;    // Aastra Matra Transport IP Protocol Type and Port
    MT_IP_PROTO_PORT Registration; // Aastra Matra Registration IP Protocol Type and Port
} MT_AASTRA_MATRA_PARAMS;

typedef struct
{
    union
    {
     MT_SCCP_PARAMS CISCO;                    // CISCO SCCP protocol parameters
     MT_H323_PARAMS H323;                     // H323 protocol parameters
     MT_H323_PARAMS Avaya_H323;               // Avaya H323 protocol parameters
     MT_NORTEL_UNISTIM_PARAMS Nortel_Unistim; // Nortel UNIStim parameters
     MT_ERICSSON_H323_PARAMS Ericsson_H323;   // Ericsson H323 protocol parameters
     MT_SIP_PARAMS SIP;                       // SIP parameters
     MT_SIEMENS_PARAMS Siemens;               // Siemens protocol parameters
     MT_ALCATEL_PARAMS Alcatel;               // Alcatel Protocol parameters
     MT_NEC_PROTIMS_PARAMS NEC_Protims;       // NEC PROTIMS parameters
     MT_INTERTEL_PARAMS Intertel;             // Intertel parameters
     MT_SHORETEL_MGCP_PARAMS ShoreTelMgcp;    // ShoreTel Mgcp parameters
     MT_AASTRA_MATRA_PARAMS AastraMatra;      // Aastra Matra parameters

    };

}MT_IP_PROTOCOL_PARAMS, *PMT_IP_PROTOCOL_PARAMS;

//
// Station Parameters
//
typedef struct
{
    ULONG          StationID;       // Station ID
    USHORT         ProtocolID;      // Protocol ID
} MT_IP_STATION, *PMT_IP_STATION;

//
// IP Station Information is protocol dependant.  The data structures below
// represent the current known set of station parameters for each of the 
// supported protocols.
//
typedef struct
{
    MT_IP_PROTO_PORT SCCP;          // CISCO "Skinny Client Control Protocol" parameters
}MT_SCCP_STATION;

typedef struct
{
   MT_IP_PROTO_PORT H225CS;         // H225 Call Signaling IP Protocol Type and Port
   MT_IP_PROTO_PORT H225RAS;        // H225 Registration Admission and Status IP Protocol Type and Port
}MT_H323_STATION;

typedef struct
{
    MT_IP_PROTO_PORT LTPS;          // Nortel UNIStim "Line Terminal Proxy Server" parameters
    MT_IP_PROTO_PORT IPPhone;       // Nortel IP Phone parameters
}MT_NORTEL_UNISTIM_STATION;

typedef struct
{
   MT_IP_PROTO_PORT H225CS;         // H225 Call Signaling IP Protocol Type and Port
   MT_IP_PROTO_PORT H225RAS;        // H225 Registration Admission and Status IP Protocol Type and Port
   // Ericsson specific parameters, TBD
}MT_ERICSSON_STATION;

typedef struct
{
    MT_IP_PROTO_PORT Transport;     // SIP IP Protocol Type and Port
}MT_SIP_STATION;

typedef struct
{
    MT_IP_PROTO_PORT Proprietary;   // Siemens IP Protocol Type and Port
    MT_IP_PROTO_PORT H225CS;        // H225 Call Signaling IP Protocol Type and Port
}MT_SIEMENS_STATION;


typedef struct
{
    MT_IP_PROTO_PORT Transport;   // Alcatel IP Protocol Type and Port
}MT_ALCATEL_STATION;

typedef struct
{
    MT_IP_PROTO_PORT Transport;         // NEC PROTIMS D-Channel IP Protocol Type and Port
    MT_IP_PROTO_PORT MediaSignaling;    // NEC PROTIMS Media Signaling IP Protocol Type and Port
}MT_NEC_PROTIMS_STATION;

typedef struct
{
    MT_IP_PROTO_PORT Transport;   // Intertel IP Protocol Type and Port
    MT_IP_PROTO_PORT Registration; // Phone's Registration Protocol Type and Port
}MT_INTERTEL_STATION;

typedef struct
{
    MT_IP_PROTO_PORT CallAgent;    // ShoreTel MGCP Call agent IP Protocol Type and Port
    MT_IP_PROTO_PORT Gateway;      // ShoreTel MGCP Gateway IP Protocol Type and Port
}MT_SHORETEL_MGCP_STATION;

typedef struct
{
    MT_IP_PROTO_PORT Transport;    // Aastra Matra Transport IP Protocol Type and Port
    MT_IP_PROTO_PORT Registration; // Aastra Matra Registration IP Protocol Type and Port
} MT_AASTRA_MATRA_STATION;

typedef struct
{
    USHORT          ProtocolID;    // Protocol ID 
    ULONG           IpAddress;     // IP Address
    union
    {
        MT_SCCP_STATION CISCOStation;
        MT_H323_STATION H323Station;
        MT_H323_STATION AvayaH323Station;
        MT_NORTEL_UNISTIM_STATION NortelUnistimStation;
        MT_ERICSSON_STATION EricssonH323Station;
        MT_SIP_STATION SIPStation;
        MT_SIEMENS_STATION SiemensStation;
        MT_ALCATEL_STATION AlcatelStation;
        MT_NEC_PROTIMS_STATION NECProtimsStation;
        MT_INTERTEL_STATION IntertelStation;
		MT_SHORETEL_MGCP_STATION ShoreTelMgcpStation;
        MT_AASTRA_MATRA_STATION AastraMatraStation;
    };
}MT_IP_STATION_PARAMS, *PMT_IP_STATION_PARAMS;

//
// IP Media Session Params
//
typedef struct
{
    ULONG   SessionID;                 // Session ID
    ULONG   PrimaryStationIPAddress;   // IP address of primary station
    ULONG   SecondaryStationIPAddress; // IP address of secondary station
    USHORT  PrimaryStationUDPPort;     // UDP port of primary station
    USHORT  SecondaryStationUDPPort;   // UDP port of secondary station
    union
    {
      ULONG   Codec;                   // Codec used for backward compability, being obsoleted
      ULONG   PrimaryCodec;            // Primary Codec used for the conversation
    };
    ULONG   SecondaryCodec;            // Secondary Codec used for the conversation
    ULONG   CallRef;                   // Call reference
} MT_IP_SESSION_PARAMS, *PMT_IP_SESSION_PARAMS;

//
// IP Forwarding Criteria Element
//
typedef enum
{
    ROUTE_BY_NONE    = 0,
    ROUTE_BY_SESSION = 1,
    ROUTE_BY_STATION = 2,
    ROUTE_BY_IP      = 3
} MT_IP_MEDIA_ROUTING_TYPE, *PMT_IP_MEDIA_ROUTING_TYPE;

typedef struct
{
    ULONG   MinSession;                 // Starting session number
    ULONG   MaxSession;                 // Ending session number
    ULONG   DestPrimaryStationIPAddr;   // Destination IP address for primary station 
    USHORT  DestPrimaryIncOffset;       // Destination primary station incremental offset
    USHORT  DestPrimaryStationBaseID;   // Starting primary station resource (UDP Port, Channel)
    ULONG   DestSecondaryStationIPAddr; // Destination IP address for secondary station
    USHORT  DestSecondaryIncOffset;     // Destination secondary station incremental offset
    USHORT  DestSecondaryStationBaseID; // Starting secondary station resource (UDP Port, Channel)
} MT_IP_ROUTE_BY_SESSION_PARAMS, *PMT_IP_ROUTE_BY_SESSION_PARAMS;

typedef struct
{
    ULONG   MinStation;                 // Starting station number
    ULONG   MaxStation;                 // Ending station number
    ULONG   DestPrimaryStationIPAddr;   // Destination IP address for primary station 
    USHORT  DestPrimaryIncOffset;       // Destination primary station incremental offset
    USHORT  DestPrimaryStationBaseID;   // Starting primary station resource (UDP Port, Channel)
    ULONG   DestSecondaryStationIPAddr; // Destination IP address for secondary station
    USHORT  DestSecondaryIncOffset;     // Destination secondary station incremental offset
    USHORT  DestSecondaryStationBaseID; // Starting secondary station resource (UDP Port, Channel)
} MT_IP_ROUTE_BY_STATION_PARAMS, *PMT_IP_ROUTE_BY_STATION_PARAMS;

typedef struct
{
    ULONG   MinPrimaryStationIPAddr;    // Starting primary station IP address
    ULONG   MaxPrimaryStationIPAddr;    // Ending primary station IP address
    ULONG   DestPrimaryStationIPAddr;   // Destination IP address for primary station 
    USHORT  DestPrimaryIncOffset;       // Destination primary station incremental offset
    USHORT  DestPrimaryStationBaseID;   // Starting primary station resource (UDP Port, Channel)
    ULONG   DestSecondaryStationIPAddr; // Destination IP address for secondary station
    USHORT  DestSecondaryIncOffset;     // Destination secondary station incremental offset
    USHORT  DestSecondaryStationBaseID; // Starting secondary station resource (UDP Port, Channel)
} MT_IP_ROUTE_BY_IP_PARAMS, *PMT_IP_ROUTE_BY_IP_PARAMS;

typedef struct
{
    IP_TARGET_OPTS  fTarget;                    // Destination type (i.e., network, host)
    MT_IP_MEDIA_ROUTING_TYPE RuleType;          // Routing rule type                                                             
    union                                       // structure to be used based on specified 
    {                                           // rule type
        MT_IP_ROUTE_BY_SESSION_PARAMS RouteBySessionParams;
        MT_IP_ROUTE_BY_STATION_PARAMS RouteByStationParams;
        MT_IP_ROUTE_BY_IP_PARAMS      RouteByIpParams;
    };
} MT_IP_ROUTING_PARAMS, *PMT_IP_ROUTING_PARAMS;

//
//	IP Forwarding per Session Parameters
//
typedef struct
{
    IP_TARGET_OPTS  fTarget;                    // Destination type (i.e., network, host)
    ULONG           DestPrimaryStationIPAddr;   // Destination IP address for primary station 
    ULONG           DestSecondaryStationIPAddr; // Destination IP address for secondary station
    USHORT          DestPrimaryStationBaseID;   // Primary station resource (UDP Port, Channel)
    USHORT          DestSecondaryStationBaseID; // Secondary station resource (UDP Port, Channel)
} MT_IP_SESSION_DEST, *PMT_IP_SESSION_DEST;

//
// IP Transport Statistics
//
typedef struct
{
    ULONG        IPChecksumErrors;
    ULONG        TCPChecksumErrors;
    ULONG        TCPOutOfOrderPackets;
    ULONG        TCPDroppedSegments;
    ULONG        TCPTotalPacketsReceived;
    ULONG        TCPTotalPacketsProcessed;
    ULONG        UDPChecksumErrors;
    ULONG        UDPTotalPacketsReceived;
    ULONG        UDPTotalPacketsProcessed;
} MT_IP_TRANSPORT_STATS, *PMT_IP_TRANSPORT_STATS; 

//
// Station Call Statistics
//
typedef struct
{
    USHORT      StationID;
    ULONG       InboundCallCount;
    ULONG       OutboundCallCount;
    ULONG       AverageCallDuration;		// in units of 10ms
    ULONG       PeakCallDuration;			// in units of 10ms
    ULONG       CallsConnected;
    ULONG       CallsAbandoned;
    ULONG       CallsRejected;
    ULONG       CallsReleased;
    ULONG	    CallsSuspended;
    ULONG       CallsSetup;
    ULONG       CallsAlerting;
    ULONG       CallsInProgress;
    ULONG       CallsTimeoutRelease;
    ULONG       CallsRetrieved;
    ULONG       CallsResumed;
    ULONG       CallsHeld;
} MT_IP_STATION_CALL_STATS, *PMT_IP_STATION_CALL_STATS;


//
// Network Port Status
//
#define  MT_IP_PHY_PORT_ADDRESS_LENGTH   16
typedef struct
{
	UCHAR PhysicalLinkStatus;
	UCHAR PhysicalAddress[MT_IP_PHY_PORT_ADDRESS_LENGTH];
	ULONG IPAddress;
	ULONG SubnetMask; 
} MT_IP_PORT_STATUS, *PMT_IP_PORT_STATUS;


//
// Network Port Statistics
//
typedef struct
{
   ULONG        PacketsTransmitted;
   ULONG        BytesTransmitted;
   ULONG        PacketsReceived;
   ULONG        BytesReceived;
   ULONG        TxFifoErrorCount;
   ULONG        RxDroppedPacketCount;
   ULONG        RxMalformedErrorCount;
   ULONG        MulticastErrorCount;
   ULONG        CollisionErrorCount;
} MT_IP_PORT_STATS, *PMT_IP_PORT_STATS;

//
// Board Trace Destination
//
typedef enum
{
    LOG_LOCAL = 0,  // Trace to a local file
    LOG_NETWORK     // Trace to a network server
} MT_TRACE_TYPE;

//
// Board Trace Level
//
#define MT_TRACE_LEVEL_EMERGENCY  0x00000001
#define MT_TRACE_LEVEL_ALERT      0x00000002
#define MT_TRACE_LEVEL_CRITICAL   0x00000004
#define MT_TRACE_LEVEL_ERROR      0x00000008
#define MT_TRACE_LEVEL_WARNING    0x00000010
#define MT_TRACE_LEVEL_NOTICE     0x00000020
#define MT_TRACE_LEVEL_INFO       0x00000040
#define MT_TRACE_LEVEL_DEBUG      0x00000080

//
// Board Trace Default Transport
//
#define SYSLOG_DEFAULT_SERVER_TRANSPORT_PORT    514
#define SYSLOG_DEFAULT_SERVER_TRANSPORT_PROTO   MT_UDP


#define MAX_TRACELOG_NAME_LENGTH 75
 
//
// Board Trace Logging Parameters
//
typedef struct
{
    ULONG            TraceLevelMask;        // One or more trace levels OR'ed together
    MT_TRACE_TYPE    TraceType;             // Specifies LOCAL or NETWORK tracing
    CHAR             LogFileOrServer[MAX_TRACELOG_NAME_LENGTH];   
                                            // Filename for LOCAL logging, or server's dot-notated
                                            // IP address or hostname for NETWORK logging.
    MT_IP_PROTO_PORT Transport;             // Trace logging server's port and protocol
} MT_TRACE_LOGGING_PARAMS, *PMT_TRACE_LOGGING_PARAMS;

//
// Forward Destination Identifier
//
typedef enum
{
    NONE_SIDE = 0,        
    PRIMARY_SIDE,        
    SECONDARY_SIDE
} MT_IP_FWD_DEST;

//
// EVT_DEST_UNREACHABLE event
//
typedef struct
{
    ULONG  SessionId;
    ULONG  IpAddr;
    USHORT Port;
    MT_IP_FWD_DEST FwdDest;
} MT_IP_DEST_UNREACHABLE_INFO, *PMT_IP_DEST_UNREACHABLE_INFO;


//
// EVT_LICENSE_FILE_LOADED event
//
typedef struct
{
    union
    {
        ULONG MaxSessions;
        ULONG MaxForwarding;
    };
    ULONG MaxMonitoring;
} MT_IP_LICENSE_FILE_INFO, *PMT_IP_LICENSE_FILE_INFO;

typedef struct
{
	UCHAR LicBoardSerialNumber[LIC_STRING_LEN];
    union
    {
	    ULONG LicBoardMaxSessions;
        ULONG LicBoardMaxForwarding;
    };
	UCHAR LicBoardProductID[LIC_STRING_LEN];
    ULONG LicBoardMaxMonitoring;
} MT_IP_BOARD_LICENSE_INFO, *PMT_IP_BOARD_LICENSE_INFO;

typedef struct
{
	UCHAR LicSalesOrderNumber[LIC_LONG_STRING_LEN];
	UCHAR LicCustomerName[LIC_LONG_STRING_LEN];
	UCHAR LicSerialNumber[LIC_STRING_LEN];
	union
    {
	    ULONG LicMaxSessions;
        ULONG LicMaxForwarding;
    };
	UCHAR LicDateIssued[LIC_STRING_LEN];
	UCHAR LicExpirationDate[LIC_STRING_LEN];
	UCHAR LicProductID[LIC_STRING_LEN];
	MT_IP_BOARD_LICENSE_INFO BoardLic;
    ULONG LicMaxMonitoring;
} MT_IP_LICENSE_INFO, *PMT_IP_LICENSE_INFO;

/* FM 8/16/2007: New caller ID data structures */
typedef enum {CIDTYPE_BELL202 = 0x00, CIDTYPE_NTT} MT_CID_TYPE;
typedef enum {CIDTWIST_POS = 0, CIDTWIST_ZERO, CIDTWIST_NEG} MT_CID_TWIST;

typedef struct 
{
    MT_CID_TYPE type;
    USHORT level;
    MT_CID_TWIST twist;
} MT_CIDPARAMS, *PMT_CIDPARAMS;

//
// Station MAC Address
//
typedef struct
{
	UCHAR StationTxMacAddress[MT_IP_PHY_PORT_ADDRESS_LENGTH];
	UCHAR StationRxMacAddress[MT_IP_PHY_PORT_ADDRESS_LENGTH];

}MT_ST_MAC_ADDRESS, *PMT_ST_MAC_ADDRESS;

//
// Structure of Configurable Board Parameters
//
typedef struct _MTADAPTER_CONFIGURATION {

    ULONG   SystemIndex;            // Adapter index in SDK: 0 - (MAX_BRD_DEVICES-1)
                                    // Field 1

    ULONG   AdapterType;            // This field is read only: VR6400, VR6409, and etc.
                                    // UNKNOWN_CARD for no board in index 'SystemIndex'

    // General adapter parameters
    UCHAR   MasterMode;             // MODE_SLAVE(default), MODE_MASTER, or MODE_MASTER_B
                                    // Field 3

    UCHAR   MasterClock;            // Master clock source reference value, default
                                    // CLOCK_SOURCE_OSC
    UCHAR   Sec8kNetrefClock;       // MVIP Sec8K or H100 Netref clock output, default
                                    // SPRM_SOURCE_DISABLE
    UCHAR   PresentationPreference; // Board ordering preference for the associated adapter:
                                    // Range: 0..MAX_BRD_DEVICES (default)
                                    // Not implemented for Linux yet, 9/8/03

    // General CT bus parameters
    ULONG   CTBusType;              // MUX_MVIP, MUX_H100, or MUX_NONE(default)
                                    // Field 7

    ULONG   TDMEncoding;            // ULAWPXD for u-law, ALAWPXD for a-law
    ULONG   CTBusSegmentIndex;      // Bus sigment index the associated adapter is on

    // Interface parameters
    ULONG   SMSize;                 // Shared memory size in bytes, min MIN_SMSIZE, default 8K
                                    // Field 10

    // NGX parameters               // Index 0 for base card, 1 for 1st daughter
                                    // card, 2 for 2nd daughter card
    ULONG   CTBusTermination;       // 0 for disable, others for enable
    ULONG   PBXType[MAX_DC];        // PBX supported
    ULONG   DChOption[MAX_DC];      // 0 for D-channel disabled
    ULONG   Termination[MAX_DC];    // 0 for Hi-Z, 1 for 120 ohm

    // DT/DP trunk parameters       // Index 0 for 1st trunk and etc.
    ULONG   E1Option;               // T1_OPTION for T1, E1_OPTION for E1
                                    // Field 21

    ULONG   Framing[MAX_TRUNKS];            // FRM_E1_G704(default), FRM_T1_SF, etc.
    ULONG   LineCoding[MAX_TRUNKS];         // LC_AMI(default), LC_E1_HDB3, or LC_T1_B8ZS
    ULONG   T1LineBuildOut[MAX_TRUNKS];     // LBO_T1_15DB(default), etc.
    ULONG   E1LineBuildOut[MAX_TRUNKS];     // LBO_E1_120OHM(default) or LBO_E1_75OHM
    ULONG   ZeroCodeSupression[MAX_TRUNKS]; // ZCS_NONE(default), ZCS_GTE, etc.

    ULONG   ProtocolSignaling[MAX_TRUNKS];  // SIGNALING_NONE, SIGNALING_ISDN, SIGNALING_NFAS, etc.
                                            // Field 42 to 45

    ULONG   ProtocolVariant[MAX_TRUNKS];    // PROTOCOL_ISDN_ETS300, PROTOCOL_RBS_LOOP_FXO, etc.,
                                            // Field 46 to 49

    ULONG   ISDNInterfaceSide[MAX_TRUNKS];  // SUPPORT_TE(default) or SUPPORT_NT,
                                            // Field 50 to 53

    ULONG   RBSMaxDigits[MAX_TRUNKS];       // Default 10,
                                            // Field 54 to 57

    ULONG   RBSInterDigitTime[MAX_TRUNKS];  // Min 10. Default 3000, in unit of ms
                                            // Field 57 to 60

    ULONG   RBSDialingDelay[MAX_TRUNKS];    // Min 300, Default 1000, in unit of ms
                                            // Field 62 to 65

    ULONG   NFASIndex[MAX_TRUNKS];          // 0 to (MAX_NFAS - 1)
                                            // Field 66 to 69

    // LD parameters
    ULONG   OffhookImpedance;       // Configurable off-hook impedance:
                                    //  0: FCC       600 ohms resistive
                                    //  1: ETSI      270 ohm+750 ohm || 0.15 micro-F
                                    //  2: Australia 220 ohm+680 ohm || 0.12 micro-F
                                    //  3: China     200 ohm+680 ohm || 0.1 micro-F
                                    // Field 70

	// IPX/HPX parameters
    MT_IPCONFIG IpNetParams[MAX_IP_NET_PORTS]; // IPX specific port configuration
    ULONG       DefaultGtwBootp;        // Default Gateway protocol (static or DHCP)
    ULONG       DefaultGtwSrcInterface; // Ethernet Interface to get DHCP info from
    ULONG       DefaultGateway;         // IP address of the default gateway
    ULONG       eth0Bootp;              // Boot protocol for NI 0 (static or DHCP)
    ULONG       eth1Bootp;              // Boot protocol for NI 1 (static or DHCP)
    ULONG       eth2Bootp;              // Boot protocol for NI 2 (static or DHCP)
    ULONG       DNSBootp;               // DNS protocol (static or DHCP)
    ULONG       DNSSrcInterface;        // Ethernet Interface to get DHCP info from
    ULONG       perferredDNSAddr;       // Primary DNS server
    ULONG       alternateDNSAddr;       // Secondary DNS server
    ULONG       VLANEnable;             // VLAN enable/disable flag
    ULONG       VLANId;                 // Id number of the VLAN      
    ULONG       RTPEnable;              // Media Session RTP enable/disable flag
    ULONG       RTPTimeout;             // Media Session RTP timeout
    ULONG       TCPSyncCount;           // TCP reassembly sync count
    ULONG       RTCPQoSEnable;
    ULONG       NATTopologyEnable;      // Flag indicating presence of NAT in network
    ULONG       CallSessionTimeout;     // Time () to wait before timing out a call
    ULONG       MonPort0Enable;         // Enable/Disable flag for NIC 1
    char        MonPort0InterfaceId[MAX_PATH]; // ANSI String representation of GUID
    ULONG       MonPort1Enable;         // Enable/Disable flag for NIC 2
    char        MonPort1InterfaceId[MAX_PATH]; // ANSI String representation of GUID
} MTADAPTER_CONFIGURATION, *PMTADAPTER_CONFIGURATION;

/**** PXD structure  ****/
typedef struct{
        SHORT    volume;                /* volume for playback (dB) */
        SHORT    gain;                  /* gain for conference (dB) */
        USHORT   g165enable;            /* echo cancellation enable */
        USHORT   g165adapt;             /* echo training enable */
} MT_PSTNPXD, *PMT_PSTNPXD;

/* Station Mapping for the IPX/HPX */
typedef enum 
{ 
    STATION_NETWORK_ID_TYPE_IPV4_ADDRESS = 0, 
    STATION_NETWORK_ID_TYPE_MAC_ADDRESS 
} StationNetworkIdType;

typedef struct 
{
    USHORT nStation;    
    StationNetworkIdType NetworkIdType;
    union 
    {
        ULONG IpV4Address;
        unsigned char MacAddress[MT_IP_PHY_PORT_ADDRESS_LENGTH];
    } NetworkIdValue;
} MT_STATION_MAPPING_INFO, *PMT_STATION_MAPPING_INFO;

typedef struct
{
    ULONG RemainingForwardingSessions;  // Number of forwarding session licenses remaining to be used.
    ULONG MaxForwardingSessions;        // Maximum number of forwarding sessions according to the license file.
    ULONG RemainingMonitoringSessions;  // Number of monitoring session licenses remaining to be used.
    ULONG MaxMonitoringSessions;        // Maximum number of monitoring sessions according to the license file.
} MT_LICENSE_COUNTS, *PMT_LICENSE_COUNTS;


#if _WIN32
#include <poppack.h>
#endif

#include "NTiDataCC.h"
#include "NTiDataDCC.h"

#endif

// End of NTiData.h
