#ifndef PARSETAP_H
#define PARSETAP_H

#include <iostream>
using std::cout;
using std::cin;
using std::endl;

#include <string>
using std::string;

#include <vector>
using std::vector;

#define TIXML_USE_STL

// Warnings disabled
#pragma warn -8027
#include "..\3rdParty\tinyxml\tinyxml.h"
// Warnings enabled
#pragma warn +8027

#include "XMLTFAppParser.h"

class ParseTap : public XMLTFAppParser
{
   public:

      ParseTap(string fileName,Log* outLog);

      virtual ~ParseTap(void);

      //Retruns the Voice.DBF file from the tap xml file.
      string GetVoiceFile(void);

      //Returns the SearchPath for a DBF file from the tap xml file.
      string GetSearchVoiceFile(void);

      //Returns the number of Tam files found in the TapFile.
      int TapSize(void);

      //Returns the Tam filepath specified by 'int tamIndex'.
      string TamAt(int tamIndex);//Returns a tam filepath

      //Stores in 'string& data' the attribute value of the 'function=' attribute of a step.
      //Returns 0 on failure and 1 on Success.
      int TapGetOpCode(int tamIndex, string& data);

      //Stores in 'string& data' the attribute value of the 'label=' attribute.
      int TapGetLabel(int tamIndex, string& data);

      //Stores in 'string& data' the attribute value of the 'current=' attribute.
      int TapGetStepNum(int tamIndex, string& data);

      //Returns 1 or 0 on succes or failure and stores the paramtervalue found in 'string& value'
      bool TapParameterValue(int parameterValue,string& value);

      //Returns 1 or 0 on success or failure and stores the Parametername found in 'string& value'
      int TapParameterName(int parameterValue,string& value);

      //Returns 1 or 0 on success or failure and stores the type in 'int* num'
      int TapParameterType(int parameterValue, int* num);

      //Checks wheter or not the logdisable field is set.
      //True or false is returned when the log is disabled or enabled. 
      bool GetLogDisabled(int tamIndex);

      //Creates the proper VoiceFilename for the VoiceDBF File.
      int FixVoiceFileName(string& file);
      
   private:

      //Stores in 'string& filepath' the tam filepath with the .xml extension instead of the .xml extension.
      void ConvertToXmlExt(string& filePath);

      void LoadTamFiles(void);

      //Stores the filepath of the tam in file.
      void FixFilePath(string& file);

      //holds the filename of the tams files
      vector<string> TamFiles;

      //holds the index for the node vector of the steps containig the tam files.
      vector<int> TamFilesStepID;

      //will hold the filepath for the voice dbf file.
      string voiceDbf;

      //will hold the search paths for the voice dbf file.
      string searchVoiceDbf;

      Log *ivLog;                                   
};
#endif
