#include <stdlib.h>               // atoi
#include "step_audiorecord.h"
#include "..\escutil\iv_str.h"	// strtrim
#include "..\BoardDLLs\brd_generic.h"

StepAudioRecord::StepAudioRecord(STEPREC& stepRecord) : Step(stepRecord)
{
   bInt = (bool) stepRecord.u1;
   // Default value of 0 indicates AGC should be used (i.e. true)
   automaticGainControl = !((bool) stepRecord.u2);
   disableBeep = (bool) stepRecord.u4;
   format = (SOUND_FORMAT) stepRecord.u3;
   maxTime = stepRecord.s4.arr;
   maxSilence = stepRecord.s2.arr;
   fileNm = stepRecord.s6.arr;
}

StepAudioRecord::~StepAudioRecord(void)
{
}

const char* StepAudioRecord::GetStepName(void) const
{
	return("Record");
}

// -------------------------------------------------------------------------
//   Name:        StepAudioRecord::Execute
//
//   Notes:       - I don't think this function should use the global timeout
//                  default.  I think if an invalid time is set, it should
//                  default to forever.
// -------------------------------------------------------------------------
IV_STATUS StepAudioRecord::Execute(PIVRINFO pInfo)
{
   IV_STATUS status;
   // Evaluate all parameters before exiting
   bool bParmsValid = true;
   int volLvl;
   int iMaxTime;
   int iMaxSilence;
   char* value;
   string completeFileName;
   string pathName;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

	if(PortRequired(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FATAL);
   }

   if((value = pInfo->variables->ParseParameter("Maximum record time", maxTime.c_str(),
      "-1", "Empty Maximum record time defaults to none")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      iMaxTime = atoi(value);  // iMaxTime is in milliseconds.
      delete[] value;
   }

   if((value = pInfo->variables->ParseParameter("Silence TIMEOUT", maxSilence.c_str(), "@TIMEOUT")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      iMaxSilence = atoi(value);  // iMaxSilence is in milliseconds.
      delete[] value;

      if(iMaxSilence == 0)
      {
         pInfo->ivLog->Print("      ERROR: Silence TIMEOUT '%s' cannot be 0\n",
            pInfo->variables->IsParsedEmpty(maxSilence.c_str()) ? "@TIMEOUT" : maxSilence.c_str());
         bParmsValid = false;
      }
   }

   // Get volume information
   volLvl = atoi(pInfo->variables->Get("@VOLUME"));

   if((value = pInfo->variables->ParseParameter("Voice file", fileNm.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      strtrim(value);
      completeFileName = value;
      delete[] value;

      pathName = value = pInfo->variables->FilePath(completeFileName.c_str());
      delete[] value;
   }

   if(!bParmsValid)
   {
      return(IVR_FATAL);
   }

	// Flush TT buffer so that we don't accidentally interrupt record before it starts
   if((status = pInfo->ivBoard->FlushDigits()) != IVR_SUCCESS)
   {
      return(status);
   }

   pInfo->ivLog->Print("      Recording '%s'\n", pathName.c_str());

   status = pInfo->ivBoard->AudioRecord(pathName.c_str(), bInt, format, volLvl, iMaxTime, iMaxSilence, disableBeep, automaticGainControl);

   return(status);
}
