#include <stdlib.h>               // atoi
#include "step_audiostartrecord.h"
#include "..\escutil\iv_str.h"	// strtrim
#include "..\BoardDLLs\brd_generic.h"

StepAudioStartRecord::StepAudioStartRecord(STEPREC& stepRecord) : Step(stepRecord)
{
   format = (SOUND_FORMAT) stepRecord.u1;
   fileNm = stepRecord.s1.arr;
   disableBeep = (bool) stepRecord.u4;
   // Default value of 0 indicates AGC should be used (i.e. true)
   automaticGainControl = !((bool) stepRecord.u2);
}

StepAudioStartRecord::~StepAudioStartRecord(void)
{
}

const char* StepAudioStartRecord::GetStepName(void) const
{
	return("Start Record");
}

IV_STATUS StepAudioStartRecord::Execute(PIVRINFO pInfo)
{
   IV_STATUS status;
   int volLvl;
   char* value;
   char* pathName;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

	if(PortRequired(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FATAL);
   }

   // Get volume information
   volLvl = atoi(pInfo->variables->Get("@VOLUME"));

   if((value = pInfo->variables->ParseParameter("Voice file", fileNm.c_str())) == NULL)
   {
      return(IVR_FATAL);
   }

   strtrim(value);
   pathName = pInfo->variables->FilePath(value);
   delete[] value;

   pInfo->ivLog->Print("      Recording '%s'\n", pathName);

   status = pInfo->ivBoard->StartRecordFile(pathName, false, format, volLvl, INFINITE, INFINITE, disableBeep, automaticGainControl);

   delete[] pathName;

   return(status);
}
