#pragma hdrstop
#include "..\escutil\security.h"
#include "..\BoardDLLs\brd_generic.h"
#include "step_decrypt.h"

StepDecrypt::StepDecrypt(STEPREC& stepRecord) : Step(stepRecord)
{
   resultVar = stepRecord.s3.arr;
   text = stepRecord.s1.arr;
   key = stepRecord.s2.arr;
}

StepDecrypt::~StepDecrypt(void)
{
}

const char* StepDecrypt::GetStepName(void) const
{
	return("Decrypt");
}

IV_STATUS StepDecrypt::Execute(PIVRINFO pInfo)
{
   char* value;
   // Evaluate all parameters before exiting
   bool bParmsValid = true;
   string parsedText;
   string parsedKey;
   char* pszResult;
   IV_STATUS status;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   if((value = pInfo->variables->ParseParameter("Encrypted text", text.c_str(), "")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      parsedText = value;
      delete[] value;
      if(pInfo->variables->IsEmpty(parsedText.c_str()))
      {
         bParmsValid = false;
         pInfo->ivLog->Print("      ERROR: Encrypted text cannot be empty\n");
      }
   }

   if((value = pInfo->variables->ParseParameter("Decryption key", key.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      parsedKey = value;
      delete[] value;
   }

   if(!bParmsValid)
   {
      return(IVR_FATAL);
   }

   if((pszResult = Decrypt(parsedText.c_str(), parsedKey.c_str())) == NULL)
   {
      pInfo->ivLog->Print("      ERROR: Decrypt returned invalid result\n");
      return(IVR_FATAL);
   }

   status = pInfo->variables->Set(resultVar.c_str(), pszResult) ? IVR_SUCCESS : IVR_FATAL;

   delete[] pszResult;

   return(status);
 }

