#ifndef STEP_PARSE_H
#define STEP_PARSE_H

#include "step.h"

class StepParse : public Step
{
   public:

      enum PARSE_START
      {
         PS_BEGIN = 1,
         PS_END,
         PS_CURRENT,
         PS_SPECIFIC
      };

      enum PARSE_TYPE
      {
         PT_TEXT = 1,
         PT_NUMBER,
         PT_ANYCHAR,
         PT_ELEMENT
      };

      StepParse(STEPREC& stepRecord);
      virtual ~StepParse(void);

		const char* GetStepName(void) const;

      IV_STATUS Execute(PIVRINFO pInfo);

   protected:

      PARSE_TYPE   iSearchType;     // U1   types = text, real number, or any char
      PARSE_START  iStartCode;      // U2   indicates where to start searching from
      bool         bCaseSensitive;  // U3   search is case sensitive (Y/N)
      bool         bForward;        // U4   search forward? (Y/N)
      string       parse;           // S1   string to be parsed
      string       startPos;        // S2   defined starting position
      string       occurrence;      // S3   occurrence number to search for
      string       varOffset;       // S4   result (var to receive new offset position)
      string       search;          // S6   sub-string to search for

      int FindString(const char* pszText, int iStartPos, int iOccurrence,
         const char*   pszSearch, bool bForward, bool bCaseSensitive);

      int FindNumber(const char* pszText, int iStartPos, int iOccurrence, bool bForward);

      int FindAnyChar(const char* pszText, int iStartPos, int iOccurrence,
         const char* pszSearch, bool bForward, bool bCaseSensitive);
};

#endif
