#include <errno.h>
#include "step_scandir.h"
#include "..\escutil\iv_str.h"	// charadd, stradd
#include "..\escutil\file_fns.h" // ffblk, findfirst, findnext

StepScanDir::StepScanDir(STEPREC& stepRecord) : Step(stepRecord)
{
   dirNm = stepRecord.s6.arr;
   fileMask = stepRecord.s1.arr;
   dirVarNm = stepRecord.s2.arr;
   cntVarNm = stepRecord.s3.arr;
}

StepScanDir::~StepScanDir(void)
{
}

const char* StepScanDir::GetStepName(void) const
{
	return("Scan Folder");
}

IV_STATUS StepScanDir::Execute(PIVRINFO pInfo)
{
   IV_STATUS status = IVR_SUCCESS;
   int           iCount;
   int           iFindResult;
   int           iPathSize;
   int           iBufSize = 0;
   // Evaluate all parameters before exiting
   bool bParmsValid = true;
   char*         value;
   char*         fullPath;
   string        parsedFileMask;
   char*         pszBuf   = NULL;
   char          pszCount[8];
   FindData      findData;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   // Get path and file name to open.
   if((value = pInfo->variables->ParseParameter("Directory", dirNm.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      fullPath = pInfo->variables->FilePath(value);
      delete[] value;

   	// Truncate ending slash
      if(fullPath[strlen(fullPath) - 1] == '\\')
      {
      	fullPath[strlen(fullPath) - 1] = '\0';
      }
   }

   // Get file mask.
   if((value = pInfo->variables->ParseParameter("Directory", fileMask.c_str(), "*.*")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      parsedFileMask = value;
      delete[] value;
   }

   if(!bParmsValid)
   {
      delete[] fullPath;
      return(IVR_FATAL);
   }

   if(FindFirst(fullPath, &findData) == -1)
   {
	   // Path not found or No such directory (but not root either)
   	if((_doserrno == ENOPATH) ||
      	((_doserrno == ENOENT) && !(fullPath[strlen(fullPath) - 2] == ':')))
      {
		   pInfo->ivLog->Print("      ERROR: Unable to find path or directory '%s'\n", fullPath);
         delete[] fullPath;
	   	return(IVR_FAIL);
      }
   }

   iPathSize = strlen(fullPath) + 1;
   charadd(fullPath, '\\', iPathSize);
   stradd(fullPath, parsedFileMask.c_str(), iPathSize);

   iFindResult = FindFirst(fullPath, &findData);
   // Load directory entries into buffer.  Separate entries with newline.
   iCount = 0;
   while(iFindResult == 0)
   {
      ++iCount;
      stradd(pszBuf, GetName(&findData), iBufSize);
      charadd(pszBuf, '\n', iBufSize);
      iFindResult = FindNext(&findData);
   }

   // If variable defined, update variable to number of directory entries.
   if(!cntVarNm.empty())
   {
      sprintf (pszCount, "%d", iCount);

      if(!pInfo->variables->Set(cntVarNm.c_str(), pszCount))
      {
         pInfo->ivLog->Print("      ERROR: Failed to set variable: %s\n", cntVarNm.c_str());
         status = IVR_FATAL;
      }
   }

   // Store the directory entries into the variable specified.
   if(!dirVarNm.empty() && !pInfo->variables->Set(dirVarNm.c_str(), pszBuf))
   {
      pInfo->ivLog->Print("      ERROR: Failed to set variable: %s\n", dirVarNm.c_str());
      status = IVR_FATAL;
   }

   delete[] pszBuf;
   delete[] fullPath;

   return(status);
}
