#include "..\escutil\odbc_db.h"
#include "step_sqldisconnect.h"

StepSQLDisconnect::StepSQLDisconnect(STEPREC& stepRecord) : Step(stepRecord)
{
   odbcConnection = stepRecord.s2.arr;
}

StepSQLDisconnect::~StepSQLDisconnect(void)
{
}

const char* StepSQLDisconnect::GetStepName(void) const
{
	return("DB Disconnect");
}

IV_STATUS StepSQLDisconnect::Execute(PIVRINFO pInfo)
{
   char* valueConnectHandle = NULL;
   bool bParmsValid = true;   // to evaluate all parameters before exiting
   IV_STATUS  status;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   if(pInfo->odbcManager == NULL)
   {
      pInfo->ivLog->Print("      ERROR: SQL not initialized.\n");
      return(IVR_FATAL);
   }

   // Parse ODBC connection handle
   if((valueConnectHandle = pInfo->variables->ParseParameter("ODBC Connection", odbcConnection.c_str(), "")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      if(strcmp(valueConnectHandle,"") == 0)
      {
         pInfo->ivLog->Print("      Using default ODBC handle\n");
      }
   }

   if(!bParmsValid)
   {
      delete[] valueConnectHandle;  // safe to delete, pointer was initialized to NULL
      return(IVR_FATAL);
   }

   status = pInfo->odbcManager->Disconnect(valueConnectHandle);

   delete[] valueConnectHandle;

   return(status);
}

