#include <stdlib.h>             // Needed for "atoi".
#include "step_xmlload.h"
#include "..\xmlutil\xmllist.h"

StepXMLLoad::StepXMLLoad(STEPREC& stepRecord) : Step(stepRecord)
{
   handle = stepRecord.s1.arr;
   source = stepRecord.s3.arr;
   iSrcType = stepRecord.u1;
}

StepXMLLoad::~StepXMLLoad(void)
{
}

const char* StepXMLLoad::GetStepName(void) const
{
	return("XML Load");
}

IV_STATUS StepXMLLoad::Execute (PIVRINFO pInfo)
{
   char* value;
   // Evaluate all parameters before exiting
   bool bParmsValid = true;
   int iHandle;
   string parsedSource;
   XML* pxmlReq;
   IV_STATUS status;
   char* pathName;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   if(pInfo->xmlList == NULL)
   {
      pInfo->ivLog->Print("      ERROR: XML not initialized.\n");
      return(IVR_FATAL);
   }

   // --- Retrieve handle number
   if((value = pInfo->variables->ParseParameter("Handle", handle.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      iHandle = atoi(value);
      delete[] value;
   }

   // --- Retrieve source
   if((value = pInfo->variables->ParseParameter("Variable / file to load", source.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      parsedSource = value;
      delete[] value;
   }

   if(!bParmsValid)
   {
      return(IVR_FATAL);
   }

   // --- Search for previous XML document open under this handle,
   //     and if found, close it.
   pxmlReq = pInfo->xmlList->FindXML (iHandle);
   if (pxmlReq)
   {
      pInfo->ivLog->Print("      WARNING: Handle %d still open.  Closing handle.\n", iHandle);
      delete pxmlReq;
   }

   // --- Create an XML object for this document request
   pxmlReq = new XML (pInfo->ivLog);
   pxmlReq->SetReferenceNo (iHandle);

   // --- Retrieve the XML document
   switch (iSrcType)
   {
      case 0:
      {
         status = pxmlReq->LoadFromBuffer(parsedSource.c_str());
         break;
      }
      case 1:
      {
         pathName = pInfo->variables->FilePath(parsedSource.c_str());
         pInfo->ivLog->Print("      Loading from '%s'\n", pathName);
         status = pxmlReq->LoadFromFile(pathName);
         delete[] pathName;
         break;
      }
      default:
      {
         pInfo->ivLog->Print("      ERROR: Undefined XML source type.\n");
         return IVR_FATAL;
      }
   }

   if(status == IVR_SUCCESS)
   {
      // --- Add it to the linked list of open XML documents
      pInfo->xmlList->AddLast ((Link*)pxmlReq);
   }

   return(status);
}
// StepXMLLoad::Execute
