#include <map>
using std::map;
#include <vector>
using std::vector;
#include <fstream>
using std::ofstream;

#include "TFServer_communication.h"
#include "TFServerRegistrationInfo.h"
#include "..\TFEngine\TFEngine_communication.h"
#include "TFTCPClient_communication.h"
#include "..\inet\Socket.h"
#include "..\inet\inet_msg.h"

// Windows thread function to execute TCPClientManager::Run.
// clientManager must be a pointer to an instance of TCPClientManager.
unsigned int __stdcall TCPClientManagerMain(void* clientManager);

// This class is designed to handle TCP client connections and requests, executed
// in the Run method, and updates from TeleFlow applications.
// This class is desgined to be thread-safe.
class TCPClientManager : public CommunicationDestination<TFServerCommunicationPacket::TF_DESTINATION_ID>
{
   public:

      TCPClientManager(unsigned short binport,
                       unsigned short httpport,
                       CommunicationHub<TFServerCommunicationPacket::TF_DESTINATION_ID>& commHub,
                       TFServerRegistrationInfo* pRegInfo);

      virtual ~TCPClientManager(void);

      void BroadcastAppMessage(TFEngineCommunicationPacket::TF_ENGINE_EVENT eventID);
      void Clear(void);
      TFTCPClientCommunicationDestination* GetCommunicationDestination(void);
      TFServerCommunicationPacket::TF_DESTINATION_ID ID(void);
      void ProcessCurrentClientRequest(void);
      void RefreshWaitObjects(void);
      void RemoveCurrentClient(void);
      void Run(void);

      // Send the TF_ENGINE_EVENT specified to the TeleFlow application defined
      // by the app ID currently stored in the (NET_MSG) clientRequest property
      void SendAppMessage(TFEngineCommunicationPacket::TF_ENGINE_EVENT eventID);
      void SendAppStreamMessage(TFEngineCommunicationPacket::TF_ENGINE_EVENT eventID);
      void SendOutput(void);

      // Send binary update of all application thread activity to TFMonitor
      // client pointed to by 'currentClient' member (connected on binary port)
      void SendUpdate(void);

      // Send XML update of all application thread activity to the client
      // pointed to by 'currentClient' member (connected on HTTP port)
      void SendXMLUpdate(void);

      // Respond to invalid HTTP request with an error message to the
      // client pointed to by 'currentClient' member (connected on HTTP port)
      void SendErrorResponse(void);

      // Individual Engine threads call this method to update their
      // application information stored in the 'appsInfo' property (map)
      void UpdateAppInfo(EXPANDED_CLIENT_MSG& appInfo);

      // Individual Engine threads call this method to update their status
      // flag only in application info stored in the 'appsInfo' property (map)
      void UpdateAppStatus(int appID, char status);

      // Translate a message event sent to the server from the client
      // into char strings for output in Log
      const char* messageTypeString(unsigned long msgID);

   protected:
      // Outputs info about the remote client connection into monitor log (monLog)
      void DisplayRemoteInfo(TCPSocket* tcpClient);

      CommunicationHub<TFServerCommunicationPacket::TF_DESTINATION_ID>& commHub;
      map<unsigned int, EXPANDED_CLIENT_MSG> appsInfo;
      vector<TCPSocket*> clients;
      SocketServer connectionRequestBinSocket;   // (original) binary Monitor protocol
      SocketServer connectionRequestHTTPSocket;  // support for HTTP interface
      vector<HANDLE> waitObjects;
      TCPSocket* currentClient;
      NET_MSG clientRequest;
      Mutex dataMutex;
      string logfolder;
      string tfstart;
      GUILog* monLog;
      const unsigned short binaryInterfacePort;
      const unsigned short httpInterfacePort;
};
