#include <winsock2.h>
#include <windows.h>

#include <string>
using std::string;

#include "tf_registry.h"
#include "TFServerRegistrationInfo.h"

TFServerRegistrationInfo::TFServerRegistrationInfo(void)
{
   pszBoardDLL = NULL;
   pszInstallationPath = NULL;
   pszLogDir = NULL;
   pszAppListFileNm = NULL;
   pszTFStart = NULL;
}

TFServerRegistrationInfo::~TFServerRegistrationInfo(void)
{
   Clear();
}

void TFServerRegistrationInfo::Clear(void)
{
   delete[] pszBoardDLL;
   pszBoardDLL = NULL;
   delete[] pszInstallationPath;
   pszInstallationPath = NULL;
   delete[] pszLogDir;
   pszLogDir = NULL;
   delete[] pszAppListFileNm;
   pszAppListFileNm = NULL;
   delete[] pszTFStart;
   pszTFStart = NULL;
}

bool TFServerRegistrationInfo::Load(FileLog& sysLog)
{
   HKEY hkey;
   DWORD dwLen;
   DWORD dwType;
   bool result = true;

   Clear();

   if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, TFSERVER_REG_KEY, 0, KEY_READ, &hkey) != ERROR_SUCCESS)
   {
      sysLog.Print("ERROR: Failed to open registry key '%s'\n", TFSERVER_REG_KEY);
      return(false);
   }

   // **** LogDirectory *******************************************************
   if(RegQueryValueEx(hkey, "LogDirectory", NULL, &dwType, NULL, &dwLen) != ERROR_SUCCESS)
   {
      sysLog.Print("ERROR: Failed to open registry key '%s\\LogDirectory'\n", TFSERVER_REG_KEY);
      result = false;
   }
   else
   {
      pszLogDir = new char[dwLen + 1];
      pszLogDir[0] = '\0';

      if(RegQueryValueEx(hkey, "LogDirectory", NULL, &dwType, (LPBYTE) pszLogDir, &dwLen) != ERROR_SUCCESS)
      {
         sysLog.Print("ERROR: Failed to open registry key '%s\\LogDirectory'\n", TFSERVER_REG_KEY);
         result = false;
      }
   }

   // **** InstallationPath ***********************************************************
   if(RegQueryValueEx(hkey, "InstallDirectory", NULL, &dwType, NULL, &dwLen) != ERROR_SUCCESS)
   {
      sysLog.Print("ERROR: Failed to open registry key '%s\\InstallDirectory'\n", TFSERVER_REG_KEY);
      result = false;
   }
   else
   {
      pszInstallationPath = new char[dwLen + 1];
      pszInstallationPath[0] = '\0';

      if(RegQueryValueEx(hkey, "InstallDirectory", NULL, &dwType, (LPBYTE) pszInstallationPath, &dwLen) != ERROR_SUCCESS)
      {
         sysLog.Print("ERROR: Failed to open registry key '%s\\InstallDirectory'\n", TFSERVER_REG_KEY);
         result = false;
      }
   }

   // **** BoardDLL ***********************************************************
   if(RegQueryValueEx(hkey, "BoardDLL", NULL, &dwType, NULL, &dwLen) != ERROR_SUCCESS)
   {
      sysLog.Print("ERROR: Failed to open registry key '%s\\BoardDLL'\n", TFSERVER_REG_KEY);
      result = false;
   }
   else
   {
      pszBoardDLL = new char[dwLen + 1];
      pszBoardDLL[0] = '\0';

      if(RegQueryValueEx(hkey, "BoardDLL", NULL, &dwType, (LPBYTE) pszBoardDLL, &dwLen) != ERROR_SUCCESS)
      {
         sysLog.Print("ERROR: Failed to open registry key '%s\\BoardDLL'\n", TFSERVER_REG_KEY);
         result = false;
      }
   }

   // **** AppListFileName ****************************************************
   if(RegQueryValueEx(hkey, "AppListFileName", NULL, &dwType, NULL, &dwLen) != ERROR_SUCCESS)
   {
      sysLog.Print("ERROR: Failed to open registry key '%s\\AppListFileName'\n", TFSERVER_REG_KEY);
      result = false;
   }
   else
   {
      pszAppListFileNm = new char[dwLen + 1];
      pszAppListFileNm[0] = '\0';

      if(RegQueryValueEx(hkey, "AppListFileName", NULL, &dwType, (LPBYTE) pszAppListFileNm, &dwLen) != ERROR_SUCCESS)
      {
         sysLog.Print("ERROR: Failed to open registry key '%s\\AppListFileName'\n", TFSERVER_REG_KEY);
         result = false;
      }
   }

   // **** MonitorPort ********************************************************
   dwLen = sizeof(DWORD);
   if(RegQueryValueEx(hkey, "MonitorPort", NULL, &dwType, (LPBYTE) &dwMonitorPort, &dwLen) != ERROR_SUCCESS)
   {
      sysLog.Print("WARNING: Failed to open registry key '%s\\MonitorPort'\n", TFSERVER_REG_KEY);
      sysLog.Print("   Monitor port set to 5050\n");
      dwMonitorPort = 5050;
   }

   // **** MonitorHTTPPort *****************************************************
   dwLen = sizeof(DWORD);
   if(RegQueryValueEx(hkey, "MonitorHTTPPort", NULL, &dwType, (LPBYTE) &dwMonitorHTTPPort, &dwLen) != ERROR_SUCCESS)
   {
      sysLog.Print("WARNING: Failed to open registry key '%s\\MonitorHTTPPort'\n", TFSERVER_REG_KEY);
      dwMonitorHTTPPort = dwMonitorPort + 1;
      sysLog.Print("   Monitor HTTP port set to %i\n", dwMonitorHTTPPort);
   }

   // **** LogLines ***********************************************************
   dwLen = sizeof(DWORD);
   if(RegQueryValueEx(hkey, "LogLines", NULL, &dwType, (LPBYTE) &dwLogLines, &dwLen) != ERROR_SUCCESS)
   {
      sysLog.Print("WARNING: Failed to open registry key '%s\\LogLines'\n", TFSERVER_REG_KEY);
      sysLog.Print("   Log lines set to 200\n");
      dwLogLines = 200;
   }
   else if(dwLogLines < 50)
   {
      sysLog.Print("WARNING: Log lines was %d and must be at least 50\n", dwLogLines);
      sysLog.Print("   Log lines set to 50\n");
      dwLogLines = 50;
   }
   else if(dwLogLines > 20000)
   {
      sysLog.Print("WARNING: Log lines was %d and must be no more than 20000\n", dwLogLines);
      sysLog.Print("   Log lines set to 20000\n");
      dwLogLines = 20000;
   }

   // **** MaxLogFileSize *****************************************************
   dwLen = sizeof(DWORD);
   if(RegQueryValueEx(hkey, "MaxLogFileSize", NULL, &dwType, (LPBYTE) &dwMaxLogFileSize, &dwLen) != ERROR_SUCCESS)
   {
      sysLog.Print("WARNING: Failed to open registry key '%s\\MaxLogFileSize'\n", TFSERVER_REG_KEY);
      sysLog.Print("   Maximum log size set to 500 kB\n");
      dwMaxLogFileSize = 500;
   }

   // **** LogSRAudio *********************************************************
   dwLen = sizeof(DWORD);
   if(RegQueryValueEx(hkey, "LogSRAudio", NULL, &dwType, (LPBYTE) &dwLogSRAudio, &dwLen) != ERROR_SUCCESS)
   {
      sysLog.Print("WARNING: Failed to open registry key '%s\\LogSRAudio'\n", TFSERVER_REG_KEY);
      sysLog.Print("   SR logging set to off\n");
      dwLogSRAudio = 0;
   }

   RegCloseKey(hkey);

   return(result);
}

bool TFServerRegistrationInfo::OpenSystemLog(FileLog& sysLog)
{
   HKEY hkey;
   DWORD dwLen;
   DWORD dwType;

   if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, TFSERVER_REG_KEY, 0, KEY_READ, &hkey) != ERROR_SUCCESS)
   {
      sysLog.Open("TFServer", false, true);
      sysLog.Print("ERROR: Failed to open registry key '%s'\n", TFSERVER_REG_KEY);
      return(false);
   }

   // **** LogDirectory *******************************************************
   if(RegQueryValueEx(hkey, "LogDirectory", NULL, &dwType, NULL, &dwLen) != ERROR_SUCCESS)
   {
      sysLog.Open("TFServer", false, true);
      sysLog.Print("ERROR: Failed to open registry key '%s\\LogDirectory'\n", TFSERVER_REG_KEY);
      return(false);
   }

   delete[] pszLogDir;
   pszLogDir = new char[dwLen + 1];
   pszLogDir[0] = '\0';

   if(RegQueryValueEx(hkey, "LogDirectory", NULL, &dwType, (LPBYTE) pszLogDir, &dwLen) != ERROR_SUCCESS)
   {
      sysLog.Open("TFServer", false, true);
      sysLog.Print("ERROR: Failed to open registry key '%s\\LogDirectory'\n", TFSERVER_REG_KEY);
      return(false);
   }

   // Open syslog
   sysLog.Open(((string) pszLogDir + (string) "\\TFServer").c_str(), true, true);

   RegCloseKey(hkey);

   return(true);
}
