#include <stdio.h>
#include <process.h>
#include <dos.h>

#include <string.h>
#include <string>
using std::string;
#include <algorithm>
using std::transform;
#include "gui_log.h"
#include "ivr_defs.h"	// TELEFLOW_APP_NAME_MAX
#include "..\escutil\odbc_db.h"
#include "license_defs.h"
#include "get_mac.h"
/*
  #include "..\TFLicensing\license_defs.h"
#include "..\TFLicensing\get_mac.h"

*/
#include "..\escutil\security.h"
#include "app_mgr.h"


#include "tf_registry.h"

static int DetermineTALVersion(char* pszFileNm);

/* -------------------------------------------------------------------------
   Name:        IVRApps::IVRApps
   ------------------------------------------------------------------------- */
IVRApps::IVRApps(void) : Head()
{
   hMutex = CreateMutex(0, false, 0);
}
/*IVRApps::IVRApps*/

/* -------------------------------------------------------------------------
   Name:        IVRApps::~IVRApps
   ------------------------------------------------------------------------- */
IVRApps::~IVRApps(void)
{
   CloseHandle(hMutex);
}
/*IVRApps::~IVRApps*/

/* -------------------------------------------------------------------------
   Name:        IVRApps::Lock
   ------------------------------------------------------------------------- */
bool IVRApps::Lock(unsigned long timeout)
{
   DWORD  ret;

   ret = WaitForSingleObject(hMutex, timeout);
   if(ret == WAIT_OBJECT_0)
	{
      return(true);
   }

   return(false);
}
/*IVRApps::Lock*/

/* -------------------------------------------------------------------------
   Name:        IVRApps::Unlock
   ------------------------------------------------------------------------- */
void IVRApps::Unlock(void)
{
   ReleaseMutex(hMutex);
}
/*IVRApps::Unlock*/

/* -------------------------------------------------------------------------
   Name:        IVRApp::IVRApp
   ------------------------------------------------------------------------- */
IVRApp::IVRApp(DWORD dwLogLines, DWORD dwMaxFileSize) : Link()
{
   memset(&data, 0, sizeof(IVRINFO));
   data.portLicense = UNLICENSED;
   // 3 second application restart delay
   data.restartDelay    = 3000;
   data.bRestartInfinite = false;
   data.bLogOutput = true;
   data.bLogAppend = true;
   data.bRestart = true;
   data.bTerminate = false;
   data.reloadRequested = false;
   data.shutdownRequested = false;
   data.safeToShutdown = true;
   data.dataMutex = CreateMutex(0, false, 0);
   data.ivLog = new GUILog(data.dataMutex, dwMaxFileSize, dwLogLines);
   data.dwLogLines = dwLogLines;
}
/*IVRApp::IVRApp*/

/* -------------------------------------------------------------------------
   Name:        IVRApp::~IVRApp
   ------------------------------------------------------------------------- */
IVRApp::~IVRApp(void)
{
   ReleaseInstance(data.ivBoard);
   delete [] data.pszInstallationPath;
   delete [] data.pszProtocol;
   delete [] data.pszAppNm;
   delete [] data.pszDebugParam;
   delete [] data.pszAppFile;
   delete [] data.pszLogDir;

   for(int i = 0; i < data.iParamCount; i++)
   {
      delete [] data.pszParams[i];
   }

   delete data.ivLog;
   CloseHandle(data.dataMutex);
}
/*IVRApp::~IVRApp*/

/* -------------------------------------------------------------------------
   Name:        DetermineTALVersion

   Purpose:     Reads the version number from the .TAL file.  If no header
                is found, the file is assumed to be in first version format.
   ------------------------------------------------------------------------- */
int DetermineTALVersion(char* pszFileNm)
{
   int    version;
   FILE*  fTAL;
   char   szLine[1024];
   char   szVersion[4];   // Format expected: "XXX"

   if((fTAL = fopen (pszFileNm, "r")) == NULL)
   {
      return 0;                                  // Unable to open file.
   }

   fgets(szLine, 1024, fTAL);
   if(!strncmp (szLine, "TAL Version: ", 13))
   {
      strncpy(szVersion, &(szLine[13]), 4);
      szVersion[3] = 0;

      version = atoi (szVersion);
      if(version < 2)
      {
         version = 0;       // ERROR: Headers weren't included until version 2.
      }

   }
   else
   {
      version = 1;
   }

   fclose(fTAL);
   return(version);
}
/*DetermineTALVersion*/

/* -------------------------------------------------------------------------
   Name:        IVRApps::LoadTAL

   Purpose:     Calls the correct file loader for the file format.
   ------------------------------------------------------------------------- */
bool IVRApps::LoadTAL(char* pszFileNm, char* pszLogDir, DWORD dwLogLines,
	DWORD dwMaxLogFileSize, char* pszInstallationPath, HANDLE hLoadingMutex,
   FileLog* sysLog, HINSTANCE boardLibrary, int *pFirstSIPPort)
{
   FILE* fTAL;
   char szLine[1024];   // String to hold field read in from file.
   int iOffset;         // Offset into szLine.
   int ch;              // Character read in from file.
   int iFieldNo;        // Current field being read from file.
   IVRApp* temp = NULL;
   bool bHaveApp = false;
   int version = DetermineTALVersion(pszFileNm);
   DWORD logSRAudio;
   int SIPStarts = 0;   // Denote where SIP ports start
                        // In TeleFlow SIP prototype all ports >= this number
                        // are deemed to be SIP ports - TF 2005/08/03

   // vars for determining the license state
   int iNumports = 0;
   //bool bExpired = true;
   bool bLicensed = false;
   bool bTempLicense = true;
   int iExpDate;
   int iCurrDate;

   // get registry information for the number of ports
   HKEY  hkey;
   DWORD dwLen;
   DWORD dwType;

   // this is the LICENSE
   RegOpenKeyEx (HKEY_LOCAL_MACHINE, TFSERVER_REG_KEY, 0, KEY_READ, &hkey);
   RegQueryValueEx (hkey, "TFKEY", (LPDWORD)0, &dwType, NULL, &dwLen);
   unsigned char* tflicense = new unsigned char[dwLen + 1];
   RegQueryValueEx (hkey, "TFKEY", 0, &dwType, tflicense, &dwLen);
   string strTFLicense = (char*)tflicense;
   delete[] tflicense;

   if(!strTFLicense.empty())
   // only check the other registry settings if there IS a license key
   {

      // this should be the encrypted NUMPORTS eventually
      // actual numport comparison done on a line by line basis as .TAL is loaded
      RegOpenKeyEx (HKEY_LOCAL_MACHINE, TFSERVER_REG_KEY, 0, KEY_READ, &hkey);
      RegQueryValueEx (hkey, "NUMPORTS", (LPDWORD)0, &dwType, NULL, &dwLen);
      unsigned char* numports = new unsigned char[dwLen + 1];
      RegQueryValueEx (hkey, "NUMPORTS", 0, &dwType, numports, &dwLen);
      string strNumPorts = (char*)numports;
      delete[] numports;
      iNumports = atoi(strNumPorts.c_str());

      // this should be the encrypted EXPIRARY DATE eventually
      RegOpenKeyEx (HKEY_LOCAL_MACHINE, TFSERVER_REG_KEY, 0, KEY_READ, &hkey);
      RegQueryValueEx (hkey, "EXPDATE", (LPDWORD)0, &dwType, NULL, &dwLen);
      unsigned char* expdate = new unsigned char[dwLen + 1];
      RegQueryValueEx (hkey, "EXPDATE", 0, &dwType, expdate, &dwLen);
      string strExpDate = (char*)expdate;
      delete[] expdate;
      iExpDate = atoi(strExpDate.c_str());

      // this should be the encrypted TRANSACTION ID eventually
      RegOpenKeyEx (HKEY_LOCAL_MACHINE, TFSERVER_REG_KEY, 0, KEY_READ, &hkey);
      RegQueryValueEx (hkey, "TFTRANS", (LPDWORD)0, &dwType, NULL, &dwLen);
      unsigned char* transactionid = new unsigned char[dwLen + 1];
      RegQueryValueEx (hkey, "TFTRANS", 0, &dwType, transactionid, &dwLen);
      string strTransactionID = (char*)transactionid;
      delete[] transactionid;

      //get the MAC address
      string strMACAddr = get_mac_addr();

      // Convert MAC address to upper case
      transform(strMACAddr.begin(), strMACAddr.end(), strMACAddr.begin(), toupper);

      // JSH TEMP
      //sysLog->Print(" license: %s\n numports: %s\n expdate: %s\n tftrans: %s\n MAC: %s\n", strTFLicense.c_str(), strNumPorts.c_str(), strExpDate.c_str(), strTransactionID.c_str(), strMACAddr.c_str());

      // BEGIN COMPARE LICENSES

      // has registry values
      string strHashString;
      string strHashedTFLicense;

      strHashString = strNumPorts;
      strHashString += strMACAddr;
      strHashString += strExpDate;
      strHashString += strTransactionID;
      strHashString += TFBASE;

      strHashedTFLicense = MD5Hash(strHashString.c_str(), true);

      if(strTFLicense == strHashedTFLicense)
      {
       bLicensed = true;
      }

      // END COMPARE LICENSES

      if(bLicensed)
      {

      // BEGIN EXPIRARY DATE COMPARISON
   
      struct date currDate;
      getdate(&currDate);
      string strCurrDate;

      char cYear[5];
      char cMon[3];
      char cDay[3];

      sprintf(cYear, "%i", (int)currDate.da_year);
      strCurrDate = cYear;

      if( currDate.da_mon < 10)
      {
       sprintf(cMon, "%s%i", "0", (int)currDate.da_mon);
      }
      else
      {
       sprintf(cMon, "%i", (int)currDate.da_mon);
      }
      strCurrDate += cMon;

      if( currDate.da_day < 10)
      {
       sprintf(cDay, "%s%i", "0", (int)currDate.da_day);
      }
      else
      {
       sprintf(cDay, "%i", (int)currDate.da_day);
      }
      strCurrDate += cDay;

      iCurrDate = atoi(strCurrDate.c_str());

      // expired license
      if(iCurrDate > iExpDate)
      {
       bLicensed = false;
      }

      // temporary license
      if(iCurrDate <= iExpDate && iExpDate != 99991231)
      {
       bLicensed = true;
       bTempLicense = true;
      }

      // permanent license
      if(iExpDate == 99991231)
      {
       bLicensed = true;
       bTempLicense = false;

      }

      // END EXPIRARY DATE COMPARISON

      } // end if(bLicensed)

   } // end if(strTFLicense != NULL)

   // **** Log SR audio *******************************************************
   dwLen = sizeof(DWORD);
   if(RegQueryValueEx(hkey, "LogSRAudio", 0, &dwType, (LPBYTE) &(logSRAudio), &dwLen) != ERROR_SUCCESS)
   {
      // Off by default
      logSRAudio = 0;
   }

   RegCloseKey (hkey);

   // end of registry fetch

   // server level syslong messages for licensing
   if(!bLicensed)
   {
    sysLog->Print("This is an unlicensed or evaluation version of Teleflow.\n");
   }
   else if(bLicensed)
   {
    sysLog->Print("This installation of Teleflow is licensed for %i ports.\n", iNumports);
   }

   if(bTempLicense && bLicensed)
   {
    sysLog->Print("This is an temporary licensed version of Teleflow.\n");
    sysLog->Print("There are %i days left on your license before this product reverts to an evaluation version.\n", (iExpDate - iCurrDate));
   }

   if((fTAL = fopen (pszFileNm, "r")) == NULL)		// Unable to open file.
   {
      sysLog->Print("ERROR: Unable to open TeleFlow Application List file\n");
      sysLog->Print("    '%s'.  Please ensure that the Active\n", pszFileNm);
      sysLog->Print("    TeleFlow Application List file is correctly specified in TFConfig.\n");
      return(false);
   }

   ch = fgetc (fTAL);
   iFieldNo = 1;
   iOffset = 0;

	while(!feof(fTAL))
	{
      if(ch == '\t' || ch == '\r' || ch == '\n') // ----- Found end of field
      {
         szLine[iOffset] = 0;

         // Version filter
         // - current version is now 4 - TF Feb.19/02
         switch(version)
         {
         	case 1:
            {
            	switch(iFieldNo)
               {
               	case 5:	iFieldNo = 6;	break;
               	case 7: iFieldNo = 11; break;
               }
            }

            case 2:
            {
            	switch(iFieldNo)
               {
               	case 5: iFieldNo = 6; break;
               	case 10: iFieldNo = 11; break;
               }
            }

            case 3:
            {
            	switch(iFieldNo)
               {
               	case 10: iFieldNo = 11; break;
               }
            }
         }

         switch(iFieldNo)
         {
            case 1:  // ----------- FIELD 1: --------------------------- iAppID
            {
               temp = new IVRApp(dwLogLines, dwMaxLogFileSize);
               temp->data.iAppID = atoi(szLine);
               break;
            }

            case 2:  // ----------- FIELD 2: -------------------- pszDebugParam
            {
               temp->data.pszDebugParam = new char[iOffset + 1];
               strcpy(temp->data.pszDebugParam, szLine);
               break;
            }

            case 3:  // ----------- FIELD 3: ------------------------- pszAppNm
            {
					// Do not copy more than allowable TELEFLOW_APP_NAME_MAX characters as defined in ivr_defs.h
               temp->data.pszAppNm = new char[TELEFLOW_APP_NAME_MAX + 1];
               strncpy(temp->data.pszAppNm, szLine, TELEFLOW_APP_NAME_MAX);
               temp->data.pszAppNm[TELEFLOW_APP_NAME_MAX] = '\0';
               break;
            }

            case 4:  // ----------- FIELD 4: ------------------------ pszFileNm
            {
               temp->data.pszAppFile = new char[iOffset + 1];
               strcpy(temp->data.pszAppFile, szLine);
               break;
            }

            case 5:  // ----------- FIELD 5: ---------------------- pszProtocol
         	{
               temp->data.pszProtocol = new char[iOffset + 1];
               strcpy(temp->data.pszProtocol, szLine);
               break;
            }

            case 6:  // ----------- FIELD 6: -------------------- iRestartCount
         	{
               temp->data.restartMax = atoi(szLine);
               bHaveApp = true;
               break;
            }

            case 7:  // ----------- FIELD 7: ----------------- bRestartInfinite
         	{
               temp->data.bRestartInfinite = (szLine[0] == 'Y') ? true : false;
               break;
            }

            case 8:  // ----------- FIELD 8: ----------------------- bLogOutput
            {
               temp->data.bLogOutput = (szLine[0] == 'Y') ? true : false;
               break;
            }

            case 9:  // ----------- FIELD 9: ----------------------- bLogAppend
            {
               temp->data.bLogAppend = (szLine[0] == 'Y') ? true : false;
               break;
            }

				case 10: // ----------- FIELD 10: ---------------------- iPort
            {
               temp->data.iPort = temp->data.clientMsg.lineNo = atoi(szLine);

               if(atoi(szLine) <= iNumports && bLicensed == true)
               {
                  if(bTempLicense)
                  {
                     temp->data.portLicense = TEMPORARY;
                     sysLog->Print("Port %i has a temporary license\n", atoi(szLine));
                  }
                  else
                  {
                     temp->data.portLicense = LICENSED;
                     sysLog->Print("Port %i has a full license\n", atoi(szLine));
                  }
               }

               if(atoi(szLine) > iNumports && bLicensed == true)
               {
                  // this should REALLY be in the application log... extend it as a class
                  // (that way ALL syslog info should be logged to the app log)
                  sysLog->Print("The total number of ports in the application file is greater than the number of ports licensed.\n");
                  sysLog->Print("%i ports are licensed and will continue to run without the Teleflow Evaluation message\n", iNumports);

                  // NEED TO DO A CHECK AGAINST THE DATE TO DETERMINE WHETHER OR NOT THIS IS UNLICENSED OR TEMPORARY
                  temp->data.portLicense = UNLICENSED;
               }

               if(!bLicensed)
               {
                  sysLog->Print("Port %i is an unlicensed port\n", atoi(szLine));
                  temp->data.portLicense = UNLICENSED;
               }

               // check protocol to determine where our SIP channels start
               // In TeleFlow SIP prototype all ports >= this number are deemed to be SIP ports
               // - TF 2005/08/03
               if((SIPStarts == 0) && (stricmp(temp->data.pszProtocol, "sip0") == 0))
               {
                  SIPStarts = temp->data.iPort;
               }

               break;
            }

            default: // ----------- FIELDS 11-19: ----- Optional params (max 9)
            {
               temp->data.pszParams[iFieldNo - 11] = new char[iOffset + 1];
               strcpy(temp->data.pszParams[iFieldNo - 11], szLine);
               temp->data.iParamCount++;
            }
         }

         // Reset variables to receive next field.
         iOffset = 0;
         if((ch == '\r') || (ch == '\n'))
         {
            iFieldNo = 1;

            if(bHaveApp)
            {
     	         if(pszLogDir != NULL)
               {
        	         temp->data.pszLogDir = new char[strlen (pszLogDir) + 1];
           	      strcpy(temp->data.pszLogDir, pszLogDir);
              	}
               else
               {
                 	temp->data.pszLogDir = new char[2];
                  strcpy(temp->data.pszLogDir, "\\");
  	            }

     	         if(pszInstallationPath != NULL)
               {
                 	temp->data.pszInstallationPath = new char[strlen(pszInstallationPath) + 1];
                  strcpy(temp->data.pszInstallationPath, pszInstallationPath);
  	            }

					// Fields not in earlier version files
					switch(version)
               {
						case 1:
                  {
			            temp->data.bRestartInfinite = false;
      			      temp->data.bLogOutput = true;
         	   		temp->data.bLogAppend = true;
                     // Fall through
                  }
                  case 2:
                  {
							temp->data.pszProtocol = new char[5];
							strcpy(temp->data.pszProtocol, "lps0");
                     // Fall through
                  }
                  case 3:
                  {
							if(temp->data.pszParams[0])
                     {
								temp->data.iPort = atoi(temp->data.pszParams[0]);
                     }
                     else
                     {
                     	temp->data.iPort = 0;
                     }
                  }
               }

               temp->data.hLoadingMutex = hLoadingMutex;
               temp->data.boardLibrary = boardLibrary;
               temp->data.logSRAudio = logSRAudio;

					// Make @P1 to @P2 always exist
  	            temp->data.iParamCount   = 9;

        	      if(temp->data.bLogOutput)
               {
           	      strcpy(szLine, pszLogDir);
                 	strcat(szLine, temp->data.pszAppNm);
  	               temp->data.ivLog->Open(szLine, temp->data.bLogAppend, true);
     	         }

        	      for(int i = temp->data.iParamCount; i < 9; i++)
					{
              		temp->data.pszParams[i] = "";
               }

           	   AddLast((Link*)temp);

               temp = NULL;

               bHaveApp = false;
            }
            else
            {
               delete temp;
               temp = NULL;
            }
         }
         else
         {
            ++iFieldNo;
         }
      }
      else
      {
         // Build up field data until the end of the field is found.
         szLine[iOffset] = (char) ch;
         ++iOffset;
      }
      ch = fgetc (fTAL);
   } // while

   delete temp;
   fclose(fTAL);

   if(Empty())
   {
      sysLog->Print("ERROR: No applications were loaded from the TeleFlow Application List file\n");
      sysLog->Print("   '%s'.   Please verify\n", pszFileNm);
      sysLog->Print("   the Active TeleFlow Application List file in TFLineList.\n");
      return(false);
   }

   if(pFirstSIPPort != NULL)
   {
      *pFirstSIPPort = SIPStarts;
   }

   return(true);
}
