/* -------------------------------------------------------------------------
   File:        NT_SERVER.CPP

   Purpose:
   ------------------------------------------------------------------------- */
#include <winsock2.h>
#include <windows.h>
#include <process.h>
#include <stdio.h>
#include <time.h>
#include "nt_service.h"
#include "..\escutil\link.h"
#include "ace\Init_ACE.h"
#include "brd_load.h"
#include "file_log.h"
#include "threadlink.h"
#include "TFServer_communication.h"
#include "TFEngine_communication_manager.h"
#include "TFApplicationsManager.h"
#include "TFServerRegistrationInfo.h"

// for error msgs
#include "..\TFMessages\tf_msgs.h"

#include <string>
using std::string;
#include <fstream>
using std::ofstream;

#define MAX_CLIENTS  500

//#define SERVICE_FILE_NM     TEXT ("%SystemRoot%\\system\\TFServer.exe")
//#define SERVICE_FILE_NM     TEXT ("d:\\teleflow\\bin\\TFServer.exe")
#define SERVICE_DISPLAY_NM  TEXT ("TeleFlow Server")
#define SERVICE_NM          TEXT ("TFServer")

// 2003-06-19 JSH: stuff to create unique instance locking....
// this code is from MS Knowledge Base http://support.microsoft.com/default.aspx?scid=kb%3Ben-us%3B243953
// Article Q243953 (in case URL doesn't work)
//
class CLimitSingleInstance
{
protected:
  DWORD  m_dwLastError;
  HANDLE m_hMutex;



public:

  CLimitSingleInstance(TCHAR *strMutexName)
  {
    //be sure to use a name that is unique for this application otherwise
    //two apps may think they are the same if they are using same name for
    //3rd parm to CreateMutex
    m_hMutex = CreateMutex(NULL, FALSE, strMutexName); //do early
    m_dwLastError = GetLastError(); //save for use later...
  }

  ~CLimitSingleInstance()
  {
    if (m_hMutex)  //don't forget to close handles...
    {
       CloseHandle(m_hMutex); //do as late as possible
       m_hMutex = NULL; //good habit to be in
    }
  }

  BOOL IsAnotherInstanceRunning()
  {
     int myReturn = m_dwLastError;
     return ((ERROR_ALREADY_EXISTS == myReturn) || (ERROR_ACCESS_DENIED == myReturn));
  }
};

// 2003-06-04 JSH: struct for reading reg values when installing
typedef struct
{
   char* pszBoardService;
   char* pszInstallDir;  // JSH 2003-07-21
} REG_SERVICE_INFO;

SERVICE_STATUS          MyServiceStatus;
SERVICE_STATUS_HANDLE   MyServiceStatusHandle;
HANDLE                  hEventSD;
bool                    bConsoleMode;

VOID SvcDebugOut (LPSTR String, DWORD Status);
VOID _stdcall MyServiceStart (DWORD argc, LPTSTR *argv);
VOID _stdcall MyServiceCtrlHandler (DWORD Opcode);
DWORD MyServiceInitialization (DWORD argc, LPTSTR *argv, DWORD *specificError);
VOID CreateEngineService (const char *pszLocation);
VOID DeleteEngineService();
VOID UpdateStatusStart (DWORD dwState, DWORD *dwCheckPoint, DWORD dwWaitHint);
VOID UpdateStatusProcessing (DWORD *dwCheckPoint);
VOID ControlEngineService (DWORD fdwControl);
VOID UpdateStatusDone (
   DWORD dwState,
   DWORD dwW32ExitCode = NO_ERROR,
   DWORD dwSrvExitCode = 0);
unsigned __stdcall EngineServerMain (void *arg);
static BOOL WINAPI DebugControlHandler (DWORD dwCtrlType);
void LogEvent (WORD errtype, DWORD errcode, const char* szMsg);
char* ServerStartPause(FileLog* psysLog);
bool SetServerPriority(FileLog* psysLog);

/* -------------------------------------------------------------------------
   Name:        PrintErrorMsg
   ------------------------------------------------------------------------- */
void PrintErrorMsg (const char *fn)
{
   DWORD  errorID = GetLastError();
   char   MsgBuf[1024];

   FormatMessage (
            FORMAT_MESSAGE_FROM_SYSTEM,
            0,
            errorID,
            MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
            MsgBuf,
            1024,
            0);
   printf ("%s returned:\n%x: %s", fn, errorID, MsgBuf);
}
/*PrintErrorMsg*/

/* -------------------------------------------------------------------------
   Name:        NTServiceMain

   Purpose:     Starts service running.

                If the first parameter is "INSTALL", the service is
                installed.

                If the first parameter is "REMOVE", the service is
                uninstalled.
   ------------------------------------------------------------------------- */
void NTServiceMain (int argc, char *argv[])
{
	/*
      2003-06-20 JSH
         - re-inserted code for running TFServer as a service
         - mutex code needs to be called within scope of service or console start otherwise it gets called when installing the service
   */

   if(argc == 1) // run TFServer from the console
   {
      // JSH: set up the mutex to determine if another instance is running
      // -this value genereated by the GUIDGEN tool using its 'Registry Format'
      CLimitSingleInstance g_SingleInstanceObj(TEXT(MUTEX_GUID));

      // printf("Welcome to Teleflow Server...\n");

      if (g_SingleInstanceObj.IsAnotherInstanceRunning())
		{
			printf("TFServer is already running, either in another console or as a service. Process terminating.\n");
         ExitProcess(1);
		}
      else
      {
         printf("TFServer running in console mode.\n");
      }



      hEventSD = CreateEvent (NULL, false, false, NULL);
      bConsoleMode = true;
      SetConsoleCtrlHandler (DebugControlHandler, true);
      EngineServerMain (0); // MyServiceInitialization (0, 0, 0);
      CloseHandle (hEventSD);
   }
   else if (!stricmp (argv[1], "SERVICE"))
   {
      // JSH: set up the mutex to determine if another instance is running
      CLimitSingleInstance g_SingleInstanceObj(TEXT(MUTEX_GUID));

      if (g_SingleInstanceObj.IsAnotherInstanceRunning())
      {
         LogEvent(EVENTLOG_ERROR_TYPE, TFSERVER_ALREADY_RUNNING, NULL);

         ControlEngineService(SERVICE_CONTROL_STOP);
      }
      else
      {

         LogEvent(EVENTLOG_INFORMATION_TYPE, TFSERVER_STARTED, NULL);

      }

      SERVICE_TABLE_ENTRY  DispatchTable[] = {
         { SERVICE_NM,         MyServiceStart },
         { NULL,               NULL           }
      };

      hEventSD = CreateEvent (NULL, false, false, NULL);

      if (!StartServiceCtrlDispatcher (DispatchTable))
      {
         PrintErrorMsg ("NTServiceMain");
         printf ("           To install TFSERVER as a service, specify "
                 "INSTALL as the parameter.\n           To remove TFSERVER "
                 "as a service, specify REMOVE as the parameter.\n");
      }


      CloseHandle (hEventSD);
   }
   else if (!stricmp (argv[1], "INSTALL")) {
      if (argc > 2)
         CreateEngineService (argv[2]);
      else
         printf ("SYNTAX: TFServer.exe INSTALL <location of TFServer.exe>\n"
                 "E.G. tf_server.exe INSTALL "
                 "c:\\winnt\\system32\\TFServer.exe SERVICE\n");

   } else if (!stricmp (argv[1], "REMOVE")) {
      DeleteEngineService();

   } else if (!stricmp (argv[1], "DEBUG")) {
      hEventSD = CreateEvent (NULL, false, false, NULL);
      bConsoleMode = true;
      SetConsoleCtrlHandler (DebugControlHandler, true);
      EngineServerMain (0); // MyServiceInitialization (0, 0, 0);
      CloseHandle (hEventSD);
   }
}
/*NTServiceMain*/

/* -------------------------------------------------------------------------
   Name:        DebugControlHandler

   Purpose:     This function is used when debugging the service.  It won't
                be used when the service is live.  This function is
                installed as a Ctrl-C handler to simulate the service STOP
                code.
   ------------------------------------------------------------------------- */
BOOL WINAPI DebugControlHandler (DWORD dwCtrlType)
{
    switch (dwCtrlType)
    {
        case CTRL_BREAK_EVENT:  /* use Ctrl+C or Ctrl+Break to simulate */
        case CTRL_C_EVENT:      /* SERVICE_CONTROL_STOP in debug mode   */
            printf ("Stopping %s.\n", SERVICE_DISPLAY_NM);

            /* Behave as if told to shut down by SCM: */
            MyServiceCtrlHandler (SERVICE_CONTROL_STOP);

            return TRUE;
    }

    return FALSE;

}
/*DebugControlHandler*/

/* -------------------------------------------------------------------------
   Name:        MyErrorExit

   Purpose:     This is *NOT* called by the service.  This is called by
                functions to manipulate the service from programs started
                from the command line and not the SCM.
   ------------------------------------------------------------------------- */
VOID MyErrorExit (LPSTR String)
{
   printf ("ERROR: %s (%d)\n", String, GetLastError());
   exit (0);
}
/*MyErrorExit*/

/* -------------------------------------------------------------------------
   Name:        SvcDebugOut
   ------------------------------------------------------------------------- */
VOID SvcDebugOut (LPSTR String, DWORD Status)
{
   CHAR  Buffer[1024];
   if (strlen (String) < 1000) {
      sprintf (Buffer, String, Status);
      OutputDebugStringA (Buffer);
   }
}
/*SvcDebugOut*/

/* -------------------------------------------------------------------------
   Name:        MyServiceStart

   Purpose:     Starts the service running and then waits for the service to
                terminate or be terminated.

   Notes:       - This service does not support pausing.
   ------------------------------------------------------------------------- */
VOID _stdcall MyServiceStart (DWORD argc, LPTSTR *argv)
{
   DWORD   status;
   DWORD   specificError;


   MyServiceStatus.dwServiceType             = SERVICE_WIN32;
   MyServiceStatus.dwCurrentState            = SERVICE_START_PENDING;
   MyServiceStatus.dwControlsAccepted        = SERVICE_ACCEPT_STOP | SERVICE_ACCEPT_SHUTDOWN;
//                                           | SERVICE_ACCEPT_PAUSE_CONTINUE;
   MyServiceStatus.dwWin32ExitCode           = 0;
   MyServiceStatus.dwServiceSpecificExitCode = 0;
   MyServiceStatus.dwCheckPoint              = 0;
   MyServiceStatus.dwWaitHint                = 0;
   MyServiceStatusHandle = RegisterServiceCtrlHandler (
                              SERVICE_NM,
                              MyServiceCtrlHandler);

   if (MyServiceStatusHandle == (SERVICE_STATUS_HANDLE)0) {
      SvcDebugOut (" [MY_SERVICE] RegisterServiceCtrlHandler failed %d\n",
                   GetLastError());
      return;
   }

   // Initialization code goes here.
   status = MyServiceInitialization (argc, argv, &specificError);

   // Handle error condition
   if (status != NO_ERROR) {
      MyServiceStatus.dwCurrentState            = SERVICE_STOPPED;
      MyServiceStatus.dwCheckPoint              = 0;
      MyServiceStatus.dwWaitHint                = 0;
      MyServiceStatus.dwWin32ExitCode           = status;
      MyServiceStatus.dwServiceSpecificExitCode = specificError;
      SetServiceStatus (MyServiceStatusHandle, &MyServiceStatus);
      return;
   }

   // Initialization complete - report running status.

   MyServiceStatus.dwCurrentState       = SERVICE_RUNNING;
   MyServiceStatus.dwCheckPoint         = 0;
   MyServiceStatus.dwWaitHint           = 0;

   if (!SetServiceStatus (MyServiceStatusHandle, &MyServiceStatus)) {
      status = GetLastError();
      SvcDebugOut (" [MY_SERVICE] SetServiceStatus error %ld\n",status);
   }

   // This is where the service does its work.
   // WaitForSingleObject (hEventSD, INFINITE);

   SvcDebugOut (" [MY_SERVICE] Returning the Main Thread \n", 0);
   return;
}
/*MyServiceStart*/

/* -------------------------------------------------------------------------
   Name:        MyServiceInitialization

   Purpose:     Performs any initialization required to start the service.

                If this takes longer than a second it is necessary to send
                status updates periodically to indicate progress is being
                made.

                If initialization is successful, return NO_ERROR.
   ------------------------------------------------------------------------- */
DWORD MyServiceInitialization (
   DWORD   , // argc,
   LPTSTR* , // argv,
   DWORD* ) // specificError)
{
   unsigned  threadId;
   HANDLE    hThread;

   hThread = (HANDLE) _beginthreadex (
      NULL,                   // NT security attributes
      0,                      // stack size
      EngineServerMain,       // start address
      NULL,                   // argument list
      0,                      // Thread creation flags
      &threadId);             // Thread id
   CloseHandle (hThread);

   return 0;
}
/*MyServiceInitialization*/

/* -------------------------------------------------------------------------
   Name:        UpdateStatusStart
   ------------------------------------------------------------------------- */
VOID UpdateStatusStart (DWORD dwState, DWORD *dwCheckPoint, DWORD dwWaitHint)
{
   DWORD   status;

   if (bConsoleMode)
      return;

   MyServiceStatus.dwCurrentState = dwState;
   MyServiceStatus.dwWaitHint     = dwWaitHint;
   *dwCheckPoint = 1;
   MyServiceStatus.dwCheckPoint   = *dwCheckPoint;

   if (!SetServiceStatus (MyServiceStatusHandle, &MyServiceStatus)) {
      status = GetLastError();
      SvcDebugOut (" [MY_SERVICE] SetServiceStatus error %ld\n", status);
   }
}
/*UpdateStatusStart*/

/* -------------------------------------------------------------------------
   Name:        UpdateStatusProcessing
   ------------------------------------------------------------------------- */
VOID UpdateStatusProcessing (DWORD *dwCheckPoint)
{
   DWORD   status;

   if (bConsoleMode)
      return;

   (*dwCheckPoint)++;
   MyServiceStatus.dwCheckPoint = *dwCheckPoint;

   if (!SetServiceStatus (MyServiceStatusHandle, &MyServiceStatus)) {
      status = GetLastError();
      SvcDebugOut (" [MY_SERVICE] SetServiceStatus error %ld\n", status);
   }
}
/*UpdateStatusProcessing*/

/* -------------------------------------------------------------------------
   Name:        UpdateStatusDone
   ------------------------------------------------------------------------- */
VOID UpdateStatusDone (DWORD dwState, DWORD dwW32ExitCode, DWORD dwSrvExitCode)
{
   DWORD   status;

   if (bConsoleMode)
      return;

   MyServiceStatus.dwCurrentState            = dwState;
   MyServiceStatus.dwWaitHint                = 0;
   MyServiceStatus.dwCheckPoint              = 0;
   MyServiceStatus.dwWin32ExitCode           = dwW32ExitCode;
   MyServiceStatus.dwServiceSpecificExitCode = dwSrvExitCode;

   if (!SetServiceStatus (MyServiceStatusHandle, &MyServiceStatus)) {
      status = GetLastError();
      SvcDebugOut (" [MY_SERVICE] SetServiceStatus error %ld\n", status);
   }

    // CloseHandle(hTFServerMap); // JSH MARK: close the map handle
}
/*UpdateStatusDone*/

/* -------------------------------------------------------------------------
   Name:        EngineServerMain

   Purpose:     Initializes the service, starts Engine threads, and then
                monitors events until service terminates.  If threads
                terminate, they are restarted unless the service is shutting
                down.

   Steps:       [X] Load registry.
                [X] Load DLL(s).
                [-] Load INI.
                [X] Load TAL.
                [X] Create event queue to allow threads to inform this
                    thread of special events.
                [-] Spawn TCCS monitor thread.
                [-] Spawn TCP/IP monitor thread.
                [X] Spawn Engine threads.
                    [X] Create Log.
                    [X] Populate parameter structure.
                    [X] Create event to inform thread to terminate.
                    [-] Assign monitor array element.
                    [X] Create thread.
                [X] Wait for events.
                    [X] Monitor service shutdown event.
                    [X] Monitor thread events.
                        [-] TCP/IP crashes.
                        [-] TCCS crashes.
                        [X] Engine crashes.
                [X] Shutdown service.
                    [X] Attempt to shutdown threads (wait 20 seconds).
                    [ ] Termiate threads (after 20 seconds expire).
   ------------------------------------------------------------------------- */
unsigned __stdcall EngineServerMain (void * /*arg*/)
{
   DWORD                  dwCheckPoint;
   bool                   bContinue;
   DWORD                  ret;
   TFServerRegistrationInfo regInfo;
   LARGE_INTEGER          waitTimeout;
   HANDLE                 hWaitTimer;
   HANDLE                 hMonThread;
   HANDLE                 waitObjs[2];
   HINSTANCE              boardLibrary;
   unsigned               threadId;
   FileLog                sysLog;
   CommunicationHub<TFServerCommunicationPacket::TF_DESTINATION_ID> commHub;
   TFServerCommunicationDestination TFServerMessages;
   TCPClientManager* tcpClientManager;
   TFApplicationsManager* applicationsManager;

   commHub.Add(&TFServerMessages);

   // **** INITIALIZING *******************************************************
   UpdateStatusStart(SERVICE_START_PENDING, &dwCheckPoint, 1000);

   if(!regInfo.OpenSystemLog(sysLog) || !regInfo.Load(sysLog))
   {
      UpdateStatusDone(SERVICE_STOPPED, ERROR_SERVICE_SPECIFIC_ERROR, TFS_NO_REGISTRY_ENTRIES);
      sysLog.Print("Error code: %d (no registry entries)", TFS_NO_REGISTRY_ENTRIES);
      return(0);
   }

   UpdateStatusProcessing (&dwCheckPoint);

   sysLog.Print("Checking Windows up-time.\n");
   regInfo.pszTFStart = ServerStartPause(&sysLog);
   // Deliberate shallow-copy; saves the TFSTART value with the rest of the registry info

   sysLog.Print("Adjusting process settings\n");
   // return result (boolean) currently ignored; if it fails, Windows default will be used instead
   SetServerPriority(&sysLog);

   sysLog.Print ("Dynamically loading board DLL at run-time.\n");
   if((boardLibrary = LoadBoardFunctions(&sysLog, regInfo.pszBoardDLL)) == NULL)
   {
      UpdateStatusDone (SERVICE_STOPPED, ERROR_SERVICE_SPECIFIC_ERROR,
                        TFS_BOARD_DLL_PROBLEM);
      sysLog.Print("Error code: %d (board DLL problem)", TFS_BOARD_DLL_PROBLEM);
      return 0;
   }

   UpdateStatusDone(SERVICE_RUNNING, NO_ERROR, 0);

   sysLog.Print("Board DLL: %s\n", regInfo.pszBoardDLL);
   if(OpenHardwareDriver != NULL)
   {
   	if(OpenHardwareDriver(&sysLog) != true)
      {
			sysLog.Print("Warning: Could not start telephony hardware driver!\n");
      }
   }

   if((ret = Socket::NetStartup()) != Socket::SUCCESS)
   {
      UpdateStatusDone(SERVICE_STOPPED, ERROR_SERVICE_SPECIFIC_ERROR, TFS_WINSOCK_INIT_FAILED);
      sysLog.Print("ERROR: Could not start Winsock: '%s'\n", Socket::WSAErrorString(ret));
      return(0);
   }

   sysLog.Print ("Starting Monitor thread...\n");

   tcpClientManager = new TCPClientManager((unsigned short) regInfo.dwMonitorPort,
                                           (unsigned short) regInfo.dwMonitorHTTPPort,
                                           commHub,
                                           &regInfo);
   commHub.Add(tcpClientManager);

   hMonThread = (HANDLE) _beginthreadex (
      NULL,               // NT security attributes
      0,                  // stack size (0 = use same size as this thread)
      TCPClientManagerMain,  // start address
      tcpClientManager,           // argument list
      0,                  // Thread creation flags
      &threadId);         // Thread id
   // WAS:
   /*
   CloseHandle (hMonThread);
   */
   // Why were we closing a handle we just created? - TF 2008-04-17
   // Moved to the end of EngineServerMain(), shutdown code - TF 2008-06-16

   sysLog.Print ("Creating application manager\n");
   applicationsManager = new TFApplicationsManager(&commHub, tcpClientManager, sysLog, boardLibrary);
   commHub.Add(applicationsManager);

   sysLog.Print ("Loading .TAL file into application list structure.\n");
   if(!applicationsManager->LoadTAL())
   {
      UpdateStatusDone (SERVICE_STOPPED, ERROR_SERVICE_SPECIFIC_ERROR,
                        TFS_LOAD_APP_LIST_FAILED);
      sysLog.Print("Error code: %d (loading Application List failed)", TFS_LOAD_APP_LIST_FAILED);
      return 0;
   }

   // **** RUNNING ************************************************************
   sysLog.Print ("Starting Engine threads...\n");
   applicationsManager->StartApplications();

   waitObjs[0] = hEventSD;
   waitObjs[1] = TFServerMessages.GetMessageSignal();

   sysLog.Print ("Waiting for events from threads.\n");
   bContinue = true;
   while (bContinue)                   // Loop until service shuts down.
   {
      ret = WaitForMultipleObjects (2, waitObjs, false, INFINITE);
      if (ret == WAIT_FAILED) {
         sysLog.Print ("Wait failed.  Terminating service...\n");
         UpdateStatusDone (SERVICE_STOPPED, ERROR_SERVICE_SPECIFIC_ERROR,
                           TFS_WAIT_FAILED);
         return 0;
      }

      if (ret == WAIT_OBJECT_0) {      // Service is shutting down.
         sysLog.Print ("Service shutdown requested.\n");
         bContinue = false;
      }

      else if (ret == WAIT_OBJECT_0 + 1)
      {
         TFServerCommunicationPacket* message = (TFServerCommunicationPacket*) TFServerMessages.ReceiveMessage();

         if(message != NULL)
         {
   		   sysLog.Print ("Received Server event: %d\n", message->event);
            switch(message->event)
            {


            	case TFServerCommunicationPacket::SRV_EVT_ENG_THRD_DONE:
               {
            	   sysLog.Print("Engine thread completed successfully.\n");
              	   sysLog.Print("Starting new Engine thread.\n");
                  applicationsManager->StartApplication(message->appID);
               	break;
               }

   	      	case TFServerCommunicationPacket::SRV_EVT_ENG_TERMINATED:
               {
            	   sysLog.Print("Engine thread terminated\n");
   	            break;
               }

            	case TFServerCommunicationPacket::SRV_EVT_ENG_FATAL_ERROR:
               {
            	   sysLog.Print("Engine thread terminated fatally\n");
   	            break;
               }

	            case TFServerCommunicationPacket::SRV_EVT_MON_TERMINATED:
      	      case TFServerCommunicationPacket::SRV_EVT_MON_FATAL_ERROR:
               {
         	      sysLog.Print ("Monitor thread terminated\n");
               	break;
               }

               case TFServerCommunicationPacket::SRV_EVT_RELOAD_TAL:
               {
         	      sysLog.Print ("Reload TAL requested\n");
		   		   // Need a waitable timer here to ensure it doesn't take longer than 20
			   	   // seconds.
				      hWaitTimer = CreateWaitableTimer (0, false, 0);
				      waitTimeout.QuadPart = -(LONGLONG) 5 * 10000000;
   				   SetWaitableTimer(hWaitTimer, &waitTimeout, 0, 0, 0, false);
	   			   waitObjs[0] = hWaitTimer;
		   			ret = !WAIT_OBJECT_0;
			   	   while(!applicationsManager->Empty() && (ret != WAIT_OBJECT_0))
                  {
				         ret = WaitForMultipleObjects(2, waitObjs, false, 1000);
				         if(ret == WAIT_FAILED)
                     {
	   			         sysLog.Print ("Wait failed.  Terminating service...\n");
		   		         UpdateStatusDone (SERVICE_STOPPED, ERROR_SERVICE_SPECIFIC_ERROR,
                              TFS_WAIT_FAILED);
				            return 0;
				         }

   				      UpdateStatusProcessing (&dwCheckPoint);

		   		      if(ret == WAIT_OBJECT_0 + 1)
                     {
                        TFServerCommunicationPacket* message = (TFServerCommunicationPacket*) TFServerMessages.ReceiveMessage();

                        if(message != NULL)
                        {
   			   	         switch(message->event)
	   			            {
		   			            case TFServerCommunicationPacket::SRV_EVT_ENG_THRD_DONE:
			   		            case TFServerCommunicationPacket::SRV_EVT_ENG_TERMINATED:
				   	            case TFServerCommunicationPacket::SRV_EVT_ENG_FATAL_ERROR:
                              {
                                 applicationsManager->RemoveApplication(message->appID);
		   			               break;
   		   		            }
                           }
                        }

                        delete message;
                     }
				         if(applicationsManager->Empty() || (ret == WAIT_OBJECT_0))
				         {
				            CancelWaitableTimer (hWaitTimer);
   				      }

	   			      if (ret == WAIT_TIMEOUT)
                     {
			            	sysLog.Print ("----- TIMEOUT -----\n");
				            printf ("----- TIMEOUT -----\n");
           		   		sysLog.Print("There is atleast one engine thread still running.\n");
			               printf ("There is atleast one engine thread still running.\n");
   				      }
	   			   }

   	            CancelWaitableTimer (hWaitTimer);
				      waitObjs[0] = hEventSD;

                  sysLog.Print ("Loading .TAL file into application list structure.\n");
                  if(!applicationsManager->LoadTAL())
				      {
				         UpdateStatusDone (SERVICE_STOPPED, ERROR_SERVICE_SPECIFIC_ERROR,
                           TFS_LOAD_APP_LIST_FAILED);
   				      return 0;
	   			   }
		   		   UpdateStatusProcessing (&dwCheckPoint);

				      sysLog.Print ("Starting Engine threads...\n");
                  applicationsManager->StartApplications();

                  break;
               }
            }

            delete message;
         }
      }
   }
   sysLog.Print ("**** SERVICE SHUTTING DOWN!!! ****\n");
   printf ("**** SERVICE SHUTTING DOWN!!! ****\n");

   // **** SHUTTING DOWN ******************************************************
   UpdateStatusStart (SERVICE_STOP_PENDING, &dwCheckPoint, 10000);

   // Tell IVR threads to terminate.
   applicationsManager->StopApplications();;

   // Need a waitable timer here to ensure it doesn't take longer than 20
   // seconds.
   hWaitTimer = CreateWaitableTimer (0, false, 0);
   waitTimeout.QuadPart = -(LONGLONG) 5 * 10000000;
   SetWaitableTimer (hWaitTimer, &waitTimeout, 0, 0, 0, false);
   waitObjs[0] = hWaitTimer;

   for(;;)
   {
      if((ret = WaitForMultipleObjects(2, waitObjs, false, 1000)) == WAIT_FAILED)
      {
         sysLog.Print("Wait failed.  Terminating service...\n");
         UpdateStatusDone(SERVICE_STOPPED, ERROR_SERVICE_SPECIFIC_ERROR, TFS_WAIT_FAILED);
         return(0);
      }

      if(ret == WAIT_OBJECT_0 + 1)
      {
         TFServerCommunicationPacket* message = (TFServerCommunicationPacket*) TFServerMessages.ReceiveMessage();

         if(message != NULL)
         {
            switch(message->event)
            {
               case TFServerCommunicationPacket::SRV_EVT_ENG_THRD_DONE:
               case TFServerCommunicationPacket::SRV_EVT_ENG_TERMINATED:
               case TFServerCommunicationPacket::SRV_EVT_ENG_FATAL_ERROR:
               {
                  sysLog.Print ("   Application %i terminated\n", message->appID);
                  applicationsManager->RemoveApplication(message->appID);
                  break;
               }

               case TFServerCommunicationPacket::SRV_EVT_MON_THRD_DONE:
               case TFServerCommunicationPacket::SRV_EVT_MON_TERMINATED:
               case TFServerCommunicationPacket::SRV_EVT_MON_FATAL_ERROR:
               {
                  sysLog.Print ("   Monitor thread terminated\n");
                  break;
               }
            }

            delete message;
         }
      }

      if(applicationsManager->Empty() || (ret == WAIT_OBJECT_0))
      {
         sysLog.Print ("All applications stopped\n");
         CancelWaitableTimer(hWaitTimer);
         break;
      }

      if(ret == WAIT_TIMEOUT)
      {
         sysLog.Print ("----- TIMEOUT -----\n");
         printf("----- TIMEOUT -----\n");

         if(!applicationsManager->Empty())
         {
            applicationsManager->PrintRunningApplications();
         }
      }
   }

   UpdateStatusProcessing(&dwCheckPoint);

   // Tell Monitor thread to terminate
   commHub.SendMessage(TFServerCommunicationPacket::TF_TCPCLIENT_MANAGER, new TFTCPClientCommunicationPacket(NULL, TFTCPClientCommunicationPacket::TCPCLIENT_EVT_SHUTDOWN));

   // If all applications shutdown within the allotted time, clean up properly.
   // If one application is still running, server may hang if an attempt is made
   // to clean up properly.
   if(ret == WAIT_OBJECT_0 + 1)
   {
      sysLog.Print ("Closing handles...\n");
      CloseHandle (hWaitTimer);
      //
      // Moved from immediately after call to _beginthreadex to start the Monitor thread - TF 2008-06-16
      CloseHandle (hMonThread);
      UpdateStatusProcessing (&dwCheckPoint);

      delete applicationsManager;

      sysLog.Print ("Stopping telephony hardware...\n");
      if(CloseHardwareDriver != NULL)
      {
         // Set stop pending for an additional 20 seconds (max. Windows allows)
         // to give time for hardware driver to shutdown (recommended by Ai-Logix)
         UpdateStatusStart (SERVICE_STOP_PENDING, &dwCheckPoint, 20000);
         if(CloseHardwareDriver(&sysLog) != true)
         {
            sysLog.Print("Warning: Could not stop telephony hardware driver!\n");
         }
         UpdateStatusProcessing(&dwCheckPoint);
      }
      UnloadBoardFunctions();

      delete tcpClientManager;
   }

   Socket::NetShutdown();
   ACE::fini();

   sysLog.Print ("**** Service terminated. ****\n");
   printf ("**** Service terminated. ****\n");

   UpdateStatusDone (SERVICE_STOPPED, NO_ERROR, 0);

   return 0;
}
//EngineServerMain

/* -------------------------------------------------------------------------
   Name:        MyServiceCtrlHandler

   Purpose:     Callback function.
   ------------------------------------------------------------------------- */
VOID _stdcall MyServiceCtrlHandler (DWORD Opcode)
{
   DWORD status;
   switch (Opcode)
   {
   case SERVICE_CONTROL_PAUSE:
      // Do whatever it takes to pause here.
      MyServiceStatus.dwCurrentState = SERVICE_PAUSED;
      break;

   case SERVICE_CONTROL_CONTINUE:
      // Do whatever it takes to continue here.
      MyServiceStatus.dwCurrentState = SERVICE_RUNNING;
      break;

   case SERVICE_CONTROL_STOP:
      LogEvent(EVENTLOG_WARNING_TYPE, TFSERVER_STOPPING, NULL);
      SetEvent (hEventSD);
      // EngineServerMain() will receive the hEventSD signal and handle/manage the shutdown
      return;

   case SERVICE_CONTROL_SHUTDOWN:
      // Service needs to shutdown quickly.
      SetEvent (hEventSD);
      return;

   case SERVICE_CONTROL_INTERROGATE:
      // Fall through to send current status.
      break;

   default:
      SvcDebugOut (" [MY_SERVICE] Unrecognized opcode %ld\n", Opcode);
   }

   // Send current status.
   if (!SetServiceStatus (MyServiceStatusHandle,  &MyServiceStatus)) {
      status = GetLastError();
      SvcDebugOut (" [MY_SERVICE] SetServiceStatus error %ld\n",status);
   }
   return;
}

/* -------------------------------------------------------------------------
   Name:        CreateEngineService

   Purpose:     Registers the service with Windows NT Service Control
                Manager (SCM).

                JSH 2003-06-18: should wrap this in a try-exception
   ------------------------------------------------------------------------- */
VOID CreateEngineService (const char *pszLocation)
{
   LPCTSTR    lpszBinaryPathName = pszLocation;
   LPCTSTR    lpszDisplayName    = SERVICE_DISPLAY_NM;
   SC_HANDLE  schSCManager;
   SC_HANDLE  schService;

   // registry stuff for creating board-service dependencies
   REG_SERVICE_INFO* serviceInfo = new REG_SERVICE_INFO; // JSH: the struct for the service info from the registry
   LONG  ret;
   HKEY  hkey;
   DWORD dwLen;
   DWORD dwType;


    // for event logging....
    HKEY hk2;
    DWORD dwData;
    // UCHAR szBuf[80];
    char szBuf[80];

   ret = RegOpenKeyEx (HKEY_LOCAL_MACHINE, TFSERVER_REG_KEY, 0, KEY_READ, &hkey);
   if (ret != ERROR_SUCCESS)
   {
      // delete[] serviceInfo->pszBoardService;
      // memset (&serviceInfo, 0, sizeof(REG_SERVICE_INFO));
      printf("Cannot open reg key for TFServer\n");
      RegCloseKey (hkey);
   }


   ret = RegQueryValueEx (hkey, "BOARDSERVICE", (LPDWORD)0, &dwType, NULL, &dwLen);
   if (ret != ERROR_SUCCESS)
   {
      // delete[] serviceInfo->pszBoardService;
      memset (&serviceInfo, 0, sizeof(REG_SERVICE_INFO));
      printf("Cannot get reg key info for TFServer\n");
      RegCloseKey (hkey);
   }


   serviceInfo->pszBoardService = new char[dwLen+1];

   ret = RegQueryValueEx(hkey, "BOARDSERVICE", 0, &dwType, (unsigned char*) serviceInfo->pszBoardService, &dwLen);

    if (ret != ERROR_SUCCESS)
   {
      delete[] serviceInfo->pszBoardService;
      memset (&serviceInfo, 0, sizeof(REG_SERVICE_INFO));
      // LogEvent (EVENTLOG_ERROR_TYPE, "Cannot get board service info for TFServer\n");
      printf("Cannot get board service info for TFServer\n");
      RegCloseKey (hkey);
   }


   // **** Install Directory ******************************************************* JSH 2003-07-21
   ret = RegQueryValueEx (hkey, "InstallDirectory", 0, &dwType, NULL, &dwLen);
   if (ret != ERROR_SUCCESS)
   {
      printf("Cannot get install directory info for TFServer\n");
      RegCloseKey (hkey);
   }

   serviceInfo->pszInstallDir = new char [dwLen + 1];
   ret = RegQueryValueEx(hkey, "InstallDirectory", 0, &dwType, (unsigned char*) serviceInfo->pszInstallDir, &dwLen);

   if (ret != ERROR_SUCCESS)
   {
      printf("Cannot get install directory info for TFServer\n");
      delete[] serviceInfo->pszInstallDir;
   }

   RegCloseKey (hkey);


    // Add your source name as a subkey under the Application
    // key in the EventLog registry key.

    ret = RegCreateKey(HKEY_LOCAL_MACHINE, TFSERVER_APP_LOG_REG_KEY, &hk2);
    if (ret != ERROR_SUCCESS)
    {
        printf("Could not create the registry key.\n");
    }

    //ErrorExit("Could not create the registry key.");

    // Set the name of the message file.

    strcpy(szBuf, serviceInfo->pszInstallDir);
    strcat(szBuf, "tf_msgs.dll");

    // Add the name to the EventMessageFile subkey.

    ret = RegSetValueEx(hk2,             // subkey handle
            "EventMessageFile",       // value name
            0,                        // must be zero
            REG_EXPAND_SZ,            // value type
            (LPBYTE) szBuf,           // pointer to value data
            strlen(szBuf) + 1);       // length of value data
        //ErrorExit("Could not set the event message file.");
    if (ret != ERROR_SUCCESS)
    {
        printf("Could not set the event message file.\n");
    }


    // Set the supported event types in the TypesSupported subkey.

    
    // set the Type of events available
    dwData = EVENTLOG_ERROR_TYPE | EVENTLOG_WARNING_TYPE | EVENTLOG_INFORMATION_TYPE;

    ret = RegSetValueEx(hk2,      // subkey handle
            "TypesSupported",  // value name 
            0,                 // must be zero
            REG_DWORD,         // value type
            (LPBYTE) &dwData,  // pointer to value data 
            sizeof(DWORD));    // length of value data

    if (ret != ERROR_SUCCESS)
    {
       printf("Could not set the supported types.\n");
    }

        //ErrorExit("Could not set the supported types.");
 
    RegCloseKey(hk2);
    // end of for event logging

   
   printf("Installing the service....\n");

   schSCManager = OpenSCManager (
      NULL,                       // Target machine name
      NULL,                       // SCM DB to open (default ServicesActive)
      SC_MANAGER_CREATE_SERVICE); // Desired access

   if (schSCManager == NULL)
   {
      printf("Cannot open the SCM\n");
      // MyErrorExit ("OpenSCManager");
   }

   schService = CreateService (
      schSCManager,               // SCManager database
      SERVICE_NM,                 // name of service
      lpszDisplayName,            // service name to display
      SERVICE_ALL_ACCESS,         // desired access
      SERVICE_WIN32_OWN_PROCESS,  // service type
      SERVICE_DEMAND_START,       // start type
      SERVICE_ERROR_NORMAL,       // error control type
      lpszBinaryPathName,         // service's binary
      NULL,                       // no load ordering group
      NULL,                       // no tag identifier
      (LPCTSTR) serviceInfo->pszBoardService, // Dependency... 2003-05-23 JSH: removed "Agmon\0", as NMS might not be the card....
      NULL,                       // LocalSystem account
      NULL);                      // no password

   delete [] serviceInfo->pszBoardService;
   memset (&serviceInfo, 0, sizeof (REG_SERVICE_INFO));

   if (schService == NULL)
   {
      // MyErrorExit ("CreateService");
      printf("Cannot install TFServer as a service\n");
   }
   else
   {
      printf ("CreateService SUCCESS.\n");
   }

   CloseServiceHandle (schService);
   CloseServiceHandle (schSCManager);
}
/*CreateEngineService*/

/* -------------------------------------------------------------------------
   Name:        DeleteEngineService
   ------------------------------------------------------------------------- */
VOID DeleteEngineService()
{
   SC_HANDLE  schSCManager;
   SC_HANDLE  schService;

   // ~CLimitSingleInstance(); // JSH MARK: just in case it is hanging around, clear the single-app marker

   schSCManager = OpenSCManager (
      NULL,                       // Target machine name
      NULL,                       // SCM DB to open (default ServicesActive)
      GENERIC_WRITE);             // Desired access

   if (schSCManager == NULL)
      MyErrorExit ("OpenSCManager");

   schService = OpenService (
      schSCManager,        // SCManager database
      SERVICE_NM,          // name of service
      DELETE);             // only need DELETE access

   if (schService == NULL)
      MyErrorExit ("OpenService");

   if (! DeleteService (schService) )
      MyErrorExit ("DeleteService");
   else
      printf ("DeleteService SUCCESS\n");

   CloseServiceHandle (schService);
   CloseServiceHandle (schSCManager);
}
/*DeleteEngineService*/

/* -------------------------------------------------------------------------
   Name:        ReconfigureEngineService

   Purpose:
   ------------------------------------------------------------------------- */
VOID ReconfigureEngineService (BOOL fDisable)
{
   SC_HANDLE                   schSCManager;
   SC_HANDLE                   schService;
   SC_LOCK                     sclLock;
   LPQUERY_SERVICE_LOCK_STATUS lpqslsBuf;
   DWORD                       dwBytesNeeded;
   DWORD                       dwStartType;
   BOOL                        bRet;

   schSCManager = OpenSCManager (
      NULL,                       // Target machine name
      NULL,                       // SCM DB to open (default ServicesActive)
      SC_MANAGER_LOCK |           // Desired access
      GENERIC_WRITE);

   if (schSCManager == NULL)
      MyErrorExit ("OpenSCManager");

   // Need to acquire database lock before reconfiguring.
   sclLock = LockServiceDatabase (schSCManager);

   // If the database cannot be locked, report the details.
   if (sclLock == NULL) {
      // Exit if the database is not locked by another process.
      if (GetLastError() != ERROR_SERVICE_DATABASE_LOCKED)
         MyErrorExit ("LockServiceDatabase");

      // Allocate a buffer to get details about the lock.
      lpqslsBuf = (LPQUERY_SERVICE_LOCK_STATUS) LocalAlloc (
         LPTR,
         sizeof (QUERY_SERVICE_LOCK_STATUS) + 256);

      if (lpqslsBuf == NULL)
         MyErrorExit ("LocalAlloc");

      // Get and print the lock status information.
      bRet = QueryServiceLockStatus (
                schSCManager,
                lpqslsBuf,
                sizeof (QUERY_SERVICE_LOCK_STATUS) + 256,
                &dwBytesNeeded);
      if (!bRet)
         MyErrorExit ("QueryServiceLockStatus");
      if (lpqslsBuf->fIsLocked)
         printf ("Locked by: %s, duration: %d seconds\n",
                 lpqslsBuf->lpLockOwner,
                 lpqslsBuf->dwLockDuration);
      else
         printf ("No longer locked\n");

      LocalFree (lpqslsBuf);
      MyErrorExit ("Could not lock database");
   }

   // The database is locked, so it is safe to make changes.
   // Open a handle to the service.
   schService = OpenService (
      schSCManager,           // SCManager database
      SERVICE_NM,             // name of service
      SERVICE_CHANGE_CONFIG); // need CHANGE access
   if (schService == NULL)
      MyErrorExit ("OpenService");

   dwStartType = (fDisable) ? SERVICE_DISABLED : SERVICE_DEMAND_START;

   bRet = ChangeServiceConfig (
             schService,        // handle of service
             SERVICE_NO_CHANGE, // service type: no change
             dwStartType,       // change service start type
             SERVICE_NO_CHANGE, // error control: no change
             NULL,              // binary path: no change
             NULL,              // load order group: no change
             NULL,              // tag ID: no change
             NULL,              // dependencies: no change
             NULL,              // account name: no change
             NULL,              // password: no change
             NULL);             // Display name: no change
   if (!bRet) {
        MyErrorExit ("ChangeServiceConfig");
   } else
      printf ("ChangeServiceConfig SUCCESS\n");

   // Release the database lock.
   UnlockServiceDatabase (sclLock);

   // Close the handle to the service.
   CloseServiceHandle (schService);
   CloseServiceHandle (schSCManager);
}
/*ReconfigureEngineService*/

/* -------------------------------------------------------------------------
   Name:        GetEngineServiceConfig
   ------------------------------------------------------------------------- */
VOID GetEngineServiceConfig()
{
   SC_HANDLE              schSCManager;
   SC_HANDLE              schService;
   LPQUERY_SERVICE_CONFIG lpqscBuf;
   DWORD                  dwBytesNeeded;

   schSCManager = OpenSCManager (
      NULL,                          // Target machine name
      NULL,                          // SCM DB to open (default ServicesActive)
      SC_MANAGER_QUERY_LOCK_STATUS); // Desired access

   if (schSCManager == NULL)
      MyErrorExit ("OpenSCManager");

   // Open a handle to the service.
   schService = OpenService (
      schSCManager,           // SCManager database
      SERVICE_NM,             // name of service
      SERVICE_QUERY_CONFIG);  // need QUERY access
   if (schService == NULL) {
      CloseServiceHandle (schSCManager);
      MyErrorExit ("OpenService");
   }

   // Allocate a buffer for the information configuration.
   lpqscBuf = (LPQUERY_SERVICE_CONFIG) LocalAlloc (LPTR, 4096);

   if (lpqscBuf == NULL)
      MyErrorExit ("LocalAlloc");
   // Get and print the information configuration.
   if (! QueryServiceConfig (schService, lpqscBuf, 4096, &dwBytesNeeded) ) {
      MyErrorExit ("QueryServiceConfig");
   }
   printf ("\nSample_Srv configuration: \n");
   printf ("  Type: 0x%x\n", lpqscBuf->dwServiceType);
   printf ("  Start Type: 0x%x\n", lpqscBuf->dwStartType);
   printf ("  Err Control: 0x%x\n", lpqscBuf->dwErrorControl);
   printf ("  Binary path: %s\n", lpqscBuf->lpBinaryPathName);

   if (lpqscBuf->lpLoadOrderGroup != NULL)
      printf ("  Load order group: %s\n", lpqscBuf->lpLoadOrderGroup);
   if (lpqscBuf->dwTagId != 0)
      printf ("  Tag ID: %d\n", lpqscBuf->dwTagId);
   if (lpqscBuf->lpDependencies != NULL)
      printf ("  Dependencies: %s\n", lpqscBuf->lpDependencies);
   if (lpqscBuf->lpServiceStartName != NULL)
      printf("  Start Name: %s\n", lpqscBuf->lpServiceStartName);

   LocalFree (lpqscBuf);

   // Close the handle to the service.
   CloseServiceHandle (schService);
   CloseServiceHandle (schSCManager);
}
/*GetEngineServiceConfig*/

/* -------------------------------------------------------------------------
   Name:        ControlEngineService
   ------------------------------------------------------------------------- */
VOID ControlEngineService (DWORD fdwControl)
{
   SC_HANDLE      schSCManager;
   SC_HANDLE      schService;
   SERVICE_STATUS ssStatus;
   DWORD          fdwAccess;
// DWORD          dwStartTickCount;
// DWORD          dwWaitTime;
   bool           bRet;

   schSCManager = OpenSCManager (
      NULL,                          // Target machine name
      NULL,                          // SCM DB to open (default ServicesActive)
      SC_MANAGER_QUERY_LOCK_STATUS); // Desired access

   if (schSCManager == NULL)
      MyErrorExit ("OpenSCManager");

   // The required service object access depends on the control.
   switch (fdwControl)
   {
   case SERVICE_CONTROL_STOP:
      fdwAccess = SERVICE_STOP;
      break;

   case SERVICE_CONTROL_PAUSE:
   case SERVICE_CONTROL_CONTINUE:
      fdwAccess = SERVICE_PAUSE_CONTINUE;
      break;

   case SERVICE_CONTROL_INTERROGATE:
      fdwAccess = SERVICE_INTERROGATE;
      break;

   default:
      fdwAccess = SERVICE_INTERROGATE;
   }

   // Open a handle to the service.
   schService = OpenService (
      schSCManager,         // SCManager database
      SERVICE_NM,           // name of service
      fdwAccess);           // specify access
   if (schService == NULL)
      MyErrorExit ("OpenService");

   // Send a control value to the service.
   bRet = ControlService (
             schService,   // handle of service
             fdwControl,   // control value to send
             &ssStatus);   // address of status info
   if (!bRet) {
      MyErrorExit ("ControlService");
   }

   // Print the service status.
   printf ("\nStatus of Sample_Srv: \n");
   printf ("  Service Type: 0x%x\n", ssStatus.dwServiceType);
   printf ("  Current State: 0x%x\n", ssStatus.dwCurrentState);
   printf ("  Controls Accepted: 0x%x\n", ssStatus.dwControlsAccepted);
   printf ("  Exit Code: %d\n", ssStatus.dwWin32ExitCode);
   printf ("  Service Specific Exit Code: %d\n",
           ssStatus.dwServiceSpecificExitCode);
   printf ("  Check Point: %d\n", ssStatus.dwCheckPoint);
   printf ("  Wait Hint: %d\n", ssStatus.dwWaitHint);

   // Close the handle to the service.
   CloseServiceHandle (schService);
   CloseServiceHandle (schSCManager);

   return;
}
/*ControlEngineService*/


/* -------------------------------------------------------------------------
   Name:        LogEvent
                - used to print out to the application log
   ------------------------------------------------------------------------- */
void LogEvent (WORD errtype, DWORD errcode, const char* szMsg)
{
  HANDLE   hEventSource;
  hEventSource = RegisterEventSource(NULL, SERVICE_NM);

  if (szMsg!=NULL)
  {
    ReportEvent(hEventSource,errtype,0,errcode, NULL,1,0,&szMsg,NULL);
  }
  else
  {
    ReportEvent(hEventSource,errtype,0,errcode, NULL,0,0,NULL,NULL);
  }
}
// LogEvent

// -------------------------------------------------------------------------
// Pauses TeleFlow to give time for Windows to finish its startup
// If we query boards too soon, they may not be ready
// -------------------------------------------------------------------------
char* ServerStartPause(FileLog* psysLog)
{
   DWORD dwUpTimeTick;
   time_t systime;
   struct tm *time_now;
   unsigned char* pszSysTime = new unsigned char[15];

   dwUpTimeTick = GetTickCount();

   psysLog->Print("   Windows up-time: %u ms\n", dwUpTimeTick);

   // has Windows has been running for at least 4 minutes?
   if(dwUpTimeTick > 240000)
   {
      psysLog->Print("   Windows has been running for at least 4 minutes\n");
   }
   else
   {
      // Sleep to give telephony board(s) time to boot
      unsigned uPause = 240000 - dwUpTimeTick;  // 4 mins - tick count
      psysLog->Print("   Windows has just restarted\n");
      psysLog->Print("      Timing pause %u milliseconds...\n", uPause);
      Sleep(uPause);
   }

   // "TFSTART" indicates last date/time TFServer started

   time(&systime);
   time_now = localtime(&systime);

   strftime((char*) pszSysTime, 15, "%Y%m%d%H%M%S", time_now);
   pszSysTime[14] = '\0';

   psysLog->Print("TFSTART = %s\n", pszSysTime);

   // Cannot write to LOCAL_MACHINE if TFServer service running as a User account
   // Output to TFServer log (above) should be sufficient.

   return((char*)pszSysTime);  // Return date/time TFServer officially started (TFSTART)
}
//ServerStartPause

bool SetServerPriority(FileLog* psysLog)
{
   SYSTEM_INFO sysInfo;           // Win32 API system info structure
   DWORD iCount;                  // count of CPU cores in system
   HANDLE hProc;                  // retrieve handle of current process
   DWORD_PTR dwProcessAffinityMask;   // current process affinity (retrieved and then modified)
   DWORD_PTR dwSystemAffinityMask;    // system affinity
   INT iShift;                    // bitwise shift to set affinity to only some of the cores
   bool bSuccess;                 // result from Win32 API calls
   DWORD dwErrMode;               // captures current error mode

   if(bConsoleMode)
   {
      psysLog->Print("   Running in Console Mode; do not disable GPF error box\n");
   }
   else
   {
      psysLog->Print("   Disable GPF error box; running as a Service\n");
      dwErrMode = SetErrorMode(SEM_NOGPFAULTERRORBOX);
      SetErrorMode(dwErrMode | SEM_NOGPFAULTERRORBOX);
   }

   GetSystemInfo(&sysInfo);
   iCount = sysInfo.dwNumberOfProcessors;
   psysLog->Print("   Number of CPUs: %i\n", iCount);

   if(iCount == 1)
   {
      psysLog->Print("   Using Windows default\n");
      return(true);
   }

   // Retrieve pseudohandle to the current process
   hProc = GetCurrentProcess();

   bSuccess = GetProcessAffinityMask(hProc, &dwProcessAffinityMask, &dwSystemAffinityMask);
   if(bSuccess)
   {
      // Set affinity to half the cores in the system, rounded down
      iShift = iCount / 2;
      dwProcessAffinityMask = (dwSystemAffinityMask << iShift) & dwSystemAffinityMask;
      bSuccess = SetProcessAffinityMask(hProc, dwProcessAffinityMask);
      //
      // If necessary in the future, we can use the following function
      // to set the affinity of the individual threads
      //    SetThreadAffinityMask()
      if(bSuccess)
      {
         psysLog->Print("   Set processor affinity to upper %i %s\n", iShift, (iShift == 1)?"core":"cores");
      }
   }

   if(!bSuccess)
   {
      psysLog->Print("   WARNING: Failed to set process affinity\n");
      return(false);
   }

   // NOTES: how to get the current Priority Class
   //    DWORD dwCurrentPriority;
   //    dwCurrentPriority = GetPriorityClass(hProc);
   // Return value is the priority class of the specified process, or
   // Zero on error; to get extended error information, call GetLastError()
   // Process Priority Class is one of the following values:
   //    NORMAL_PRIORITY_CLASS
   //    IDLE_PRIORITY_CLASS
   //    HIGH_PRIORITY_CLASS
   //    REALTIME_PRIORITY_CLASS

   if(SetPriorityClass(hProc, REALTIME_PRIORITY_CLASS))
   {
      psysLog->Print("   Set process priority class to real-time\n");
      return(true);
   }
   else
   {
      psysLog->Print("   WARNING: Failed to set process priority\n");
      return(false);
   }
}
//SetServerPriority
