#ifndef __INTERNETMESSAGEUDPTRANSACTION_H__
#define __INTERNETMESSAGEUDPTRANSACTION_H__

#ifdef __DLL__
#define INTERNETMESSAGEUDPTRANSACTIONAPI __declspec(dllexport)
#else
#define INTERNETMESSAGEUDPTRANSACTIONAPI __declspec(dllimport)
#endif

#include "Socket.h"
#include "InternetMessage.h"

// Base class for request/response transaction across a TCP socket
class INTERNETMESSAGEUDPTRANSACTIONAPI InternetMessageUDPTransaction : public UDPSocket
{
   public:

      virtual int Send(const ACE_Addr& peer);
      virtual int Receive(ACE_Addr& peer);

      // Returns pointer to dynamic request object
      // All request object methods should be accessed via this method
      virtual InternetMessage* Request(void);

      // Returns pointer to dynamic response object
      // All response object methods should be accessed via this method
      virtual InternetMessage* Response(void);

   protected:

      InternetMessage* request;
      InternetMessage* response;

      // Creates a new O/S socket on the port specified, or an available port by
      // default, and creates an event handle
      // Sets the event handle to signal READ or CLOSE events for the O/S socket
      // persistant should be true if the O/S socket should not be closed by the
      // destructor
      // response and request are set to NULL
      // response and request objects must be created and destroyed by the child class
      InternetMessageUDPTransaction(unsigned short port = Socket::ANY_PORT, bool persistant = false);

      // Creates an event handle
      // Sets the event handle to signal READ or CLOSE events for the socket specified
      // by handle unless handle is ACE_INVALID_HANDLE
      // The O/S socket is not closed in the destructor
      // response and request are set to NULL
      // response and request objects must be created and destroyed by the child class
      InternetMessageUDPTransaction(ACE_HANDLE handle);

      // Has no effect
      virtual ~InternetMessageUDPTransaction(void);
};

#endif
